package db2

import androidx.room.Dao
import androidx.room.Insert
import androidx.room.Query
import androidx.room.Update
import com.ustadmobile.door.SyncNode
import kotlin.Any
import kotlin.Boolean
import kotlin.Int
import kotlin.Long
import kotlin.String
import kotlin.collections.List

@Dao
public actual abstract class ExampleSyncableDao {
  @Insert(
    entity = Any::class,
    onConflict = 3,
  )
  public actual abstract fun insert(syncableEntity: ExampleSyncableEntity): Long

  @Insert(
    entity = Any::class,
    onConflict = 3,
  )
  public actual abstract fun insertList(syncableEntityLIst: List<ExampleSyncableEntity>)

  @Insert(
    entity = Any::class,
    onConflict = 3,
  )
  public actual abstract suspend fun insertAndReturnList(entities: List<ExampleSyncableEntity>):
      List<Long>

  @Insert(
    entity = Any::class,
    onConflict = 3,
  )
  public actual abstract suspend fun insertAsync(syncableEntity: ExampleSyncableEntity): Long

  @Query(`value` = "SELECT * FROM ExampleSyncableEntity")
  public actual abstract fun findAll(): List<ExampleSyncableEntity>

  @Query(`value` = "SELECT * FROM ExampleSyncableEntity WHERE esNumber IN (:numberList)")
  public actual abstract suspend fun findByListParam(numberList: List<Long>):
      List<ExampleSyncableEntity>

  @Query(`value` = "SELECT * FROM ExampleSyncableEntity WHERE esUid = :uid")
  public actual abstract fun findByUid(uid: Long): ExampleSyncableEntity?

  @Query(`value` = "SELECT * FROM ExampleSyncableEntity WHERE esName = :name")
  public actual abstract fun findByName(name: String?): ExampleSyncableEntity?

  @Query(`value` =
      "SELECT ExampleSyncableEntity.*, OtherSyncableEntity.* FROM ExampleSyncableEntity LEFT JOIN OtherSyncableEntity ON ExampleSyncableEntity.esUid = OtherSyncableEntity.otherFk")
  public actual abstract fun findAllWithOtherByUid():
      List<ExampleSyncableEntityWithOtherSyncableEntity>

  @Query(`value` =
      "UPDATE ExampleSyncableEntity SET esNumber = :newNumber,esLcb = (SELECT nodeClientId FROM SyncNode LIMIT 1) WHERE esUid = :uid")
  public actual abstract fun updateNumberByUid(uid: Long, newNumber: Long)

  @Update(
    entity = Any::class,
    onConflict = 3,
  )
  public actual abstract suspend fun updateAsync(exampleSyncableEntity: ExampleSyncableEntity)

  @Query(`value` = "SELECT * FROM ExampleSyncableEntity LIMIT 1")
  public actual abstract suspend fun findOneFromWeb(): ExampleSyncableEntity?

  @Query(`value` = "SELECT esNumber FROM ExampleSyncableEntity LIMIT 1")
  public actual abstract suspend fun findOneValue(): Int

  @Query(`value` = "SELECT SyncNode.* FROM SyncNode LIMIT 1")
  public actual abstract fun getSyncNode(): SyncNode?

  @Query(`value` =
      "\n   INSERT INTO ExampleSyncableEntity (esUid, esLcsn, esMcsn, esLcb, esNumber, esName, publik)\n        SELECT (esUid * 5) AS esUid, 0 AS esLcsn, 0 AS esMcsn, 0 AS esLcb, 42 AS esNumber, :name AS esName, \n               :publik AS publik \n          FROM ExampleSyncableEntity\n         WHERE esName = :name\n    ")
  public actual abstract fun insertFromSelectQuery(name: String?, publik: Boolean)
}
