package db3

import com.ustadmobile.door.DoorConstants
import com.ustadmobile.door.ext.doorWrapper
import com.ustadmobile.door.ext.doorWrapperNodeId
import com.ustadmobile.door.ext.requirePagingSourceLoadParams
import com.ustadmobile.door.ext.toJsonResponse
import com.ustadmobile.door.http.DoorHttpServerConfig
import com.ustadmobile.door.http.DoorJsonRequest
import com.ustadmobile.door.http.DoorJsonResponse
import com.ustadmobile.door.log.DoorLogLevel
import com.ustadmobile.door.message.DoorMessage
import com.ustadmobile.door.paging.loadPageDataForHttp
import com.ustadmobile.door.replication.DoorReplicationEntity
import kotlin.Int
import kotlin.Long
import kotlin.String
import kotlin.Suppress
import kotlin.collections.List
import kotlin.collections.buildList
import kotlinx.coroutines.flow.first
import kotlinx.serialization.builtins.ListSerializer
import kotlinx.serialization.builtins.nullable
import kotlinx.serialization.builtins.serializer
import kotlinx.serialization.json.jsonObject

@Suppress("LocalVariableName", "RedundantSuppression", "FunctionName",
    "RedundantVisibilityModifier")
public suspend
    fun DiscussionPostDao.findByUidAsyncOverHttp_DoorHttp(serverConfig: DoorHttpServerConfig,
    request: DoorJsonRequest): DoorJsonResponse {
  val json = serverConfig.json
  val _arg_postUid : Long = request.queryParam("postUid")?.let {
    json.decodeFromString(Long.serializer(), it)
  } ?: 0L
  val _thisNodeId = request.db.doorWrapperNodeId
  val _result = findByUidAsyncOverHttp(_arg_postUid,)
  return DoorJsonResponse(
    bodyText = json.encodeToString(DiscussionPost.serializer().nullable, _result),
    headers = listOf(DoorConstants.HEADER_NODE_ID to _thisNodeId.toString()),
  )
}

@Suppress("LocalVariableName", "RedundantSuppression", "FunctionName",
    "RedundantVisibilityModifier")
public suspend
    fun DiscussionPostDao.findAllRepliesWithPosterMember_DoorHttp(serverConfig: DoorHttpServerConfig,
    request: DoorJsonRequest): DoorJsonResponse {
  val json = serverConfig.json
  val _arg_postUid : Long = request.queryParam("postUid")?.let {
    json.decodeFromString(Long.serializer(), it)
  } ?: 0L
  val _result_findAllRepliesWithPosterMember = findAllRepliesWithPosterMember(
    postUid = _arg_postUid,
  )
  val replicationEntities = buildList<DoorReplicationEntity> {
    addAll(
      _result_findAllRepliesWithPosterMember.mapNotNull {
        _row ->
        _row.discussionPost?.let {
          DoorReplicationEntity(tableId = 543,
            orUid = 0,
            entity = json.encodeToJsonElement(DiscussionPost.serializer(), it).jsonObject,
          )
        }

      }
    )
    addAll(
      _result_findAllRepliesWithPosterMember.mapNotNull {
        _row ->
        _row.posterMember?.let {
          DoorReplicationEntity(tableId = 544,
            orUid = 0,
            entity = json.encodeToJsonElement(Member.serializer(), it).jsonObject,
          )
        }

      }
    )
  }
  val _thisNodeId = request.db.doorWrapperNodeId
  return DoorJsonResponse(
    headers = buildList {
      add(Pair(DoorConstants.HEADER_NODE_ID, _thisNodeId.toString()))
    }
    ,
    bodyText = json.encodeToString(
      DoorMessage.serializer(),
      DoorMessage(
        what = DoorMessage.WHAT_REPLICATION_PULL,
        fromNode = _thisNodeId,
        toNode = request.requireNodeId(),
        replications = replicationEntities,
      )
    )
  )
}

@Suppress("LocalVariableName", "RedundantSuppression", "FunctionName",
    "RedundantVisibilityModifier")
public suspend
    fun DiscussionPostDao.findAllRepliesWithPosterMemberAsFlow_DoorHttp(serverConfig: DoorHttpServerConfig,
    request: DoorJsonRequest): DoorJsonResponse {
  val json = serverConfig.json
  val _arg_postUid : Long = request.queryParam("postUid")?.let {
    json.decodeFromString(Long.serializer(), it)
  } ?: 0L
  val _result_findAllRepliesWithPosterMemberAsFlow = findAllRepliesWithPosterMemberAsFlow(
    postUid = _arg_postUid,
  ).first()
  val replicationEntities = buildList<DoorReplicationEntity> {
    addAll(
      _result_findAllRepliesWithPosterMemberAsFlow.mapNotNull {
        _row ->
        _row.discussionPost?.let {
          DoorReplicationEntity(tableId = 543,
            orUid = 0,
            entity = json.encodeToJsonElement(DiscussionPost.serializer(), it).jsonObject,
          )
        }

      }
    )
    addAll(
      _result_findAllRepliesWithPosterMemberAsFlow.mapNotNull {
        _row ->
        _row.posterMember?.let {
          DoorReplicationEntity(tableId = 544,
            orUid = 0,
            entity = json.encodeToJsonElement(Member.serializer(), it).jsonObject,
          )
        }

      }
    )
  }
  val _thisNodeId = request.db.doorWrapperNodeId
  return DoorJsonResponse(
    headers = buildList {
      add(Pair(DoorConstants.HEADER_NODE_ID, _thisNodeId.toString()))
    }
    ,
    bodyText = json.encodeToString(
      DoorMessage.serializer(),
      DoorMessage(
        what = DoorMessage.WHAT_REPLICATION_PULL,
        fromNode = _thisNodeId,
        toNode = request.requireNodeId(),
        replications = replicationEntities,
      )
    )
  )
}

@Suppress("LocalVariableName", "RedundantSuppression", "FunctionName",
    "RedundantVisibilityModifier")
public suspend
    fun DiscussionPostDao.findByUidWithPosterMember_DoorHttp(serverConfig: DoorHttpServerConfig,
    request: DoorJsonRequest): DoorJsonResponse {
  val json = serverConfig.json
  val _arg_postUid : Long = request.queryParam("postUid")?.let {
    json.decodeFromString(Long.serializer(), it)
  } ?: 0L
  val _result_findByUidWithPosterMember = findByUidWithPosterMember(
    postUid = _arg_postUid,
  )
  val replicationEntities = buildList<DoorReplicationEntity> {
    _result_findByUidWithPosterMember?.discussionPost?.also {
      _row -> 
      add(
        DoorReplicationEntity(tableId = 543,
          orUid = 0,
          entity = json.encodeToJsonElement(DiscussionPost.serializer(), _row).jsonObject,
        )
      )
    }
    _result_findByUidWithPosterMember?.posterMember?.also {
      _row -> 
      add(
        DoorReplicationEntity(tableId = 544,
          orUid = 0,
          entity = json.encodeToJsonElement(Member.serializer(), _row).jsonObject,
        )
      )
    }
  }
  val _thisNodeId = request.db.doorWrapperNodeId
  return DoorJsonResponse(
    headers = buildList {
      add(Pair(DoorConstants.HEADER_NODE_ID, _thisNodeId.toString()))
    }
    ,
    bodyText = json.encodeToString(
      DoorMessage.serializer(),
      DoorMessage(
        what = DoorMessage.WHAT_REPLICATION_PULL,
        fromNode = _thisNodeId,
        toNode = request.requireNodeId(),
        replications = replicationEntities,
      )
    )
  )
}

@Suppress("LocalVariableName", "RedundantSuppression", "FunctionName",
    "RedundantVisibilityModifier")
public suspend
    fun DiscussionPostDao.findByUidWithPosterMemberAsFlow_DoorHttp(serverConfig: DoorHttpServerConfig,
    request: DoorJsonRequest): DoorJsonResponse {
  val json = serverConfig.json
  val _arg_postUid : Long = request.queryParam("postUid")?.let {
    json.decodeFromString(Long.serializer(), it)
  } ?: 0L
  val _result_findByUidWithPosterMemberAsFlow = findByUidWithPosterMemberAsFlow(
    postUid = _arg_postUid,
  ).first()
  val replicationEntities = buildList<DoorReplicationEntity> {
    _result_findByUidWithPosterMemberAsFlow?.discussionPost?.also {
      _row -> 
      add(
        DoorReplicationEntity(tableId = 543,
          orUid = 0,
          entity = json.encodeToJsonElement(DiscussionPost.serializer(), _row).jsonObject,
        )
      )
    }
    _result_findByUidWithPosterMemberAsFlow?.posterMember?.also {
      _row -> 
      add(
        DoorReplicationEntity(tableId = 544,
          orUid = 0,
          entity = json.encodeToJsonElement(Member.serializer(), _row).jsonObject,
        )
      )
    }
  }
  val _thisNodeId = request.db.doorWrapperNodeId
  return DoorJsonResponse(
    headers = buildList {
      add(Pair(DoorConstants.HEADER_NODE_ID, _thisNodeId.toString()))
    }
    ,
    bodyText = json.encodeToString(
      DoorMessage.serializer(),
      DoorMessage(
        what = DoorMessage.WHAT_REPLICATION_PULL,
        fromNode = _thisNodeId,
        toNode = request.requireNodeId(),
        replications = replicationEntities,
      )
    )
  )
}

@Suppress("LocalVariableName", "RedundantSuppression", "FunctionName",
    "RedundantVisibilityModifier")
public suspend fun DiscussionPostDao.findByUidList_DoorHttp(serverConfig: DoorHttpServerConfig,
    request: DoorJsonRequest): DoorJsonResponse {
  val json = serverConfig.json
  val _arg_postUids : List<Long> = request.bodyAsStringOrNull()?.let {
    json.decodeFromString(ListSerializer(Long.serializer()), it)
  } ?: mutableListOf()
  val _result_findByUidList = findByUidList(
    postUids = _arg_postUids,
  )
  val replicationEntities = buildList<DoorReplicationEntity> {
    addAll(
      _result_findByUidList.mapNotNull {
        _row ->
        _row.discussionPost?.let {
          DoorReplicationEntity(tableId = 543,
            orUid = 0,
            entity = json.encodeToJsonElement(DiscussionPost.serializer(), it).jsonObject,
          )
        }

      }
    )
    addAll(
      _result_findByUidList.mapNotNull {
        _row ->
        _row.posterMember?.let {
          DoorReplicationEntity(tableId = 544,
            orUid = 0,
            entity = json.encodeToJsonElement(Member.serializer(), it).jsonObject,
          )
        }

      }
    )
  }
  val _thisNodeId = request.db.doorWrapperNodeId
  return DoorJsonResponse(
    headers = buildList {
      add(Pair(DoorConstants.HEADER_NODE_ID, _thisNodeId.toString()))
    }
    ,
    bodyText = json.encodeToString(
      DoorMessage.serializer(),
      DoorMessage(
        what = DoorMessage.WHAT_REPLICATION_PULL,
        fromNode = _thisNodeId,
        toNode = request.requireNodeId(),
        replications = replicationEntities,
      )
    )
  )
}

@Suppress("LocalVariableName", "RedundantSuppression", "FunctionName",
    "RedundantVisibilityModifier")
public suspend
    fun DiscussionPostDao.getNumPostsSinceTime_DoorHttp(serverConfig: DoorHttpServerConfig,
    request: DoorJsonRequest): DoorJsonResponse {
  val json = serverConfig.json
  val _arg_since : Long = request.queryParam("since")?.let {
    json.decodeFromString(Long.serializer(), it)
  } ?: 0L
  val _thisNodeId = request.db.doorWrapperNodeId
  val _result = getNumPostsSinceTime(_arg_since,)
  return DoorJsonResponse(
    bodyText = json.encodeToString(Int.serializer(), _result),
    headers = listOf(DoorConstants.HEADER_NODE_ID to _thisNodeId.toString()),
  )
}

@Suppress("LocalVariableName", "RedundantSuppression", "FunctionName",
    "RedundantVisibilityModifier")
public suspend
    fun DiscussionPostDao.getNumPostsSinceTimeHttpOnly_DoorHttp(serverConfig: DoorHttpServerConfig,
    request: DoorJsonRequest): DoorJsonResponse {
  val json = serverConfig.json
  val _arg_since : Long = request.queryParam("since")?.let {
    json.decodeFromString(Long.serializer(), it)
  } ?: 0L
  val _thisNodeId = request.db.doorWrapperNodeId
  val _result = getNumPostsSinceTimeHttpOnly(_arg_since,)
  return DoorJsonResponse(
    bodyText = json.encodeToString(Int.serializer(), _result),
    headers = listOf(DoorConstants.HEADER_NODE_ID to _thisNodeId.toString()),
  )
}

@Suppress("LocalVariableName", "RedundantSuppression", "FunctionName",
    "RedundantVisibilityModifier")
public suspend
    fun DiscussionPostDao.findAllPostAsPagingSource_DoorHttp(serverConfig: DoorHttpServerConfig,
    request: DoorJsonRequest): DoorJsonResponse {
  val json = serverConfig.json
  val _arg_since : Long = request.queryParam("since")?.let {
    json.decodeFromString(Long.serializer(), it)
  } ?: 0L
  val _pagingLoadParams = request.requirePagingSourceLoadParams(
    json = json,
    keyDeserializationStrategy = Int.serializer().nullable,
  )

  val _result_findAllPostAsPagingSource_pagingSource = findAllPostAsPagingSource(
    since = _arg_since,
  ).loadPageDataForHttp(_pagingLoadParams)
  val _result_findAllPostAsPagingSource = _result_findAllPostAsPagingSource_pagingSource.data
  serverConfig.logger.log(DoorLogLevel.VERBOSE,
      "DoorPaging: loaded from: ${_pagingLoadParams.key} endOfPaginationReached=${_result_findAllPostAsPagingSource_pagingSource.endOfPaginationReached}")
  val _endOfPaginationReached =
      _result_findAllPostAsPagingSource_pagingSource.endOfPaginationReached

  val replicationEntities = buildList<DoorReplicationEntity> {
    addAll(
      _result_findAllPostAsPagingSource.map {
        _row ->
        DoorReplicationEntity(tableId = 543,
          orUid = 0,
          entity = json.encodeToJsonElement(DiscussionPost.serializer(), _row).jsonObject,
        )
      }
    )
  }
  val _thisNodeId = request.db.doorWrapperNodeId
  return DoorJsonResponse(
    headers = buildList {
      add(Pair(DoorConstants.HEADER_NODE_ID, _thisNodeId.toString()))
      add(Pair(DoorConstants.HEADER_PAGING_END_REACHED, _endOfPaginationReached.toString()))
    }
    ,
    bodyText = json.encodeToString(
      DoorMessage.serializer(),
      DoorMessage(
        what = DoorMessage.WHAT_REPLICATION_PULL,
        fromNode = _thisNodeId,
        toNode = request.requireNodeId(),
        replications = replicationEntities,
      )
    )
  )
}

@Suppress("LocalVariableName", "RedundantSuppression", "FunctionName",
    "RedundantVisibilityModifier")
public suspend
    fun DiscussionPostDao.findAllPostAsNetworkOnlyPagingSource_DoorHttp(serverConfig: DoorHttpServerConfig,
    request: DoorJsonRequest): DoorJsonResponse {
  val json = serverConfig.json
  val _arg_since : Long = request.queryParam("since")?.let {
    json.decodeFromString(Long.serializer(), it)
  } ?: 0L
  val _pagingLoadParams = request.requirePagingSourceLoadParams(
    json = json,
    keyDeserializationStrategy = Int.serializer().nullable,
  )

  val _thisNodeId = request.db.doorWrapperNodeId
  val _result = findAllPostAsNetworkOnlyPagingSource(_arg_since,).load(_pagingLoadParams)
  return _result.toJsonResponse(json = json,
    keySerializer = Int.serializer().nullable,
    valueSerializer = ListSerializer(DiscussionPost.serializer()),
    localNodeId = _thisNodeId,
  )
}

@Suppress("LocalVariableName", "RedundantSuppression", "FunctionName",
    "RedundantVisibilityModifier")
public suspend
    fun DiscussionPostDao.findAllPostAsNetworkWithFallbackPagingSource_DoorHttp(serverConfig: DoorHttpServerConfig,
    request: DoorJsonRequest): DoorJsonResponse {
  val json = serverConfig.json
  val _arg_since : Long = request.queryParam("since")?.let {
    json.decodeFromString(Long.serializer(), it)
  } ?: 0L
  val _pagingLoadParams = request.requirePagingSourceLoadParams(
    json = json,
    keyDeserializationStrategy = Int.serializer().nullable,
  )

  val _thisNodeId = request.db.doorWrapperNodeId
  val _result = findAllPostAsNetworkWithFallbackPagingSource(_arg_since,).load(_pagingLoadParams)
  return _result.toJsonResponse(json = json,
    keySerializer = Int.serializer().nullable,
    valueSerializer = ListSerializer(DiscussionPost.serializer()),
    localNodeId = _thisNodeId,
  )
}

@Suppress("LocalVariableName", "RedundantSuppression", "FunctionName",
    "RedundantVisibilityModifier")
public suspend
    fun DiscussionPostDao.findPostAndNumReplies_DoorHttp(serverConfig: DoorHttpServerConfig,
    request: DoorJsonRequest): DoorJsonResponse {
  val json = serverConfig.json
  val _arg_postUid : Long = request.queryParam("postUid")?.let {
    json.decodeFromString(Long.serializer(), it)
  } ?: 0L
  val _arg_sinceTime : Long = request.queryParam("sinceTime")?.let {
    json.decodeFromString(Long.serializer(), it)
  } ?: 0L
  val _result_findAllRepliesForPost = findAllRepliesForPost(
    postUid = _arg_postUid,
    nodeId = request.requireNodeId(),
  )
  val _result_findPostAndNumReplies = findPostAndNumReplies(
    postUid = _arg_postUid,
    sinceTime = _arg_sinceTime,
  )
  val replicationEntities = buildList<DoorReplicationEntity> {
    addAll(
      _result_findAllRepliesForPost.map {
        _row ->
        DoorReplicationEntity(tableId = 543,
          orUid = 0,
          entity = json.encodeToJsonElement(DiscussionPost.serializer(), _row).jsonObject,
        )
      }
    )
    _result_findPostAndNumReplies?.discussionPost?.also {
      _row -> 
      add(
        DoorReplicationEntity(tableId = 543,
          orUid = 0,
          entity = json.encodeToJsonElement(DiscussionPost.serializer(), _row).jsonObject,
        )
      )
    }
  }
  val _thisNodeId = request.db.doorWrapperNodeId
  return DoorJsonResponse(
    headers = buildList {
      add(Pair(DoorConstants.HEADER_NODE_ID, _thisNodeId.toString()))
    }
    ,
    bodyText = json.encodeToString(
      DoorMessage.serializer(),
      DoorMessage(
        what = DoorMessage.WHAT_REPLICATION_PULL,
        fromNode = _thisNodeId,
        toNode = request.requireNodeId(),
        replications = replicationEntities,
      )
    )
  )
}

@Suppress("LocalVariableName", "RedundantSuppression", "FunctionName",
    "RedundantVisibilityModifier")
public suspend
    fun DiscussionPostDao.findRootPostsAndNumRepliesAsPagingSource_DoorHttp(serverConfig: DoorHttpServerConfig,
    request: DoorJsonRequest): DoorJsonResponse {
  val json = serverConfig.json
  val _pagingLoadParams = request.requirePagingSourceLoadParams(
    json = json,
    keyDeserializationStrategy = Int.serializer().nullable,
  )

  val _result_findRootPostsAndNumRepliesAsPagingSource_pagingSource =
      findRootPostsAndNumRepliesAsPagingSource(
  ).loadPageDataForHttp(_pagingLoadParams)
  val _result_findRootPostsAndNumRepliesAsPagingSource =
      _result_findRootPostsAndNumRepliesAsPagingSource_pagingSource.data
  serverConfig.logger.log(DoorLogLevel.VERBOSE,
      "DoorPaging: loaded from: ${_pagingLoadParams.key} endOfPaginationReached=${_result_findRootPostsAndNumRepliesAsPagingSource_pagingSource.endOfPaginationReached}")
  val _endOfPaginationReached =
      _result_findRootPostsAndNumRepliesAsPagingSource_pagingSource.endOfPaginationReached

  val _result_findRootRepliesAsPagingSource_pagingSource = findRootRepliesAsPagingSource(
  ).loadPageDataForHttp(_pagingLoadParams)
  val _result_findRootRepliesAsPagingSource =
      _result_findRootRepliesAsPagingSource_pagingSource.data

  val replicationEntities = buildList<DoorReplicationEntity> {
    addAll(
      _result_findRootPostsAndNumRepliesAsPagingSource.mapNotNull {
        _row ->
        _row.discussionPost?.let {
          DoorReplicationEntity(tableId = 543,
            orUid = 0,
            entity = json.encodeToJsonElement(DiscussionPost.serializer(), it).jsonObject,
          )
        }

      }
    )
    addAll(
      _result_findRootRepliesAsPagingSource.map {
        _row ->
        DoorReplicationEntity(tableId = 543,
          orUid = 0,
          entity = json.encodeToJsonElement(DiscussionPost.serializer(), _row).jsonObject,
        )
      }
    )
  }
  val _thisNodeId = request.db.doorWrapperNodeId
  return DoorJsonResponse(
    headers = buildList {
      add(Pair(DoorConstants.HEADER_NODE_ID, _thisNodeId.toString()))
      add(Pair(DoorConstants.HEADER_PAGING_END_REACHED, _endOfPaginationReached.toString()))
    }
    ,
    bodyText = json.encodeToString(
      DoorMessage.serializer(),
      DoorMessage(
        what = DoorMessage.WHAT_REPLICATION_PULL,
        fromNode = _thisNodeId,
        toNode = request.requireNodeId(),
        replications = replicationEntities,
      )
    )
  )
}

@Suppress("LocalVariableName", "RedundantSuppression", "FunctionName",
    "RedundantVisibilityModifier")
public suspend
    fun DiscussionPostDao.findRootPostAndNumRepliesAsPagingSourceWithPagedParams_DoorHttp(serverConfig: DoorHttpServerConfig,
    request: DoorJsonRequest): DoorJsonResponse {
  val json = serverConfig.json
  val _pagingLoadParams = request.requirePagingSourceLoadParams(
    json = json,
    keyDeserializationStrategy = Int.serializer().nullable,
  )

  val _result_findRootPostAndNumRepliesAsPagingSourceWithPagedParams_pagingSource =
      findRootPostAndNumRepliesAsPagingSourceWithPagedParams(
  ).loadPageDataForHttp(_pagingLoadParams)
  val _result_findRootPostAndNumRepliesAsPagingSourceWithPagedParams =
      _result_findRootPostAndNumRepliesAsPagingSourceWithPagedParams_pagingSource.data
  serverConfig.logger.log(DoorLogLevel.VERBOSE,
      "DoorPaging: loaded from: ${_pagingLoadParams.key} endOfPaginationReached=${_result_findRootPostAndNumRepliesAsPagingSourceWithPagedParams_pagingSource.endOfPaginationReached}")
  val _endOfPaginationReached =
      _result_findRootPostAndNumRepliesAsPagingSourceWithPagedParams_pagingSource.endOfPaginationReached

  val _result_findReplyPostsWithOffsetAndLimit = findReplyPostsWithOffsetAndLimit(
    offset = _pagingLoadParams.key ?: 0,
    limit = _pagingLoadParams.loadSize,
  )
  val replicationEntities = buildList<DoorReplicationEntity> {
    addAll(
      _result_findRootPostAndNumRepliesAsPagingSourceWithPagedParams.mapNotNull {
        _row ->
        _row.discussionPost?.let {
          DoorReplicationEntity(tableId = 543,
            orUid = 0,
            entity = json.encodeToJsonElement(DiscussionPost.serializer(), it).jsonObject,
          )
        }

      }
    )
    addAll(
      _result_findReplyPostsWithOffsetAndLimit.map {
        _row ->
        DoorReplicationEntity(tableId = 543,
          orUid = 0,
          entity = json.encodeToJsonElement(DiscussionPost.serializer(), _row).jsonObject,
        )
      }
    )
  }
  val _thisNodeId = request.db.doorWrapperNodeId
  return DoorJsonResponse(
    headers = buildList {
      add(Pair(DoorConstants.HEADER_NODE_ID, _thisNodeId.toString()))
      add(Pair(DoorConstants.HEADER_PAGING_END_REACHED, _endOfPaginationReached.toString()))
    }
    ,
    bodyText = json.encodeToString(
      DoorMessage.serializer(),
      DoorMessage(
        what = DoorMessage.WHAT_REPLICATION_PULL,
        fromNode = _thisNodeId,
        toNode = request.requireNodeId(),
        replications = replicationEntities,
      )
    )
  )
}

@Suppress("LocalVariableName", "RedundantSuppression", "FunctionName",
    "RedundantVisibilityModifier")
public suspend
    fun DiscussionPostDao.findRepliesWithAuthCheck_DoorHttp(serverConfig: DoorHttpServerConfig,
    request: DoorJsonRequest): DoorJsonResponse {
  val json = serverConfig.json
  val _arg_postUid : Long = request.queryParam("postUid")?.let {
    json.decodeFromString(Long.serializer(), it)
  } ?: 0L
  if(!request.db.doorWrapper.getDaoByClass(MemberDao::class).checkMemberNodeHasPermission(
    postUid = _arg_postUid,
    nodeId = request.requireNodeId(),
  )) {
    return DoorJsonResponse.newErrorResponse(403)
  }
  val _result_findRepliesWithAuthCheck = findRepliesWithAuthCheck(
    postUid = _arg_postUid,
  )
  val replicationEntities = buildList<DoorReplicationEntity> {
    addAll(
      _result_findRepliesWithAuthCheck.map {
        _row ->
        DoorReplicationEntity(tableId = 543,
          orUid = 0,
          entity = json.encodeToJsonElement(DiscussionPost.serializer(), _row).jsonObject,
        )
      }
    )
  }
  val _thisNodeId = request.db.doorWrapperNodeId
  return DoorJsonResponse(
    headers = buildList {
      add(Pair(DoorConstants.HEADER_NODE_ID, _thisNodeId.toString()))
    }
    ,
    bodyText = json.encodeToString(
      DoorMessage.serializer(),
      DoorMessage(
        what = DoorMessage.WHAT_REPLICATION_PULL,
        fromNode = _thisNodeId,
        toNode = request.requireNodeId(),
        replications = replicationEntities,
      )
    )
  )
}

@Suppress("LocalVariableName", "RedundantSuppression", "FunctionName",
    "RedundantVisibilityModifier")
public suspend
    fun DiscussionPostDao.getDiscussionPostAndAuthorName_DoorHttp(serverConfig: DoorHttpServerConfig,
    request: DoorJsonRequest): DoorJsonResponse {
  val json = serverConfig.json
  val _arg_postUid : Long = request.queryParam("postUid")?.let {
    json.decodeFromString(Long.serializer(), it)
  } ?: 0L
  val _result_getDiscussionPostAndAuthorName = getDiscussionPostAndAuthorName(
    postUid = _arg_postUid,
  )
  val _result_findAuthorByPostUid =
      request.db.doorWrapper.getDaoByClass(MemberDao::class).findAuthorByPostUid(
    postUid = _arg_postUid,
  )
  val replicationEntities = buildList<DoorReplicationEntity> {
    _result_getDiscussionPostAndAuthorName?.discussionPost?.also {
      _row -> 
      add(
        DoorReplicationEntity(tableId = 543,
          orUid = 0,
          entity = json.encodeToJsonElement(DiscussionPost.serializer(), _row).jsonObject,
        )
      )
    }
    _result_findAuthorByPostUid?.also {
      _row -> 
      add(
        DoorReplicationEntity(tableId = 544,
          orUid = 0,
          entity = json.encodeToJsonElement(Member.serializer(), _row).jsonObject,
        )
      )
    }
  }
  val _thisNodeId = request.db.doorWrapperNodeId
  return DoorJsonResponse(
    headers = buildList {
      add(Pair(DoorConstants.HEADER_NODE_ID, _thisNodeId.toString()))
    }
    ,
    bodyText = json.encodeToString(
      DoorMessage.serializer(),
      DoorMessage(
        what = DoorMessage.WHAT_REPLICATION_PULL,
        fromNode = _thisNodeId,
        toNode = request.requireNodeId(),
        replications = replicationEntities,
      )
    )
  )
}

@Suppress("LocalVariableName", "RedundantSuppression", "FunctionName",
    "RedundantVisibilityModifier")
public suspend
    fun DiscussionPostDao.getDiscussionPostAndAuthorNameById_DoorHttp(serverConfig: DoorHttpServerConfig,
    request: DoorJsonRequest): DoorJsonResponse {
  val json = serverConfig.json
  val _arg_postId : Long = request.queryParam("postId")?.let {
    json.decodeFromString(Long.serializer(), it)
  } ?: 0L
  val _result_getDiscussionPostAndAuthorNameById = getDiscussionPostAndAuthorNameById(
    postId = _arg_postId,
  )
  val _result_findAuthorByPostUid =
      request.db.doorWrapper.getDaoByClass(MemberDao::class).findAuthorByPostUid(
    postUid = _arg_postId,
  )
  val replicationEntities = buildList<DoorReplicationEntity> {
    _result_getDiscussionPostAndAuthorNameById?.discussionPost?.also {
      _row -> 
      add(
        DoorReplicationEntity(tableId = 543,
          orUid = 0,
          entity = json.encodeToJsonElement(DiscussionPost.serializer(), _row).jsonObject,
        )
      )
    }
    _result_findAuthorByPostUid?.also {
      _row -> 
      add(
        DoorReplicationEntity(tableId = 544,
          orUid = 0,
          entity = json.encodeToJsonElement(Member.serializer(), _row).jsonObject,
        )
      )
    }
  }
  val _thisNodeId = request.db.doorWrapperNodeId
  return DoorJsonResponse(
    headers = buildList {
      add(Pair(DoorConstants.HEADER_NODE_ID, _thisNodeId.toString()))
    }
    ,
    bodyText = json.encodeToString(
      DoorMessage.serializer(),
      DoorMessage(
        what = DoorMessage.WHAT_REPLICATION_PULL,
        fromNode = _thisNodeId,
        toNode = request.requireNodeId(),
        replications = replicationEntities,
      )
    )
  )
}

@Suppress("LocalVariableName", "RedundantSuppression", "FunctionName",
    "RedundantVisibilityModifier")
public suspend fun DiscussionPostDao.findByName_DoorHttp(serverConfig: DoorHttpServerConfig,
    request: DoorJsonRequest): DoorJsonResponse {
  val json = serverConfig.json
  val _arg_firstName : String? = request.queryParam("firstName")?.let {
    json.decodeFromString(String.serializer().nullable, it)
  }
  val _result_findByName = findByName(
    firstName = _arg_firstName,
  )
  val replicationEntities = buildList<DoorReplicationEntity> {
    addAll(
      _result_findByName.map {
        _row ->
        DoorReplicationEntity(tableId = 543,
          orUid = 0,
          entity = json.encodeToJsonElement(DiscussionPost.serializer(), _row).jsonObject,
        )
      }
    )
  }
  val _thisNodeId = request.db.doorWrapperNodeId
  return DoorJsonResponse(
    headers = buildList {
      add(Pair(DoorConstants.HEADER_NODE_ID, _thisNodeId.toString()))
    }
    ,
    bodyText = json.encodeToString(
      DoorMessage.serializer(),
      DoorMessage(
        what = DoorMessage.WHAT_REPLICATION_PULL,
        fromNode = _thisNodeId,
        toNode = request.requireNodeId(),
        replications = replicationEntities,
      )
    )
  )
}
