package db3

import com.google.gson.Gson
import com.ustadmobile.door.AbstractDoorUriResponder
import com.ustadmobile.door.DoorConstants
import com.ustadmobile.door.DoorDaoProvider
import com.ustadmobile.door.NanoHttpdCall
import com.ustadmobile.door.ext.DoorTag
import com.ustadmobile.door.room.RoomDatabase
import fi.iki.elonen.NanoHTTPD
import fi.iki.elonen.router.RouterNanoHTTPD
import kotlin.Long
import kotlin.String
import kotlin.collections.List
import kotlin.collections.Map
import kotlin.text.substringAfterLast
import kotlinx.coroutines.runBlocking
import org.kodein.di.DI
import org.kodein.di.Instance
import org.kodein.di.instance
import org.kodein.di.on
import org.kodein.type.TypeToken

public class DiscussionPostDao_UriResponder : AbstractDoorUriResponder() {
  public fun findByUidAsyncOverHttp(
    _uriResource: RouterNanoHTTPD.UriResource,
    _urlParams: Map<String, String>,
    _session: NanoHTTPD.IHTTPSession,
    _dao: DiscussionPostDao,
    _gson: Gson,
  ): NanoHTTPD.Response {
    val __postUid : Long = _session.parameters.get("postUid")?.get(0)?.toLong() ?: 0L
    val _result = runBlocking {
      _dao.findByUidAsyncOverHttp(__postUid)
    }
    if(_result != null) {
      return NanoHTTPD.newFixedLengthResponse(NanoHTTPD.Response.Status.OK,
          DoorConstants.MIME_TYPE_JSON, _gson.toJson(_result!!))
    } else {
      return NanoHTTPD.newFixedLengthResponse(NanoHTTPD.Response.Status.NO_CONTENT,
          DoorConstants.MIME_TYPE_PLAIN, "")
    }
  }

  public fun findAllRepliesWithPosterMember(
    _uriResource: RouterNanoHTTPD.UriResource,
    _urlParams: Map<String, String>,
    _session: NanoHTTPD.IHTTPSession,
    _dao: DiscussionPostDao,
    _gson: Gson,
  ): NanoHTTPD.Response {
    val __postUid : Long = _session.parameters.get("postUid")?.get(0)?.toLong() ?: 0L
    val _result = runBlocking {
      _dao.findAllRepliesWithPosterMember(__postUid)
    }
    return NanoHTTPD.newFixedLengthResponse(NanoHTTPD.Response.Status.OK,
        DoorConstants.MIME_TYPE_JSON, _gson.toJson(_result))
  }

  public fun findAllRepliesWithPosterMemberAsFlow(
    _uriResource: RouterNanoHTTPD.UriResource,
    _urlParams: Map<String, String>,
    _session: NanoHTTPD.IHTTPSession,
    _dao: DiscussionPostDao,
    _gson: Gson,
  ): NanoHTTPD.Response {
    val __postUid : Long = _session.parameters.get("postUid")?.get(0)?.toLong() ?: 0L
    val _result = _dao.findAllRepliesWithPosterMemberAsFlow(__postUid)
    return NanoHTTPD.newFixedLengthResponse(NanoHTTPD.Response.Status.OK,
        DoorConstants.MIME_TYPE_JSON, _gson.toJson(_result))
  }

  public fun findByUidWithPosterMember(
    _uriResource: RouterNanoHTTPD.UriResource,
    _urlParams: Map<String, String>,
    _session: NanoHTTPD.IHTTPSession,
    _dao: DiscussionPostDao,
    _gson: Gson,
  ): NanoHTTPD.Response {
    val __postUid : Long = _session.parameters.get("postUid")?.get(0)?.toLong() ?: 0L
    val _result = runBlocking {
      _dao.findByUidWithPosterMember(__postUid)
    }
    if(_result != null) {
      return NanoHTTPD.newFixedLengthResponse(NanoHTTPD.Response.Status.OK,
          DoorConstants.MIME_TYPE_JSON, _gson.toJson(_result!!))
    } else {
      return NanoHTTPD.newFixedLengthResponse(NanoHTTPD.Response.Status.NO_CONTENT,
          DoorConstants.MIME_TYPE_PLAIN, "")
    }
  }

  public fun findByUidWithPosterMemberAsFlow(
    _uriResource: RouterNanoHTTPD.UriResource,
    _urlParams: Map<String, String>,
    _session: NanoHTTPD.IHTTPSession,
    _dao: DiscussionPostDao,
    _gson: Gson,
  ): NanoHTTPD.Response {
    val __postUid : Long = _session.parameters.get("postUid")?.get(0)?.toLong() ?: 0L
    val _result = _dao.findByUidWithPosterMemberAsFlow(__postUid)
    return NanoHTTPD.newFixedLengthResponse(NanoHTTPD.Response.Status.OK,
        DoorConstants.MIME_TYPE_JSON, _gson.toJson(_result))
  }

  public fun findByUidList(
    _uriResource: RouterNanoHTTPD.UriResource,
    _urlParams: Map<String, String>,
    _session: NanoHTTPD.IHTTPSession,
    _dao: DiscussionPostDao,
    _gson: Gson,
  ): NanoHTTPD.Response {
    val __postUids : List<Long> = _session.parameters["postUids"] ?.map { it.toLong() } ?: listOf()

    val _result = runBlocking {
      _dao.findByUidList(__postUids)
    }
    return NanoHTTPD.newFixedLengthResponse(NanoHTTPD.Response.Status.OK,
        DoorConstants.MIME_TYPE_JSON, _gson.toJson(_result))
  }

  public fun getNumPostsSinceTime(
    _uriResource: RouterNanoHTTPD.UriResource,
    _urlParams: Map<String, String>,
    _session: NanoHTTPD.IHTTPSession,
    _dao: DiscussionPostDao,
    _gson: Gson,
  ): NanoHTTPD.Response {
    val __since : Long = _session.parameters.get("since")?.get(0)?.toLong() ?: 0L
    val _result = runBlocking {
      _dao.getNumPostsSinceTime(__since)
    }
    return NanoHTTPD.newFixedLengthResponse(NanoHTTPD.Response.Status.OK,
        DoorConstants.MIME_TYPE_JSON, _gson.toJson(_result))
  }

  public fun getNumPostsSinceTimeHttpOnly(
    _uriResource: RouterNanoHTTPD.UriResource,
    _urlParams: Map<String, String>,
    _session: NanoHTTPD.IHTTPSession,
    _dao: DiscussionPostDao,
    _gson: Gson,
  ): NanoHTTPD.Response {
    val __since : Long = _session.parameters.get("since")?.get(0)?.toLong() ?: 0L
    val _result = runBlocking {
      _dao.getNumPostsSinceTimeHttpOnly(__since)
    }
    return NanoHTTPD.newFixedLengthResponse(NanoHTTPD.Response.Status.OK,
        DoorConstants.MIME_TYPE_JSON, _gson.toJson(_result))
  }

  public fun findAllPostAsPagingSource(
    _uriResource: RouterNanoHTTPD.UriResource,
    _urlParams: Map<String, String>,
    _session: NanoHTTPD.IHTTPSession,
    _dao: DiscussionPostDao,
    _gson: Gson,
  ): NanoHTTPD.Response {
    val __since : Long = _session.parameters.get("since")?.get(0)?.toLong() ?: 0L
    val _result = _dao.findAllPostAsPagingSource(__since)
    return NanoHTTPD.newFixedLengthResponse(NanoHTTPD.Response.Status.OK,
        DoorConstants.MIME_TYPE_JSON, _gson.toJson(_result))
  }

  public fun findAllPostAsNetworkOnlyPagingSource(
    _uriResource: RouterNanoHTTPD.UriResource,
    _urlParams: Map<String, String>,
    _session: NanoHTTPD.IHTTPSession,
    _dao: DiscussionPostDao,
    _gson: Gson,
  ): NanoHTTPD.Response {
    val __since : Long = _session.parameters.get("since")?.get(0)?.toLong() ?: 0L
    val _result = _dao.findAllPostAsNetworkOnlyPagingSource(__since)
    return NanoHTTPD.newFixedLengthResponse(NanoHTTPD.Response.Status.OK,
        DoorConstants.MIME_TYPE_JSON, _gson.toJson(_result))
  }

  public fun findAllPostAsNetworkWithFallbackPagingSource(
    _uriResource: RouterNanoHTTPD.UriResource,
    _urlParams: Map<String, String>,
    _session: NanoHTTPD.IHTTPSession,
    _dao: DiscussionPostDao,
    _gson: Gson,
  ): NanoHTTPD.Response {
    val __since : Long = _session.parameters.get("since")?.get(0)?.toLong() ?: 0L
    val _result = _dao.findAllPostAsNetworkWithFallbackPagingSource(__since)
    return NanoHTTPD.newFixedLengthResponse(NanoHTTPD.Response.Status.OK,
        DoorConstants.MIME_TYPE_JSON, _gson.toJson(_result))
  }

  public fun findPostAndNumReplies(
    _uriResource: RouterNanoHTTPD.UriResource,
    _urlParams: Map<String, String>,
    _session: NanoHTTPD.IHTTPSession,
    _dao: DiscussionPostDao,
    _gson: Gson,
  ): NanoHTTPD.Response {
    val __postUid : Long = _session.parameters.get("postUid")?.get(0)?.toLong() ?: 0L
    val __sinceTime : Long = _session.parameters.get("sinceTime")?.get(0)?.toLong() ?: 0L
    val _result = runBlocking {
      _dao.findPostAndNumReplies(__postUid,__sinceTime)
    }
    if(_result != null) {
      return NanoHTTPD.newFixedLengthResponse(NanoHTTPD.Response.Status.OK,
          DoorConstants.MIME_TYPE_JSON, _gson.toJson(_result!!))
    } else {
      return NanoHTTPD.newFixedLengthResponse(NanoHTTPD.Response.Status.NO_CONTENT,
          DoorConstants.MIME_TYPE_PLAIN, "")
    }
  }

  public fun findRootPostsAndNumRepliesAsPagingSource(
    _uriResource: RouterNanoHTTPD.UriResource,
    _urlParams: Map<String, String>,
    _session: NanoHTTPD.IHTTPSession,
    _dao: DiscussionPostDao,
    _gson: Gson,
  ): NanoHTTPD.Response {
    val _result = _dao.findRootPostsAndNumRepliesAsPagingSource()
    return NanoHTTPD.newFixedLengthResponse(NanoHTTPD.Response.Status.OK,
        DoorConstants.MIME_TYPE_JSON, _gson.toJson(_result))
  }

  public fun findRootPostAndNumRepliesAsPagingSourceWithPagedParams(
    _uriResource: RouterNanoHTTPD.UriResource,
    _urlParams: Map<String, String>,
    _session: NanoHTTPD.IHTTPSession,
    _dao: DiscussionPostDao,
    _gson: Gson,
  ): NanoHTTPD.Response {
    val _result = _dao.findRootPostAndNumRepliesAsPagingSourceWithPagedParams()
    return NanoHTTPD.newFixedLengthResponse(NanoHTTPD.Response.Status.OK,
        DoorConstants.MIME_TYPE_JSON, _gson.toJson(_result))
  }

  public fun findRepliesWithAuthCheck(
    _uriResource: RouterNanoHTTPD.UriResource,
    _urlParams: Map<String, String>,
    _session: NanoHTTPD.IHTTPSession,
    _dao: DiscussionPostDao,
    _gson: Gson,
  ): NanoHTTPD.Response {
    val __postUid : Long = _session.parameters.get("postUid")?.get(0)?.toLong() ?: 0L
    val _result = runBlocking {
      _dao.findRepliesWithAuthCheck(__postUid)
    }
    return NanoHTTPD.newFixedLengthResponse(NanoHTTPD.Response.Status.OK,
        DoorConstants.MIME_TYPE_JSON, _gson.toJson(_result))
  }

  public fun getDiscussionPostAndAuthorName(
    _uriResource: RouterNanoHTTPD.UriResource,
    _urlParams: Map<String, String>,
    _session: NanoHTTPD.IHTTPSession,
    _dao: DiscussionPostDao,
    _gson: Gson,
  ): NanoHTTPD.Response {
    val __postUid : Long = _session.parameters.get("postUid")?.get(0)?.toLong() ?: 0L
    val _result = runBlocking {
      _dao.getDiscussionPostAndAuthorName(__postUid)
    }
    if(_result != null) {
      return NanoHTTPD.newFixedLengthResponse(NanoHTTPD.Response.Status.OK,
          DoorConstants.MIME_TYPE_JSON, _gson.toJson(_result!!))
    } else {
      return NanoHTTPD.newFixedLengthResponse(NanoHTTPD.Response.Status.NO_CONTENT,
          DoorConstants.MIME_TYPE_PLAIN, "")
    }
  }

  public fun getDiscussionPostAndAuthorNameById(
    _uriResource: RouterNanoHTTPD.UriResource,
    _urlParams: Map<String, String>,
    _session: NanoHTTPD.IHTTPSession,
    _dao: DiscussionPostDao,
    _gson: Gson,
  ): NanoHTTPD.Response {
    val __postId : Long = _session.parameters.get("postId")?.get(0)?.toLong() ?: 0L
    val _result = runBlocking {
      _dao.getDiscussionPostAndAuthorNameById(__postId)
    }
    if(_result != null) {
      return NanoHTTPD.newFixedLengthResponse(NanoHTTPD.Response.Status.OK,
          DoorConstants.MIME_TYPE_JSON, _gson.toJson(_result!!))
    } else {
      return NanoHTTPD.newFixedLengthResponse(NanoHTTPD.Response.Status.NO_CONTENT,
          DoorConstants.MIME_TYPE_PLAIN, "")
    }
  }

  public fun findByName(
    _uriResource: RouterNanoHTTPD.UriResource,
    _urlParams: Map<String, String>,
    _session: NanoHTTPD.IHTTPSession,
    _dao: DiscussionPostDao,
    _gson: Gson,
  ): NanoHTTPD.Response {
    val __firstName : String? = _session.parameters.get("firstName")?.get(0)?.toString() ?: null as
        kotlin.String??
    val _result = runBlocking {
      _dao.findByName(__firstName)
    }
    return NanoHTTPD.newFixedLengthResponse(NanoHTTPD.Response.Status.OK,
        DoorConstants.MIME_TYPE_JSON, _gson.toJson(_result))
  }

  override fun `get`(
    _uriResource: RouterNanoHTTPD.UriResource,
    _urlParams: Map<String, String>,
    _session: NanoHTTPD.IHTTPSession,
  ): NanoHTTPD.Response {
    val _fnName = _session.uri.substringAfterLast('/')
    val _di = _uriResource.initParameter(0, DI::class.java)
    val _daoProvider = _uriResource.initParameter(1, DoorDaoProvider::class.java) as
        DoorDaoProvider<RoomDatabase, DiscussionPostDao>
    val _typeToken = _uriResource.initParameter(2, TypeToken::class.java) as TypeToken<RoomDatabase>
    val _call = NanoHttpdCall(_uriResource, _urlParams, _session)
    val _db: RoomDatabase by _di.on(_call).Instance(_typeToken, tag = DoorTag.TAG_DB)
    val _repo: RoomDatabase by _di.on(_call).Instance(_typeToken, tag = DoorTag.TAG_REPO)
    val _dao = _daoProvider.getDao(_db)
    val _gson : Gson by _di.instance()
    return when(_fnName) {
      "findByUidAsyncOverHttp" -> findByUidAsyncOverHttp(_uriResource, _urlParams, _session, _dao,
          _gson)
      "findAllRepliesWithPosterMember" -> findAllRepliesWithPosterMember(_uriResource, _urlParams,
          _session, _dao, _gson)
      "findAllRepliesWithPosterMemberAsFlow" -> findAllRepliesWithPosterMemberAsFlow(_uriResource,
          _urlParams, _session, _dao, _gson)
      "findByUidWithPosterMember" -> findByUidWithPosterMember(_uriResource, _urlParams, _session,
          _dao, _gson)
      "findByUidWithPosterMemberAsFlow" -> findByUidWithPosterMemberAsFlow(_uriResource, _urlParams,
          _session, _dao, _gson)
      "findByUidList" -> findByUidList(_uriResource, _urlParams, _session, _dao, _gson)
      "getNumPostsSinceTime" -> getNumPostsSinceTime(_uriResource, _urlParams, _session, _dao,
          _gson)
      "getNumPostsSinceTimeHttpOnly" -> getNumPostsSinceTimeHttpOnly(_uriResource, _urlParams,
          _session, _dao, _gson)
      "findAllPostAsPagingSource" -> findAllPostAsPagingSource(_uriResource, _urlParams, _session,
          _dao, _gson)
      "findAllPostAsNetworkOnlyPagingSource" -> findAllPostAsNetworkOnlyPagingSource(_uriResource,
          _urlParams, _session, _dao, _gson)
      "findAllPostAsNetworkWithFallbackPagingSource" ->
          findAllPostAsNetworkWithFallbackPagingSource(_uriResource, _urlParams, _session, _dao,
          _gson)
      "findPostAndNumReplies" -> findPostAndNumReplies(_uriResource, _urlParams, _session, _dao,
          _gson)
      "findRootPostsAndNumRepliesAsPagingSource" ->
          findRootPostsAndNumRepliesAsPagingSource(_uriResource, _urlParams, _session, _dao, _gson)
      "findRootPostAndNumRepliesAsPagingSourceWithPagedParams" ->
          findRootPostAndNumRepliesAsPagingSourceWithPagedParams(_uriResource, _urlParams, _session,
          _dao, _gson)
      "findRepliesWithAuthCheck" -> findRepliesWithAuthCheck(_uriResource, _urlParams, _session,
          _dao, _gson)
      "getDiscussionPostAndAuthorName" -> getDiscussionPostAndAuthorName(_uriResource, _urlParams,
          _session, _dao, _gson)
      "getDiscussionPostAndAuthorNameById" -> getDiscussionPostAndAuthorNameById(_uriResource,
          _urlParams, _session, _dao, _gson)
      "findByName" -> findByName(_uriResource, _urlParams, _session, _dao, _gson)
      else -> NanoHTTPD.newFixedLengthResponse(NanoHTTPD.Response.Status.NOT_FOUND,
          DoorConstants.MIME_TYPE_PLAIN, "")
    }
  }

  override fun post(
    _uriResource: RouterNanoHTTPD.UriResource,
    _urlParams: Map<String, String>,
    _session: NanoHTTPD.IHTTPSession,
  ): NanoHTTPD.Response {
    val _fnName = _session.uri.substringAfterLast('/')
    val _di = _uriResource.initParameter(0, DI::class.java)
    val _daoProvider = _uriResource.initParameter(1, DoorDaoProvider::class.java) as
        DoorDaoProvider<RoomDatabase, DiscussionPostDao>
    val _typeToken = _uriResource.initParameter(2, TypeToken::class.java) as TypeToken<RoomDatabase>
    val _call = NanoHttpdCall(_uriResource, _urlParams, _session)
    val _db: RoomDatabase by _di.on(_call).Instance(_typeToken, tag = DoorTag.TAG_DB)
    val _repo: RoomDatabase by _di.on(_call).Instance(_typeToken, tag = DoorTag.TAG_REPO)
    val _dao = _daoProvider.getDao(_db)
    val _gson : Gson by _di.instance()
    return NanoHTTPD.newFixedLengthResponse(NanoHTTPD.Response.Status.NOT_FOUND,
        DoorConstants.MIME_TYPE_PLAIN, "")
  }
}
