package dbonly

import androidx.room.DatabaseConfiguration
import androidx.sqlite.db.SupportSQLiteOpenHelper
import com.ustadmobile.door.DoorDatabaseWrapper
import com.ustadmobile.door.log.DoorLogger
import com.ustadmobile.door.message.DoorMessageCallback
import com.ustadmobile.door.nodeevent.NodeEventManagerAndroid
import com.ustadmobile.door.nodeevent.NodeEventManagerCommon
import com.ustadmobile.door.room.RoomDatabase
import com.ustadmobile.door.util.makeDummyInvalidationHandler
import kotlin.Any
import kotlin.IllegalArgumentException
import kotlin.Long
import kotlin.String
import kotlin.Suppress
import kotlin.reflect.KClass
import kotlinx.coroutines.Runnable
import androidx.room.InvalidationTracker as AndroidxRoomInvalidationTracker
import com.ustadmobile.door.room.InvalidationTracker as DoorRoomInvalidationTracker

@Suppress("REDUNDANT_PROJECTION", "ClassName")
public class VanillaDatabase_DoorWrapper(
  private val _db: VanillaDatabase,
  override val nodeId: Long,
  private val messageCallback: DoorMessageCallback<VanillaDatabase>,
  logger: DoorLogger,
  dbName: String,
) : VanillaDatabase(), DoorDatabaseWrapper<VanillaDatabase> {
  override val dbName: String
    get() = "DoorWrapper for [${_db.toString()}]"

  override val vanillaDao: VanillaDao
    get() = _db.vanillaDao

  override val realDatabase: RoomDatabase
    get() = _db

  override val nodeEventManager: NodeEventManagerCommon<VanillaDatabase> =
      NodeEventManagerAndroid(_db, messageCallback, logger, dbName)


  override val invalidationTracker: DoorRoomInvalidationTracker
    get() = _db.invalidationTracker

  override fun clearAllTables() {
    _db.clearAllTables()
  }

  override fun runInTransaction(runnable: Runnable) {
    _db.runInTransaction(runnable)
  }

  protected override fun createOpenHelper(config: DatabaseConfiguration): SupportSQLiteOpenHelper =
      throw IllegalAccessException("Cannot use open helper on repository")

  protected override fun createInvalidationTracker(): AndroidxRoomInvalidationTracker =
      makeDummyInvalidationHandler()

  @Suppress("UNCHECKED_CAST")
  override fun <T : Any> getDaoByClass(daoClass: KClass<T>): T = when(daoClass) {
    VanillaDao::class -> vanillaDao as T
    else -> throw IllegalArgumentException("No such DAO on this DB")
  }

  override fun close() {
    _db.close()
  }
}
