package repdb

import androidx.room.DatabaseConfiguration
import androidx.sqlite.db.SupportSQLiteOpenHelper
import com.ustadmobile.door.DoorDatabaseRepository
import com.ustadmobile.door.RepositoryConfig
import com.ustadmobile.door.RepositoryHelper
import com.ustadmobile.door.replication.DoorRepositoryReplicationClient
import com.ustadmobile.door.util.makeDummyInvalidationHandler
import io.ktor.client.HttpClient
import kotlin.Any
import kotlin.IllegalStateException
import kotlin.Int
import kotlin.Long
import kotlin.String
import kotlin.Suppress
import kotlin.Unit
import kotlinx.coroutines.flow.Flow
import androidx.room.InvalidationTracker as AndroidxRoomInvalidationTracker
import com.ustadmobile.door.room.InvalidationTracker as DoorRoomInvalidationTracker

@Suppress("LocalVariableName", "PropertyName", "FunctionName", "ClassName")
public class RepDb_Repo(
  override val db: RepDb,
  dbUnwrapped: RepDb,
  override val config: RepositoryConfig,
) : RepDb(), DoorDatabaseRepository {
  public val context: Any
    get() = config.context

  private val _db: RepDb = dbUnwrapped

  private val _endpoint: String
    get() = config.endpoint

  public val _httpClient: HttpClient
    get() = config.httpClient

  public val _repositoryHelper: RepositoryHelper = RepositoryHelper(db, config)


  public val clientId: Long
    get() = config.nodeId

  override val dbName: String
    get() = "Repository for [${_db.toString()}] - ${config.endpoint}"

  override var connectivityStatus: Int
    get() = _repositoryHelper.connectivityStatus
    set(newValue) {
      _repositoryHelper.connectivityStatus = newValue
    }

  override val clientState: Flow<DoorRepositoryReplicationClient.ClientState>
    get() = _repositoryHelper.clientState

  override val invalidationTracker: DoorRoomInvalidationTracker
    get() = _db.invalidationTracker

  private val _RepDao: RepDao_Repo by lazy {
        RepDao_Repo(db, this, db.repDao, _httpClient, clientId, _endpoint)
      }


  override val repDao: RepDao
    get() =  _RepDao

  override val compositePkDao: CompositePkDao
    get() = throw IllegalStateException("CompositePkDao is not annotated with @Repository")

  private val _RepWithAttachmentDao: RepWithAttachmentDao_Repo by lazy {
        RepWithAttachmentDao_Repo(db, this, db.repWithAttachmentDao, _httpClient, clientId,
            _endpoint)
      }


  override val repWithAttachmentDao: RepWithAttachmentDao
    get() =  _RepWithAttachmentDao

  override fun clearAllTables(): Unit = throw
      IllegalStateException("Cannot use a repository to clearAllTables!")

  override fun close() {
    _repositoryHelper.close()
  }

  override fun remoteNodeIdOrNull(): Long? = _repositoryHelper.remoteNodeIdOrNull()

  override fun remoteNodeIdOrFake(): Long = _repositoryHelper.remoteNodeIdOrFake()

  protected override fun createInvalidationTracker(): AndroidxRoomInvalidationTracker =
      makeDummyInvalidationHandler()

  protected override fun createOpenHelper(config: DatabaseConfiguration): SupportSQLiteOpenHelper =
      throw IllegalAccessException("Cannot use open helper on repository")

  public companion object {
    public const val _DB_NAME: String = "RepDb"
  }
}
