package db3

import androidx.paging.PagingSource
import com.ustadmobile.door.ext.doorPrimaryKeyManager
import com.ustadmobile.door.ext.rootDatabase
import com.ustadmobile.door.room.RoomDatabase
import com.ustadmobile.door.util.systemTimeInMillis
import kotlin.Boolean
import kotlin.Int
import kotlin.Long
import kotlin.String
import kotlin.collections.List
import kotlinx.coroutines.flow.Flow

public class DiscussionPostDao_DoorWrapper(
  private val _db: RoomDatabase,
  private val _dao: DiscussionPostDao,
) : DiscussionPostDao() {
  override suspend fun insertAsync(post: DiscussionPost): Long {
    val _pkManager = _db.rootDatabase.doorPrimaryKeyManager
    if(post.postUid == 0L) {
      val _newPk = _pkManager.nextIdAsync(543)
      post.postUid = _newPk
    }
    post.postLastModified = systemTimeInMillis()
    //must set versionid and/or primary key here
    _dao.insertAsync(post)
    return post.postUid
  }

  override suspend fun findByUid(postUid: Long): DiscussionPost? = _dao.findByUid(postUid)

  override suspend fun findByUidAsyncOverHttp(postUid: Long): DiscussionPost? =
      _dao.findByUidAsyncOverHttp(postUid)

  override suspend fun findAllRepliesWithPosterMember(postUid: Long):
      List<DiscussionPostAndPosterMember> = _dao.findAllRepliesWithPosterMember(postUid)

  override fun findAllRepliesWithPosterMemberAsFlow(postUid: Long):
      Flow<List<DiscussionPostAndPosterMember>> = _dao.findAllRepliesWithPosterMemberAsFlow(postUid)

  override suspend fun findByUidWithPosterMember(postUid: Long): DiscussionPostAndPosterMember? =
      _dao.findByUidWithPosterMember(postUid)

  override fun findByUidWithPosterMemberAsFlow(postUid: Long): Flow<DiscussionPostAndPosterMember?>
      = _dao.findByUidWithPosterMemberAsFlow(postUid)

  override suspend fun findByUidList(postUids: List<Long>): List<DiscussionPostAndPosterMember> =
      _dao.findByUidList(postUids)

  override suspend fun getNumPostsSinceTime(since: Long): Int = _dao.getNumPostsSinceTime(since)

  override suspend fun getNumPostsSinceTimeHttpOnly(since: Long): Int =
      _dao.getNumPostsSinceTimeHttpOnly(since)

  override fun findAllPostAsPagingSource(since: Long): PagingSource<Int, DiscussionPost> =
      _dao.findAllPostAsPagingSource(since)

  override fun findAllPostAsNetworkOnlyPagingSource(since: Long): PagingSource<Int, DiscussionPost>
      = _dao.findAllPostAsNetworkOnlyPagingSource(since)

  override fun findAllPostAsNetworkWithFallbackPagingSource(since: Long):
      PagingSource<Int, DiscussionPost> = _dao.findAllPostAsNetworkWithFallbackPagingSource(since)

  override suspend fun findAllRepliesForPost(postUid: Long, nodeId: Long): List<DiscussionPost> =
      _dao.findAllRepliesForPost(postUid, nodeId)

  override suspend fun findPostAndNumReplies(postUid: Long, sinceTime: Long):
      DiscussionPostAndNumReplies? = _dao.findPostAndNumReplies(postUid, sinceTime)

  override fun findRootRepliesAsPagingSource(): PagingSource<Int, DiscussionPost> =
      _dao.findRootRepliesAsPagingSource()

  override fun findRootPostsAndNumRepliesAsPagingSource():
      PagingSource<Int, DiscussionPostAndNumReplies> =
      _dao.findRootPostsAndNumRepliesAsPagingSource()

  override suspend fun findReplyPostsWithOffsetAndLimit(offset: Int, limit: Int):
      List<DiscussionPost> = _dao.findReplyPostsWithOffsetAndLimit(offset, limit)

  override fun findRootPostAndNumRepliesAsPagingSourceWithPagedParams():
      PagingSource<Int, DiscussionPostAndNumReplies> =
      _dao.findRootPostAndNumRepliesAsPagingSourceWithPagedParams()

  override fun findRootPostAndNumRepliesAsPagingSourceWithAsFlow():
      Flow<List<DiscussionPostAndNumReplies>> =
      _dao.findRootPostAndNumRepliesAsPagingSourceWithAsFlow()

  override suspend fun checkNodeHasPermission(postUid: Long, nodeId: Long): Boolean =
      _dao.checkNodeHasPermission(postUid, nodeId)

  override suspend fun findRepliesWithAuthCheck(postUid: Long): List<DiscussionPost> =
      _dao.findRepliesWithAuthCheck(postUid)

  override suspend fun getDiscussionPostAndAuthorName(postUid: Long): DiscussionPostAndAuthorName? =
      _dao.getDiscussionPostAndAuthorName(postUid)

  override suspend fun getDiscussionPostAndAuthorNameById(postId: Long):
      DiscussionPostAndAuthorName? = _dao.getDiscussionPostAndAuthorNameById(postId)

  override suspend fun update(post: DiscussionPost) {
    post.postLastModified = systemTimeInMillis()
    //must set versionid and/or primary key here
    _dao.update(post)
  }

  override suspend fun findByName(firstName: String?): List<DiscussionPost> =
      _dao.findByName(firstName)
}
