package db2

import androidx.paging.PagingSource
import com.ustadmobile.door.DoorDbType
import com.ustadmobile.door.DoorQuery
import com.ustadmobile.door.EntityInsertionAdapter
import com.ustadmobile.door.PreparedStatementConfig
import com.ustadmobile.door.ext.createArrayOrProxyArrayOf
import com.ustadmobile.door.ext.hasListOrArrayParams
import com.ustadmobile.door.ext.prepareAndUseStatement
import com.ustadmobile.door.ext.prepareAndUseStatementAsync
import com.ustadmobile.door.flow.doorFlow
import com.ustadmobile.door.jdbc.PreparedStatement
import com.ustadmobile.door.jdbc.ext.executeQueryAsyncKmp
import com.ustadmobile.door.jdbc.ext.executeUpdateAsyncKmp
import com.ustadmobile.door.jdbc.ext.getIntNullable
import com.ustadmobile.door.jdbc.ext.getLongNullable
import com.ustadmobile.door.jdbc.ext.mapNextRow
import com.ustadmobile.door.jdbc.ext.mapRows
import com.ustadmobile.door.jdbc.ext.setIntNullable
import com.ustadmobile.door.jdbc.ext.useResults
import com.ustadmobile.door.paging.DoorLimitOffsetPagingSource
import com.ustadmobile.door.room.RoomDatabase
import kotlin.Array
import kotlin.Boolean
import kotlin.Int
import kotlin.Long
import kotlin.String
import kotlin.collections.List
import kotlinx.coroutines.flow.Flow

public class ExampleDao2_JdbcImpl(
  public val _db: RoomDatabase,
) : ExampleDao2() {
  public val _insertAdapterExampleEntity2_abort: EntityInsertionAdapter<ExampleEntity2> = object :
      EntityInsertionAdapter<ExampleEntity2>(_db) {
    override fun makeSql(returnsId: Boolean): String =
        "INSERT INTO ExampleEntity2 (uid, name, someNumber, checked, rewardsCardNumber) VALUES(?, ?, ?, ?, ?)"

    override fun bindPreparedStmtToEntity(stmt: PreparedStatement, entity: ExampleEntity2) {
      if(entity.uid == 0L) {
        stmt.setObject(1, null)
      } else {
        stmt.setLong(1, entity.uid)
      }
      stmt.setString(2, entity.name)
      stmt.setLong(3, entity.someNumber)
      stmt.setBoolean(4, entity.checked)
      stmt.setIntNullable(5, entity.rewardsCardNumber)
    }
  }

  public val _insertAdapterExampleEntity2_ignore: EntityInsertionAdapter<ExampleEntity2> = object :
      EntityInsertionAdapter<ExampleEntity2>(_db) {
    override fun makeSql(returnsId: Boolean): String =
        "INSERT  OR IGNORE INTO ExampleEntity2 (uid, name, someNumber, checked, rewardsCardNumber) VALUES(?, ?, ?, ?, ?)"

    override fun bindPreparedStmtToEntity(stmt: PreparedStatement, entity: ExampleEntity2) {
      if(entity.uid == 0L) {
        stmt.setObject(1, null)
      } else {
        stmt.setLong(1, entity.uid)
      }
      stmt.setString(2, entity.name)
      stmt.setLong(3, entity.someNumber)
      stmt.setBoolean(4, entity.checked)
      stmt.setIntNullable(5, entity.rewardsCardNumber)
    }
  }

  public val _insertAdapterExampleEntity2_upsert: EntityInsertionAdapter<ExampleEntity2> = object :
      EntityInsertionAdapter<ExampleEntity2>(_db) {
    override fun makeSql(returnsId: Boolean): String =
        "INSERT OR REPLACE INTO ExampleEntity2 (uid, name, someNumber, checked, rewardsCardNumber) VALUES(?, ?, ?, ?, ?)"

    override fun bindPreparedStmtToEntity(stmt: PreparedStatement, entity: ExampleEntity2) {
      if(entity.uid == 0L) {
        stmt.setObject(1, null)
      } else {
        stmt.setLong(1, entity.uid)
      }
      stmt.setString(2, entity.name)
      stmt.setLong(3, entity.someNumber)
      stmt.setBoolean(4, entity.checked)
      stmt.setIntNullable(5, entity.rewardsCardNumber)
    }
  }

  override fun insertAndReturnId(entity: ExampleEntity2): Long {
    val _retVal = _insertAdapterExampleEntity2_abort.insertAndReturnId(entity)
    return _retVal
  }

  override suspend fun insertAsync(entity: ExampleEntity2) {
    _insertAdapterExampleEntity2_abort.insertAsync(entity)
  }

  override suspend fun insertAsyncAndGiveId(entity: ExampleEntity2): Long {
    val _retVal = _insertAdapterExampleEntity2_abort.insertAndReturnIdAsync(entity)
    return _retVal
  }

  override suspend fun insertOrIgnore(entity: ExampleEntity2) {
    _insertAdapterExampleEntity2_ignore.insertAsync(entity)
  }

  override fun insertList(entityList: List<ExampleEntity2>) {
    _insertAdapterExampleEntity2_abort.insertList(entityList)
  }

  override suspend fun insertListAsync(entityList: List<ExampleEntity2>) {
    _insertAdapterExampleEntity2_abort.insertListAsync(entityList)
  }

  override fun insertOtherList(entityList: List<ExampleEntity2>) {
    _insertAdapterExampleEntity2_abort.insertList(entityList)
  }

  override fun insertAndReturnList(entityList: List<ExampleEntity2>): List<Long> {
    val _retVal = _insertAdapterExampleEntity2_abort.insertListAndReturnIds(entityList)
    return _retVal
  }

  override fun insertListAndReturnIdsArray(entityList: List<ExampleEntity2>): Array<Long> {
    val _retVal = _insertAdapterExampleEntity2_abort.insertListAndReturnIds(entityList)
    return _retVal.toTypedArray()
  }

  override fun replace(entityList: List<ExampleEntity2>) {
    _insertAdapterExampleEntity2_upsert.insertList(entityList)
  }

  override suspend fun updateSingleItemAsync(entity: ExampleEntity2) {
    val _sql =
        "UPDATE ExampleEntity2 SET name = ?, someNumber = ?, checked = ?, rewardsCardNumber = ? WHERE uid = ?"
    _db.prepareAndUseStatementAsync(_sql) {
       _stmt ->
      _stmt.setString(1, entity.name)
      _stmt.setLong(2, entity.someNumber)
      _stmt.setBoolean(3, entity.checked)
      _stmt.setIntNullable(4, entity.rewardsCardNumber)
      _stmt.setLong(5, entity.uid)
      _stmt.executeUpdateAsyncKmp()
    }
  }

  override suspend fun updateSingleItemAndReturnCountAsync(entity: ExampleEntity2): Int {
    var _result = 0
    val _sql =
        "UPDATE ExampleEntity2 SET name = ?, someNumber = ?, checked = ?, rewardsCardNumber = ? WHERE uid = ?"
    _db.prepareAndUseStatementAsync(_sql) {
       _stmt ->
      _stmt.setString(1, entity.name)
      _stmt.setLong(2, entity.someNumber)
      _stmt.setBoolean(3, entity.checked)
      _stmt.setIntNullable(4, entity.rewardsCardNumber)
      _stmt.setLong(5, entity.uid)
      _result += _stmt.executeUpdateAsyncKmp()
    }
    return _result
  }

  override suspend fun updateListAsync(updateEntityList: List<ExampleEntity2>) {
    val _sql =
        "UPDATE ExampleEntity2 SET name = ?, someNumber = ?, checked = ?, rewardsCardNumber = ? WHERE uid = ?"
    _db.prepareAndUseStatementAsync(_sql) {
       _stmt ->
      _stmt.getConnection().setAutoCommit(false)
      for(_entity in updateEntityList) {
        _stmt.setString(1, _entity.name)
        _stmt.setLong(2, _entity.someNumber)
        _stmt.setBoolean(3, _entity.checked)
        _stmt.setIntNullable(4, _entity.rewardsCardNumber)
        _stmt.setLong(5, _entity.uid)
        _stmt.executeUpdateAsyncKmp()
      }
      _stmt.getConnection().commit()
    }
  }

  override fun deleteSingle(entity: ExampleEntity2) {
    var _numChanges = 0
    _db.prepareAndUseStatement("DELETE FROM ExampleEntity2 WHERE uid = ?") {
       _stmt ->
      _stmt.setLong(1, entity.uid)
      _numChanges += _stmt.executeUpdate()
    }
  }

  override fun deleteList(deleteList: List<ExampleEntity2>) {
    var _numChanges = 0
    _db.prepareAndUseStatement("DELETE FROM ExampleEntity2 WHERE uid = ?") {
       _stmt ->
      _stmt.getConnection().setAutoCommit(false)
      for(_entity in deleteList) {
        _stmt.setLong(1, _entity.uid)
        _numChanges += _stmt.executeUpdate()
      }
      _stmt.getConnection().commit()
    }
  }

  override suspend fun deleteListAsync(deleteList: List<ExampleEntity2>): Int {
    var _numChanges = 0
    _db.prepareAndUseStatementAsync("DELETE FROM ExampleEntity2 WHERE uid = ?") {
       _stmt ->
      _stmt.getConnection().setAutoCommit(false)
      for(_entity in deleteList) {
        _stmt.setLong(1, _entity.uid)
        _numChanges += _stmt.executeUpdateAsyncKmp()
      }
      _stmt.getConnection().commit()
    }
    return _numChanges
  }

  override fun findByUid(uid: Long): ExampleEntity2? =
      _db.prepareAndUseStatement(PreparedStatementConfig(
    sql = "SELECT * FROM ExampleEntity2 WHERE uid = CAST(? AS BIGINT)",
    postgreSql = """
    |SELECT * FROM ExampleEntity2 WHERE uid = ?
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,uid)
    _stmt.executeQuery().useResults{ _result -> 
      _result.mapNextRow(null) {
        val _tmp_uid = _result.getLong("uid")
        val _tmp_name = _result.getString("name")
        val _tmp_someNumber = _result.getLong("someNumber")
        val _tmp_checked = _result.getBoolean("checked")
        val _tmp_rewardsCardNumber = _result.getIntNullable("rewardsCardNumber")
        ExampleEntity2().apply {
          this.uid = _tmp_uid
          this.name = _tmp_name
          this.someNumber = _tmp_someNumber
          this.checked = _tmp_checked
          this.rewardsCardNumber = _tmp_rewardsCardNumber
        }
      }
    }
  }

  override suspend fun findByUidAsync(uid: Long): ExampleEntity2? =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = "SELECT * FROM ExampleEntity2 WHERE uid = CAST(? AS BIGINT)",
    postgreSql = """
    |SELECT * FROM ExampleEntity2 WHERE uid = ?
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,uid)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow(null) {
        val _tmp_uid = _result.getLong("uid")
        val _tmp_name = _result.getString("name")
        val _tmp_someNumber = _result.getLong("someNumber")
        val _tmp_checked = _result.getBoolean("checked")
        val _tmp_rewardsCardNumber = _result.getIntNullable("rewardsCardNumber")
        ExampleEntity2().apply {
          this.uid = _tmp_uid
          this.name = _tmp_name
          this.someNumber = _tmp_someNumber
          this.checked = _tmp_checked
          this.rewardsCardNumber = _tmp_rewardsCardNumber
        }
      }
    }
  }

  override suspend fun findLarge(uid: Long, min: Long): ExampleEntity2? =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql =
        "SELECT * FROM ExampleEntity2 WHERE uid > CAST(? AS BIGINT) AND someNumber > CAST(? AS BIGINT)",
    postgreSql = """
    |SELECT * FROM ExampleEntity2 WHERE uid > ? AND someNumber > ?
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,uid)
    _stmt.setLong(2,min)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow(null) {
        val _tmp_uid = _result.getLong("uid")
        val _tmp_name = _result.getString("name")
        val _tmp_someNumber = _result.getLong("someNumber")
        val _tmp_checked = _result.getBoolean("checked")
        val _tmp_rewardsCardNumber = _result.getIntNullable("rewardsCardNumber")
        ExampleEntity2().apply {
          this.uid = _tmp_uid
          this.name = _tmp_name
          this.someNumber = _tmp_someNumber
          this.checked = _tmp_checked
          this.rewardsCardNumber = _tmp_rewardsCardNumber
        }
      }
    }
  }

  override suspend fun findLargeAsync(uid: Long, min: Long): List<ExampleEntity2> =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql =
        "SELECT * FROM ExampleEntity2 WHERE uid > CAST(? AS BIGINT) AND someNumber > CAST(? AS BIGINT)",
    postgreSql = """
    |SELECT * FROM ExampleEntity2 WHERE uid > ? AND someNumber > ?
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,uid)
    _stmt.setLong(2,min)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapRows {
        val _tmp_uid = _result.getLong("uid")
        val _tmp_name = _result.getString("name")
        val _tmp_someNumber = _result.getLong("someNumber")
        val _tmp_checked = _result.getBoolean("checked")
        val _tmp_rewardsCardNumber = _result.getIntNullable("rewardsCardNumber")
        ExampleEntity2().apply {
          this.uid = _tmp_uid
          this.name = _tmp_name
          this.someNumber = _tmp_someNumber
          this.checked = _tmp_checked
          this.rewardsCardNumber = _tmp_rewardsCardNumber
        }
      }
    }
  }

  override fun findWithNullableParam(name: String?): List<ExampleEntity2> =
      _db.prepareAndUseStatement(PreparedStatementConfig(
    sql = "SELECT * FROM ExampleEntity2 WHERE name = ?",
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setString(1,name)
    _stmt.executeQuery().useResults{ _result -> 
      _result.mapRows {
        val _tmp_uid = _result.getLong("uid")
        val _tmp_name = _result.getString("name")
        val _tmp_someNumber = _result.getLong("someNumber")
        val _tmp_checked = _result.getBoolean("checked")
        val _tmp_rewardsCardNumber = _result.getIntNullable("rewardsCardNumber")
        ExampleEntity2().apply {
          this.uid = _tmp_uid
          this.name = _tmp_name
          this.someNumber = _tmp_someNumber
          this.checked = _tmp_checked
          this.rewardsCardNumber = _tmp_rewardsCardNumber
        }
      }
    }
  }

  override fun findNameByUid(uid: Long): String? =
      _db.prepareAndUseStatement(PreparedStatementConfig(
    sql = "SELECT name FROM ExampleEntity2 WHERE uid = CAST(? AS BIGINT)",
    postgreSql = """
    |SELECT name FROM ExampleEntity2 WHERE uid = ?
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,uid)
    _stmt.executeQuery().useResults{ _result -> 
      _result.mapNextRow(null) {
        _result.getString(1)
      }
    }
  }

  override suspend fun findNameByUidAsync(uid: Long): String? =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = "SELECT name FROM ExampleEntity2 WHERE uid = CAST(? AS BIGINT)",
    postgreSql = """
    |SELECT name FROM ExampleEntity2 WHERE uid = ?
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,uid)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow(null) {
        _result.getString(1)
      }
    }
  }

  override suspend fun findByUidWithLinkEntityAsync(uid: Long): ExampleEntity2WithExampleLinkEntity?
      = _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql =
        "SELECT ExampleEntity2.*, ExampleLinkEntity.* FROM  ExampleEntity2 LEFT JOIN ExampleLinkEntity ON ExampleEntity2.uid = ExampleLinkEntity.fkValue WHERE ExampleEntity2.uid = CAST(? AS BIGINT)",
    postgreSql = """
    |SELECT ExampleEntity2.*, ExampleLinkEntity.* FROM  ExampleEntity2 LEFT JOIN ExampleLinkEntity ON ExampleEntity2.uid = ExampleLinkEntity.fkValue WHERE ExampleEntity2.uid = ?
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,uid)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow(null) {
        val _tmp_uid = _result.getLong("uid")
        val _tmp_name = _result.getString("name")
        val _tmp_someNumber = _result.getLong("someNumber")
        val _tmp_checked = _result.getBoolean("checked")
        val _tmp_rewardsCardNumber = _result.getIntNullable("rewardsCardNumber")
        var _tmp_ExampleLinkEntity_nullCount = 0
        val _tmp_eeUid = _result.getLong("eeUid")
        if(_result.wasNull()) _tmp_ExampleLinkEntity_nullCount++
        val _tmp_fkValue = _result.getLong("fkValue")
        if(_result.wasNull()) _tmp_ExampleLinkEntity_nullCount++
        val _tmp_ExampleLinkEntity_isAllNull = _tmp_ExampleLinkEntity_nullCount == 2
        ExampleEntity2WithExampleLinkEntity().apply {
          this.uid = _tmp_uid
          this.name = _tmp_name
          this.someNumber = _tmp_someNumber
          this.checked = _tmp_checked
          this.rewardsCardNumber = _tmp_rewardsCardNumber
          if(!_tmp_ExampleLinkEntity_isAllNull) {
            this.link = ExampleLinkEntity().apply {
              this.eeUid = _tmp_eeUid
              this.fkValue = _tmp_fkValue
            }
          }
        }
      }
    }
  }

  override fun findAll(): List<ExampleEntity2> = _db.prepareAndUseStatement(PreparedStatementConfig(
    sql = "SELECT * FROM ExampleEntity2 ORDER BY uid",
    readOnly = true,)
  ) { _stmt -> 
    _stmt.executeQuery().useResults{ _result -> 
      _result.mapRows {
        val _tmp_uid = _result.getLong("uid")
        val _tmp_name = _result.getString("name")
        val _tmp_someNumber = _result.getLong("someNumber")
        val _tmp_checked = _result.getBoolean("checked")
        val _tmp_rewardsCardNumber = _result.getIntNullable("rewardsCardNumber")
        ExampleEntity2().apply {
          this.uid = _tmp_uid
          this.name = _tmp_name
          this.someNumber = _tmp_someNumber
          this.checked = _tmp_checked
          this.rewardsCardNumber = _tmp_rewardsCardNumber
        }
      }
    }
  }

  override suspend fun findAllAsync(): List<ExampleEntity2> =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = "SELECT * FROM ExampleEntity2",
    readOnly = true,)
  ) { _stmt -> 
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapRows {
        val _tmp_uid = _result.getLong("uid")
        val _tmp_name = _result.getString("name")
        val _tmp_someNumber = _result.getLong("someNumber")
        val _tmp_checked = _result.getBoolean("checked")
        val _tmp_rewardsCardNumber = _result.getIntNullable("rewardsCardNumber")
        ExampleEntity2().apply {
          this.uid = _tmp_uid
          this.name = _tmp_name
          this.someNumber = _tmp_someNumber
          this.checked = _tmp_checked
          this.rewardsCardNumber = _tmp_rewardsCardNumber
        }
      }
    }
  }

  override suspend fun findAllStrings(): List<String?> =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = "SELECT name FROM ExampleEntity2",
    readOnly = true,)
  ) { _stmt -> 
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapRows {
        _result.getString(1)
      }
    }
  }

  override fun findAllStringsSync(): List<String?> =
      _db.prepareAndUseStatement(PreparedStatementConfig(
    sql = "SELECT name FROM ExampleEntity2",
    readOnly = true,)
  ) { _stmt -> 
    _stmt.executeQuery().useResults{ _result -> 
      _result.mapRows {
        _result.getString(1)
      }
    }
  }

  override fun updateByParam(newName: String, num: Long): Int =
      _db.prepareAndUseStatement(PreparedStatementConfig(
    sql = "UPDATE ExampleEntity2 SET name = ? WHERE someNumber >= CAST(? AS BIGINT)",
    postgreSql = """
    |UPDATE ExampleEntity2 SET name = ? WHERE someNumber >= ?
    |""".trimMargin(),
    readOnly = false,)
  ) { _stmt -> 
    _stmt.setString(1,newName)
    _stmt.setLong(2,num)
    _stmt.executeUpdate()
  }

  override suspend fun updateByParamAsync(newName: String, num: Long): Int =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = "UPDATE ExampleEntity2 SET name = ? WHERE someNumber >= CAST(? AS BIGINT)",
    postgreSql = """
    |UPDATE ExampleEntity2 SET name = ? WHERE someNumber >= ?
    |""".trimMargin(),
    readOnly = false,)
  ) { _stmt -> 
    _stmt.setString(1,newName)
    _stmt.setLong(2,num)
    _stmt.executeUpdateAsyncKmp()
  }

  override fun updateByParamNoReturn(newName: String, number: Long) {
    _db.prepareAndUseStatement(PreparedStatementConfig(
      sql = "UPDATE exampleentity2 SET name = ? WHERE someNumber >= CAST(? AS BIGINT)",
      postgreSql = """
      |UPDATE exampleentity2 SET name = ? WHERE someNumber >= ?
      |""".trimMargin(),
      readOnly = false,)
    ) { _stmt -> 
      _stmt.setString(1,newName)
      _stmt.setLong(2,number)
      _stmt.executeUpdate()
    }
  }

  override fun countNumEntities(): Int = _db.prepareAndUseStatement(PreparedStatementConfig(
    sql = "SELECT COUNT(*) FROM ExampleEntity2",
    readOnly = true,)
  ) { _stmt -> 
    _stmt.executeQuery().useResults{ _result -> 
      _result.mapNextRow(0) {
        _result.getInt(1)
      }
    }
  }

  override suspend fun countNumEntitiesAsync(): Int =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = "SELECT COUNT(*) FROM ExampleEntity2",
    readOnly = true,)
  ) { _stmt -> 
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow(0) {
        _result.getInt(1)
      }
    }
  }

  override fun queryUsingArray(uidList: List<Long>): List<ExampleEntity2> =
      _db.prepareAndUseStatement(PreparedStatementConfig(
    sql = "SELECT * FROM ExampleEntity2 WHERE uid IN (?)",
    hasListParams = true,
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setArray(1, _stmt.getConnection().createArrayOrProxyArrayOf("BIGINT",
        uidList.toTypedArray()))
    _stmt.executeQuery().useResults{ _result -> 
      _result.mapRows {
        val _tmp_uid = _result.getLong("uid")
        val _tmp_name = _result.getString("name")
        val _tmp_someNumber = _result.getLong("someNumber")
        val _tmp_checked = _result.getBoolean("checked")
        val _tmp_rewardsCardNumber = _result.getIntNullable("rewardsCardNumber")
        ExampleEntity2().apply {
          this.uid = _tmp_uid
          this.name = _tmp_name
          this.someNumber = _tmp_someNumber
          this.checked = _tmp_checked
          this.rewardsCardNumber = _tmp_rewardsCardNumber
        }
      }
    }
  }

  override fun rawQueryForList(query: DoorQuery): List<ExampleEntity2> =
      _db.prepareAndUseStatement(PreparedStatementConfig(query.sql, hasListParams =
      query.hasListOrArrayParams(), readOnly = true)
  ) { _stmt -> 
    query.bindToPreparedStmt(_stmt, _db)
    _stmt.executeQuery().useResults{ _result -> 
      _result.mapRows {
        val _tmp_uid = _result.getLong("uid")
        val _tmp_name = _result.getString("name")
        val _tmp_someNumber = _result.getLong("someNumber")
        val _tmp_checked = _result.getBoolean("checked")
        val _tmp_rewardsCardNumber = _result.getIntNullable("rewardsCardNumber")
        ExampleEntity2().apply {
          this.uid = _tmp_uid
          this.name = _tmp_name
          this.someNumber = _tmp_someNumber
          this.checked = _tmp_checked
          this.rewardsCardNumber = _tmp_rewardsCardNumber
        }
      }
    }
  }

  override suspend fun rawQueryForListAsyc(query: DoorQuery): List<ExampleEntity2> =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(query.sql, hasListParams =
      query.hasListOrArrayParams(), readOnly = true)
  ) { _stmt -> 
    query.bindToPreparedStmt(_stmt, _db)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapRows {
        val _tmp_uid = _result.getLong("uid")
        val _tmp_name = _result.getString("name")
        val _tmp_someNumber = _result.getLong("someNumber")
        val _tmp_checked = _result.getBoolean("checked")
        val _tmp_rewardsCardNumber = _result.getIntNullable("rewardsCardNumber")
        ExampleEntity2().apply {
          this.uid = _tmp_uid
          this.name = _tmp_name
          this.someNumber = _tmp_someNumber
          this.checked = _tmp_checked
          this.rewardsCardNumber = _tmp_rewardsCardNumber
        }
      }
    }
  }

  override fun rawQueryForListWithEmbeddedVals(query: DoorQuery):
      List<ExampleEntity2WithExampleLinkEntity> =
      _db.prepareAndUseStatement(PreparedStatementConfig(query.sql, hasListParams =
      query.hasListOrArrayParams(), readOnly = true)
  ) { _stmt -> 
    query.bindToPreparedStmt(_stmt, _db)
    _stmt.executeQuery().useResults{ _result -> 
      _result.mapRows {
        val _tmp_uid = _result.getLong("uid")
        val _tmp_name = _result.getString("name")
        val _tmp_someNumber = _result.getLong("someNumber")
        val _tmp_checked = _result.getBoolean("checked")
        val _tmp_rewardsCardNumber = _result.getIntNullable("rewardsCardNumber")
        var _tmp_ExampleLinkEntity_nullCount = 0
        val _tmp_eeUid = _result.getLong("eeUid")
        if(_result.wasNull()) _tmp_ExampleLinkEntity_nullCount++
        val _tmp_fkValue = _result.getLong("fkValue")
        if(_result.wasNull()) _tmp_ExampleLinkEntity_nullCount++
        val _tmp_ExampleLinkEntity_isAllNull = _tmp_ExampleLinkEntity_nullCount == 2
        ExampleEntity2WithExampleLinkEntity().apply {
          this.uid = _tmp_uid
          this.name = _tmp_name
          this.someNumber = _tmp_someNumber
          this.checked = _tmp_checked
          this.rewardsCardNumber = _tmp_rewardsCardNumber
          if(!_tmp_ExampleLinkEntity_isAllNull) {
            this.link = ExampleLinkEntity().apply {
              this.eeUid = _tmp_eeUid
              this.fkValue = _tmp_fkValue
            }
          }
        }
      }
    }
  }

  override fun rawQueryForSingleValue(query: DoorQuery): ExampleEntity2? =
      _db.prepareAndUseStatement(PreparedStatementConfig(query.sql, hasListParams =
      query.hasListOrArrayParams(), readOnly = true)
  ) { _stmt -> 
    query.bindToPreparedStmt(_stmt, _db)
    _stmt.executeQuery().useResults{ _result -> 
      _result.mapNextRow(null) {
        val _tmp_uid = _result.getLong("uid")
        val _tmp_name = _result.getString("name")
        val _tmp_someNumber = _result.getLong("someNumber")
        val _tmp_checked = _result.getBoolean("checked")
        val _tmp_rewardsCardNumber = _result.getIntNullable("rewardsCardNumber")
        ExampleEntity2().apply {
          this.uid = _tmp_uid
          this.name = _tmp_name
          this.someNumber = _tmp_someNumber
          this.checked = _tmp_checked
          this.rewardsCardNumber = _tmp_rewardsCardNumber
        }
      }
    }
  }

  override fun rawQueryWithArrParam(query: DoorQuery): List<ExampleEntity2> =
      _db.prepareAndUseStatement(PreparedStatementConfig(query.sql, hasListParams =
      query.hasListOrArrayParams(), readOnly = true)
  ) { _stmt -> 
    query.bindToPreparedStmt(_stmt, _db)
    _stmt.executeQuery().useResults{ _result -> 
      _result.mapRows {
        val _tmp_uid = _result.getLong("uid")
        val _tmp_name = _result.getString("name")
        val _tmp_someNumber = _result.getLong("someNumber")
        val _tmp_checked = _result.getBoolean("checked")
        val _tmp_rewardsCardNumber = _result.getIntNullable("rewardsCardNumber")
        ExampleEntity2().apply {
          this.uid = _tmp_uid
          this.name = _tmp_name
          this.someNumber = _tmp_someNumber
          this.checked = _tmp_checked
          this.rewardsCardNumber = _tmp_rewardsCardNumber
        }
      }
    }
  }

  override fun insertFromSelectQuery(uid: Int) {
    _db.prepareAndUseStatement(PreparedStatementConfig(
      sql = """
      |
      |   INSERT INTO ExampleEntity2(uid, name, someNumber)
      |        SELECT (uid + 1) AS uid, name AS name, (someNumber * 2) AS someNumber
      |          FROM ExampleEntity2
      |         WHERE uid = ? 
      |    
      """.trimMargin(),
      readOnly = false,)
    ) { _stmt -> 
      _stmt.setInt(1,uid)
      _stmt.executeUpdate()
    }
  }

  override fun queryAllAsFlow(): Flow<List<ExampleEntity2>> =
      _db.doorFlow(arrayOf("ExampleEntity2")) {
    _db.prepareAndUseStatementAsync(PreparedStatementConfig(
      sql = "SELECT * FROM ExampleEntity2",
      readOnly = true,)
    ) { _stmt -> 
      _stmt.executeQueryAsyncKmp().useResults{ _result -> 
        _result.mapRows {
          val _tmp_uid = _result.getLong("uid")
          val _tmp_name = _result.getString("name")
          val _tmp_someNumber = _result.getLong("someNumber")
          val _tmp_checked = _result.getBoolean("checked")
          val _tmp_rewardsCardNumber = _result.getIntNullable("rewardsCardNumber")
          ExampleEntity2().apply {
            this.uid = _tmp_uid
            this.name = _tmp_name
            this.someNumber = _tmp_someNumber
            this.checked = _tmp_checked
            this.rewardsCardNumber = _tmp_rewardsCardNumber
          }
        }
      }
    }
  }

  override fun findSingleNotNullableEntity(greaterThan: Int): ExampleEntity2 =
      _db.prepareAndUseStatement(PreparedStatementConfig(
    sql = "SELECT * FROM ExampleEntity2 WHERE someNumber > ? LIMIT 1",
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setInt(1,greaterThan)
    _stmt.executeQuery().useResults{ _result -> 
      _result.mapNextRow() {
        val _tmp_uid = _result.getLong("uid")
        val _tmp_name = _result.getString("name")
        val _tmp_someNumber = _result.getLong("someNumber")
        val _tmp_checked = _result.getBoolean("checked")
        val _tmp_rewardsCardNumber = _result.getIntNullable("rewardsCardNumber")
        ExampleEntity2().apply {
          this.uid = _tmp_uid
          this.name = _tmp_name
          this.someNumber = _tmp_someNumber
          this.checked = _tmp_checked
          this.rewardsCardNumber = _tmp_rewardsCardNumber
        }
      }
    }
  }

  override suspend fun findSingleNotNullableEntityAsync(greaterThan: Int): ExampleEntity2 =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = "SELECT * FROM ExampleEntity2 WHERE someNumber > ? LIMIT 1",
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setInt(1,greaterThan)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow() {
        val _tmp_uid = _result.getLong("uid")
        val _tmp_name = _result.getString("name")
        val _tmp_someNumber = _result.getLong("someNumber")
        val _tmp_checked = _result.getBoolean("checked")
        val _tmp_rewardsCardNumber = _result.getIntNullable("rewardsCardNumber")
        ExampleEntity2().apply {
          this.uid = _tmp_uid
          this.name = _tmp_name
          this.someNumber = _tmp_someNumber
          this.checked = _tmp_checked
          this.rewardsCardNumber = _tmp_rewardsCardNumber
        }
      }
    }
  }

  override fun findSingleNotNullablePrimitive(greaterThan: Int): Int =
      _db.prepareAndUseStatement(PreparedStatementConfig(
    sql = "SELECT someNumber FROM ExampleEntity2 WHERE someNumber > ? LIMIT 1",
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setInt(1,greaterThan)
    _stmt.executeQuery().useResults{ _result -> 
      _result.mapNextRow(0) {
        _result.getInt(1)
      }
    }
  }

  override suspend fun findSingleNotNullablePrimitiveAsync(greaterThan: Int): Int =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = "SELECT someNumber FROM ExampleEntity2 WHERE someNumber > ? LIMIT 1",
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setInt(1,greaterThan)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow(0) {
        _result.getInt(1)
      }
    }
  }

  override fun findSingleNullablePrimitive(greaterThan: Int): Int? =
      _db.prepareAndUseStatement(PreparedStatementConfig(
    sql = "SELECT someNumber FROM ExampleEntity2 WHERE someNumber > ? LIMIT 1",
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setInt(1,greaterThan)
    _stmt.executeQuery().useResults{ _result -> 
      _result.mapNextRow(null) {
        _result.getIntNullable(1)
      }
    }
  }

  override suspend fun findSingleNullablePrimitiveAsync(greaterThan: Long): Long? =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = "SELECT someNumber FROM ExampleEntity2 WHERE someNumber > CAST(? AS BIGINT) LIMIT 1",
    postgreSql = """
    |SELECT someNumber FROM ExampleEntity2 WHERE someNumber > ? LIMIT 1
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,greaterThan)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow(null) {
        _result.getLongNullable(1)
      }
    }
  }

  override suspend fun findWithNullableIntAsync(rewardsNum: Int?): ExampleEntity2? =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT ExampleEntity2.* FROM ExampleEntity2
    |         WHERE (CASE
    |                WHEN ? IS NULL THEN (ExampleEntity2.rewardsCardNumber IS NULL)
    |                ELSE (ExampleEntity2.rewardsCardNumber = ?) 
    |                END)
    |    
    """.trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setIntNullable(1,rewardsNum)
    _stmt.setIntNullable(2,rewardsNum)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow(null) {
        val _tmp_uid = _result.getLong("uid")
        val _tmp_name = _result.getString("name")
        val _tmp_someNumber = _result.getLong("someNumber")
        val _tmp_checked = _result.getBoolean("checked")
        val _tmp_rewardsCardNumber = _result.getIntNullable("rewardsCardNumber")
        ExampleEntity2().apply {
          this.uid = _tmp_uid
          this.name = _tmp_name
          this.someNumber = _tmp_someNumber
          this.checked = _tmp_checked
          this.rewardsCardNumber = _tmp_rewardsCardNumber
        }
      }
    }
  }

  override fun findAllWithRewardNumberAsPagingSource(minNumber: Int):
      PagingSource<Int, ExampleEntity2> = object : DoorLimitOffsetPagingSource<ExampleEntity2>(db =
      _db
  , tableNames = arrayOf("ExampleEntity2")
  ) {
    override suspend fun loadRows(_limit: Int, _offset: Int): List<ExampleEntity2> =
        _db.prepareAndUseStatementAsync(PreparedStatementConfig(
      sql = """
      |SELECT * FROM (
      |        SELECT * 
      |          FROM ExampleEntity2
      |         WHERE rewardsCardNumber IS NOT NULL
      |           AND rewardsCardNumber >= ? 
      |      ORDER BY rewardsCardNumber    
      |    ) AS _PagingData LIMIT ? OFFSET ?
      """.trimMargin(),
      readOnly = true,)
    ) { _stmt -> 
      _stmt.setInt(1,minNumber)
      _stmt.setInt(2,_limit)
      _stmt.setInt(3,_offset)
      _stmt.executeQueryAsyncKmp().useResults{ _result -> 
        _result.mapRows {
          val _tmp_uid = _result.getLong("uid")
          val _tmp_name = _result.getString("name")
          val _tmp_someNumber = _result.getLong("someNumber")
          val _tmp_checked = _result.getBoolean("checked")
          val _tmp_rewardsCardNumber = _result.getIntNullable("rewardsCardNumber")
          ExampleEntity2().apply {
            this.uid = _tmp_uid
            this.name = _tmp_name
            this.someNumber = _tmp_someNumber
            this.checked = _tmp_checked
            this.rewardsCardNumber = _tmp_rewardsCardNumber
          }
        }
      }
    }

    override suspend fun countRows(): Int = _db.prepareAndUseStatementAsync(PreparedStatementConfig(
      sql = """
      |SELECT COUNT(*) FROM (
      |        SELECT * 
      |          FROM ExampleEntity2
      |         WHERE rewardsCardNumber IS NOT NULL
      |           AND rewardsCardNumber >= ? 
      |      ORDER BY rewardsCardNumber    
      |    ) AS _PagingCount
      """.trimMargin(),
      readOnly = true,)
    ) { _stmt -> 
      _stmt.setInt(1,minNumber)
      _stmt.executeQueryAsyncKmp().useResults{ _result -> 
        _result.mapNextRow(0) {
          _result.getInt(1)
        }
      }
    }
  }

  override suspend fun findAllWithRewardNumberAsListAsync(minNumber: Int): List<ExampleEntity2> =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT * 
    |          FROM ExampleEntity2
    |         WHERE rewardsCardNumber IS NOT NULL
    |           AND rewardsCardNumber >= ? 
    |      ORDER BY rewardsCardNumber    
    |    
    """.trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setInt(1,minNumber)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapRows {
        val _tmp_uid = _result.getLong("uid")
        val _tmp_name = _result.getString("name")
        val _tmp_someNumber = _result.getLong("someNumber")
        val _tmp_checked = _result.getBoolean("checked")
        val _tmp_rewardsCardNumber = _result.getIntNullable("rewardsCardNumber")
        ExampleEntity2().apply {
          this.uid = _tmp_uid
          this.name = _tmp_name
          this.someNumber = _tmp_someNumber
          this.checked = _tmp_checked
          this.rewardsCardNumber = _tmp_rewardsCardNumber
        }
      }
    }
  }

  override suspend fun findByMinSomeNumber(minNumber: Long): List<ExampleEntity2> =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT * 
    |          FROM ExampleEntity2
    |         WHERE someNumber >= CAST(? AS BIGINT) 
    |    
    """.trimMargin(),
    postgreSql = """
    |
    |        SELECT * 
    |          FROM ExampleEntity2
    |         WHERE someNumber >= ? 
    |    
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,minNumber)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapRows {
        val _tmp_uid = _result.getLong("uid")
        val _tmp_name = _result.getString("name")
        val _tmp_someNumber = _result.getLong("someNumber")
        val _tmp_checked = _result.getBoolean("checked")
        val _tmp_rewardsCardNumber = _result.getIntNullable("rewardsCardNumber")
        ExampleEntity2().apply {
          this.uid = _tmp_uid
          this.name = _tmp_name
          this.someNumber = _tmp_someNumber
          this.checked = _tmp_checked
          this.rewardsCardNumber = _tmp_rewardsCardNumber
        }
      }
    }
  }
}
