package db2

import com.ustadmobile.door.DoorDatabaseJdbc
import com.ustadmobile.door.DoorPrimaryKeyManager
import com.ustadmobile.door.ext.doorDatabaseMetadata
import com.ustadmobile.door.ext.mutableLinkedListOf
import com.ustadmobile.door.ext.nodeIdAuthCache
import com.ustadmobile.door.ext.rootDatabase
import com.ustadmobile.door.jdbc.DataSource
import com.ustadmobile.door.log.DoorLogger
import com.ustadmobile.door.room.InvalidationTracker
import com.ustadmobile.door.room.RoomDatabase
import com.ustadmobile.door.room.RoomDatabaseJdbcImplHelper
import com.ustadmobile.door.room.RoomJdbcImpl
import com.ustadmobile.door.util.NodeIdAuthCache
import com.ustadmobile.door.util.systemTimeInMillis
import kotlin.IllegalStateException
import kotlin.Int
import kotlin.String
import kotlin.Unit
import kotlin.collections.List
import kotlin.collections.toTypedArray

public class ExampleDatabase2_JdbcImpl(
  override val doorJdbcSourceDatabase: RoomDatabase?,
  override val dataSource: DataSource,
  dbUrl: String,
  override val dbName: String,
  override val jdbcQueryTimeout: Int,
  jdbcDbType: Int,
  logger: DoorLogger,
) : ExampleDatabase2(), DoorDatabaseJdbc, RoomJdbcImpl {
  override val dbVersion: Int
    get() = 2

  override val jdbcImplHelper: RoomDatabaseJdbcImplHelper = RoomDatabaseJdbcImplHelper(
        dataSource = dataSource,
        db = this,
        dbUrl = dbUrl,
        dbName = dbName,
        logger = logger,
        tableNames = this::class.doorDatabaseMetadata().allTables,
        invalidationTracker =
            InvalidationTracker(*this::class.doorDatabaseMetadata().allTables.toTypedArray()),
        dbType = jdbcDbType,
      )


  override val realNodeIdAuthCache: NodeIdAuthCache by lazy {
        if(this == rootDatabase) {
          val nodeIdAuthCache = NodeIdAuthCache(this)
          nodeIdAuthCache
        } else {
          rootDatabase.nodeIdAuthCache
        }
      }


  override val realPrimaryKeyManager: DoorPrimaryKeyManager by lazy {
        DoorPrimaryKeyManager(ExampleDatabase2::class.doorDatabaseMetadata().replicateEntities.keys)
      }


  override val invalidationTracker: InvalidationTracker
    get() = jdbcImplHelper.invalidationTracker

  public val _ExampleSyncableDao: ExampleSyncableDao_JdbcImpl by
      lazy { ExampleSyncableDao_JdbcImpl(this) }

  public val _ExampleDao2: ExampleDao2_JdbcImpl by lazy { ExampleDao2_JdbcImpl(this) }

  public val _ExampleLinkEntityDao: ExampleLinkEntityDao_JdbcImpl by
      lazy { ExampleLinkEntityDao_JdbcImpl(this) }

  public val _ExampleDaoWithInterface: ExampleDaoWithInterface_JdbcImpl by
      lazy { ExampleDaoWithInterface_JdbcImpl(this) }

  public val _ExampleEntityPkIntDao: ExampleEntityPkIntDao_JdbcImpl by
      lazy { ExampleEntityPkIntDao_JdbcImpl(this) }

  public val _AccessGrantDao: AccessGrantDao_JdbcImpl by lazy { AccessGrantDao_JdbcImpl(this) }

  override fun createAllTables(): List<String> {
    val _stmtList = mutableLinkedListOf<String>()
    when(jdbcImplHelper.dbType) {
      1 ->  {
        // - create for this SQLite 
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS _doorwayinfo (dbVersion int primary key, dbHash varchar(255))"
         _stmtList += "INSERT INTO _doorwayinfo VALUES (2, '')"
        //Begin: Create table ExampleEntity2 for SQLite
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE ExampleEntity2 RENAME to ExampleEntity2_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS ExampleEntity2 (  name  TEXT , someNumber  INTEGER  NOT NULL , checked  INTEGER  NOT NULL , rewardsCardNumber  INTEGER , uid  INTEGER  PRIMARY KEY  AUTOINCREMENT  NOT NULL )"
        _stmtList += "CREATE INDEX name_checked_idx ON ExampleEntity2 (name, checked)"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO ExampleEntity2 (name, someNumber, checked, rewardsCardNumber, uid) SELECT name, someNumber, checked, rewardsCardNumber, uid FROM ExampleEntity2_OLD")
        _stmt.executeUpdate("DROP TABLE ExampleEntity2_OLD")
        END MIGRATION*/
        //End: Create table ExampleEntity2 for SQLite

        //Begin: Create table ExampleLinkEntity for SQLite
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE ExampleLinkEntity RENAME to ExampleLinkEntity_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS ExampleLinkEntity (  eeUid  INTEGER  PRIMARY KEY  NOT NULL , fkValue  INTEGER  NOT NULL )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO ExampleLinkEntity (eeUid, fkValue) SELECT eeUid, fkValue FROM ExampleLinkEntity_OLD")
        _stmt.executeUpdate("DROP TABLE ExampleLinkEntity_OLD")
        END MIGRATION*/
        //End: Create table ExampleLinkEntity for SQLite

        //Begin: Create table ExampleEntityPkInt for SQLite
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE ExampleEntityPkInt RENAME to ExampleEntityPkInt_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS ExampleEntityPkInt (  str  TEXT , pk  INTEGER  PRIMARY KEY  AUTOINCREMENT  NOT NULL )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO ExampleEntityPkInt (str, pk) SELECT str, pk FROM ExampleEntityPkInt_OLD")
        _stmt.executeUpdate("DROP TABLE ExampleEntityPkInt_OLD")
        END MIGRATION*/
        //End: Create table ExampleEntityPkInt for SQLite

        //Begin: Create table SyncNode for SQLite
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE SyncNode RENAME to SyncNode_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS SyncNode (  nodeClientId  INTEGER  NOT NULL , PRIMARY KEY (nodeClientId) )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO SyncNode (nodeClientId) SELECT nodeClientId FROM SyncNode_OLD")
        _stmt.executeUpdate("DROP TABLE SyncNode_OLD")
        END MIGRATION*/
        //End: Create table SyncNode for SQLite

        //Begin: Create table ExampleSyncableEntity for SQLite
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE ExampleSyncableEntity RENAME to ExampleSyncableEntity_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS ExampleSyncableEntity (  esLcsn  INTEGER  NOT NULL , esMcsn  INTEGER  NOT NULL , esLcb  INTEGER  NOT NULL , esNumber  INTEGER  NOT NULL , esName  TEXT , publik  INTEGER  NOT NULL , esUid  INTEGER  PRIMARY KEY  AUTOINCREMENT  NOT NULL )"
        _stmtList +=
            "CREATE INDEX index_ExampleSyncableEntity_esNumber_esName ON ExampleSyncableEntity (esNumber, esName)"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO ExampleSyncableEntity (esLcsn, esMcsn, esLcb, esNumber, esName, publik, esUid) SELECT esLcsn, esMcsn, esLcb, esNumber, esName, publik, esUid FROM ExampleSyncableEntity_OLD")
        _stmt.executeUpdate("DROP TABLE ExampleSyncableEntity_OLD")
        END MIGRATION*/
        //End: Create table ExampleSyncableEntity for SQLite

        //Begin: Create table OtherSyncableEntity for SQLite
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE OtherSyncableEntity RENAME to OtherSyncableEntity_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS OtherSyncableEntity (  osLcb  INTEGER  NOT NULL , osMcsn  INTEGER  NOT NULL , osLcsn  INTEGER  NOT NULL , otherFk  INTEGER  NOT NULL , otherNum  INTEGER  NOT NULL , otherStr  TEXT , osUid  INTEGER  PRIMARY KEY  AUTOINCREMENT  NOT NULL )"
        _stmtList += "CREATE INDEX index_other_fk_to_num ON OtherSyncableEntity (otherFk, otherNum)"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO OtherSyncableEntity (osLcb, osMcsn, osLcsn, otherFk, otherNum, otherStr, osUid) SELECT osLcb, osMcsn, osLcsn, otherFk, otherNum, otherStr, osUid FROM OtherSyncableEntity_OLD")
        _stmt.executeUpdate("DROP TABLE OtherSyncableEntity_OLD")
        END MIGRATION*/
        //End: Create table OtherSyncableEntity for SQLite

        //Begin: Create table AccessGrant for SQLite
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE AccessGrant RENAME to AccessGrant_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS AccessGrant (  aLcsn  INTEGER  NOT NULL , aPcsn  INTEGER  NOT NULL , aLcb  INTEGER  NOT NULL , deviceId  INTEGER  NOT NULL , tableId  INTEGER  NOT NULL , entityUid  INTEGER  NOT NULL , accessId  INTEGER  PRIMARY KEY  AUTOINCREMENT  NOT NULL )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO AccessGrant (aLcsn, aPcsn, aLcb, deviceId, tableId, entityUid, accessId) SELECT aLcsn, aPcsn, aLcb, deviceId, tableId, entityUid, accessId FROM AccessGrant_OLD")
        _stmt.executeUpdate("DROP TABLE AccessGrant_OLD")
        END MIGRATION*/
        //End: Create table AccessGrant for SQLite

        //Begin: Create table DoorNode for SQLite
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE DoorNode RENAME to DoorNode_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS DoorNode (  auth  TEXT , nodeId  INTEGER  NOT NULL , rel  INTEGER  NOT NULL  DEFAULT 2 , PRIMARY KEY (nodeId) )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO DoorNode (auth, nodeId, rel) SELECT auth, nodeId, rel FROM DoorNode_OLD")
        _stmt.executeUpdate("DROP TABLE DoorNode_OLD")
        END MIGRATION*/
        //End: Create table DoorNode for SQLite

      }
      2 ->  {
        // - create for this PostgreSQL 
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS _doorwayinfo (dbVersion int primary key, dbHash varchar(255))"
         _stmtList += "INSERT INTO _doorwayinfo VALUES (2, '')"
        //Begin: Create table ExampleEntity2 for PostgreSQL
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE ExampleEntity2 RENAME to ExampleEntity2_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS ExampleEntity2 (  name  TEXT , someNumber  BIGINT  NOT NULL , checked  BOOL  NOT NULL , rewardsCardNumber  INTEGER , uid  BIGSERIAL  PRIMARY KEY  NOT NULL )"
        _stmtList += "CREATE INDEX name_checked_idx ON ExampleEntity2 (name, checked)"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO ExampleEntity2 (name, someNumber, checked, rewardsCardNumber, uid) SELECT name, someNumber, checked, rewardsCardNumber, uid FROM ExampleEntity2_OLD")
        _stmt.executeUpdate("DROP TABLE ExampleEntity2_OLD")
        END MIGRATION*/
        //End: Create table ExampleEntity2 for PostgreSQL

        //Begin: Create table ExampleLinkEntity for PostgreSQL
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE ExampleLinkEntity RENAME to ExampleLinkEntity_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS ExampleLinkEntity (  eeUid  BIGINT  PRIMARY KEY  NOT NULL , fkValue  BIGINT  NOT NULL )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO ExampleLinkEntity (eeUid, fkValue) SELECT eeUid, fkValue FROM ExampleLinkEntity_OLD")
        _stmt.executeUpdate("DROP TABLE ExampleLinkEntity_OLD")
        END MIGRATION*/
        //End: Create table ExampleLinkEntity for PostgreSQL

        //Begin: Create table ExampleEntityPkInt for PostgreSQL
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE ExampleEntityPkInt RENAME to ExampleEntityPkInt_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS ExampleEntityPkInt (  str  TEXT , pk  SERIAL  PRIMARY KEY  NOT NULL )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO ExampleEntityPkInt (str, pk) SELECT str, pk FROM ExampleEntityPkInt_OLD")
        _stmt.executeUpdate("DROP TABLE ExampleEntityPkInt_OLD")
        END MIGRATION*/
        //End: Create table ExampleEntityPkInt for PostgreSQL

        //Begin: Create table SyncNode for PostgreSQL
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE SyncNode RENAME to SyncNode_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS SyncNode (  nodeClientId  BIGINT  NOT NULL , PRIMARY KEY (nodeClientId) )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO SyncNode (nodeClientId) SELECT nodeClientId FROM SyncNode_OLD")
        _stmt.executeUpdate("DROP TABLE SyncNode_OLD")
        END MIGRATION*/
        //End: Create table SyncNode for PostgreSQL

        //Begin: Create table ExampleSyncableEntity for PostgreSQL
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE ExampleSyncableEntity RENAME to ExampleSyncableEntity_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS ExampleSyncableEntity (  esLcsn  INTEGER  NOT NULL , esMcsn  INTEGER  NOT NULL , esLcb  INTEGER  NOT NULL , esNumber  INTEGER  NOT NULL , esName  TEXT , publik  BOOL  NOT NULL , esUid  BIGSERIAL  PRIMARY KEY  NOT NULL )"
        _stmtList +=
            "CREATE INDEX index_ExampleSyncableEntity_esNumber_esName ON ExampleSyncableEntity (esNumber, esName)"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO ExampleSyncableEntity (esLcsn, esMcsn, esLcb, esNumber, esName, publik, esUid) SELECT esLcsn, esMcsn, esLcb, esNumber, esName, publik, esUid FROM ExampleSyncableEntity_OLD")
        _stmt.executeUpdate("DROP TABLE ExampleSyncableEntity_OLD")
        END MIGRATION*/
        //End: Create table ExampleSyncableEntity for PostgreSQL

        //Begin: Create table OtherSyncableEntity for PostgreSQL
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE OtherSyncableEntity RENAME to OtherSyncableEntity_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS OtherSyncableEntity (  osLcb  INTEGER  NOT NULL , osMcsn  INTEGER  NOT NULL , osLcsn  INTEGER  NOT NULL , otherFk  INTEGER  NOT NULL , otherNum  INTEGER  NOT NULL , otherStr  TEXT , osUid  BIGSERIAL  PRIMARY KEY  NOT NULL )"
        _stmtList += "CREATE INDEX index_other_fk_to_num ON OtherSyncableEntity (otherFk, otherNum)"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO OtherSyncableEntity (osLcb, osMcsn, osLcsn, otherFk, otherNum, otherStr, osUid) SELECT osLcb, osMcsn, osLcsn, otherFk, otherNum, otherStr, osUid FROM OtherSyncableEntity_OLD")
        _stmt.executeUpdate("DROP TABLE OtherSyncableEntity_OLD")
        END MIGRATION*/
        //End: Create table OtherSyncableEntity for PostgreSQL

        //Begin: Create table AccessGrant for PostgreSQL
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE AccessGrant RENAME to AccessGrant_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS AccessGrant (  aLcsn  INTEGER  NOT NULL , aPcsn  INTEGER  NOT NULL , aLcb  INTEGER  NOT NULL , deviceId  INTEGER  NOT NULL , tableId  INTEGER  NOT NULL , entityUid  BIGINT  NOT NULL , accessId  BIGSERIAL  PRIMARY KEY  NOT NULL )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO AccessGrant (aLcsn, aPcsn, aLcb, deviceId, tableId, entityUid, accessId) SELECT aLcsn, aPcsn, aLcb, deviceId, tableId, entityUid, accessId FROM AccessGrant_OLD")
        _stmt.executeUpdate("DROP TABLE AccessGrant_OLD")
        END MIGRATION*/
        //End: Create table AccessGrant for PostgreSQL

        //Begin: Create table DoorNode for PostgreSQL
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE DoorNode RENAME to DoorNode_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS DoorNode (  auth  TEXT , nodeId  BIGINT  NOT NULL , rel  INTEGER  NOT NULL  DEFAULT 2 , PRIMARY KEY (nodeId) )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO DoorNode (auth, nodeId, rel) SELECT auth, nodeId, rel FROM DoorNode_OLD")
        _stmt.executeUpdate("DROP TABLE DoorNode_OLD")
        END MIGRATION*/
        //End: Create table DoorNode for PostgreSQL

      }
    }
    return _stmtList
  }

  public fun makeClearAllTablesSql(): List<String> {
    val _stmtList = mutableListOf<String>()
    _stmtList += "DELETE FROM ExampleEntity2"
    _stmtList += "DELETE FROM ExampleLinkEntity"
    _stmtList += "DELETE FROM ExampleEntityPkInt"
    _stmtList += "DELETE FROM SyncNode"
    _stmtList += "DELETE FROM ExampleSyncableEntity"
    _stmtList += "DELETE FROM OtherSyncableEntity"
    _stmtList += "DELETE FROM AccessGrant"
    _stmtList += "DELETE FROM DoorNode"
    return _stmtList
  }

  override fun clearAllTables(): Unit = throw
      IllegalStateException("clearAllTables synchronous not supported on Javascript")

  override suspend fun clearAllTablesAsync() {
    execSQLBatchAsyncJs(*makeClearAllTablesSql().toTypedArray())
  }

  override fun exampleSyncableDao(): ExampleSyncableDao = _ExampleSyncableDao

  override fun exampleDao2(): ExampleDao2 = _ExampleDao2

  override fun exampleLinkedEntityDao(): ExampleLinkEntityDao = _ExampleLinkEntityDao

  override fun examlpeDaoWithInterface(): ExampleDaoWithInterface = _ExampleDaoWithInterface

  override fun exampleEntityPkIntDao(): ExampleEntityPkIntDao = _ExampleEntityPkIntDao

  override fun accessGrantDao(): AccessGrantDao = _AccessGrantDao

  override fun close() {
    jdbcImplHelper.close()
  }
}
