package db2

import com.ustadmobile.door.DoorDatabaseRepository
import com.ustadmobile.door.RepositoryConfig
import com.ustadmobile.door.RepositoryHelper
import com.ustadmobile.door.replication.DoorRepositoryReplicationClient
import com.ustadmobile.door.room.InvalidationTracker
import io.ktor.client.HttpClient
import kotlin.Any
import kotlin.IllegalStateException
import kotlin.Int
import kotlin.Long
import kotlin.String
import kotlin.Suppress
import kotlin.Unit
import kotlin.collections.List
import kotlinx.coroutines.flow.Flow

@Suppress("LocalVariableName", "PropertyName", "FunctionName", "ClassName")
public class ExampleDatabase2_Repo(
  override val db: ExampleDatabase2,
  dbUnwrapped: ExampleDatabase2,
  override val config: RepositoryConfig,
) : ExampleDatabase2(), DoorDatabaseRepository {
  public val context: Any
    get() = config.context

  private val _db: ExampleDatabase2 = dbUnwrapped

  private val _endpoint: String
    get() = config.endpoint

  public val _httpClient: HttpClient
    get() = config.httpClient

  public val _repositoryHelper: RepositoryHelper = RepositoryHelper(db, config)


  public val clientId: Long
    get() = config.nodeId

  override val dbName: String
    get() = "Repository for [${_db.toString()}] - ${config.endpoint}"

  override var connectivityStatus: Int
    get() = _repositoryHelper.connectivityStatus
    set(newValue) {
      _repositoryHelper.connectivityStatus = newValue
    }

  override val clientState: Flow<DoorRepositoryReplicationClient.ClientState>
    get() = _repositoryHelper.clientState

  override val invalidationTracker: InvalidationTracker
    get() = _db.invalidationTracker

  override val dbVersion: Int
    get() = 2

  private val _ExampleSyncableDao: ExampleSyncableDao_Repo by lazy {
        ExampleSyncableDao_Repo(db, this, db.exampleSyncableDao(), _httpClient, clientId, _endpoint)
      }


  private val _AccessGrantDao: AccessGrantDao_Repo by lazy {
        AccessGrantDao_Repo(db, this, db.accessGrantDao(), _httpClient, clientId, _endpoint)
      }


  override fun clearAllTables(): Unit = throw
      IllegalStateException("Cannot use a repository to clearAllTables!")

  override suspend fun clearAllTablesAsync(): Unit = throw
      IllegalStateException("Cannot use this to run clearAllTablesAsync")

  override fun close() {
    _repositoryHelper.close()
  }

  override fun remoteNodeIdOrNull(): Long? = _repositoryHelper.remoteNodeIdOrNull()

  override fun remoteNodeIdOrFake(): Long = _repositoryHelper.remoteNodeIdOrFake()

  override fun createAllTables(): List<String> = throw
      IllegalStateException("Cannot use a repository to createAllTables!")

  override fun exampleSyncableDao(): ExampleSyncableDao =  _ExampleSyncableDao

  override fun exampleDao2(): ExampleDao2 = throw
      IllegalStateException("ExampleDao2 is not annotated with @Repository")

  override fun exampleLinkedEntityDao(): ExampleLinkEntityDao = throw
      IllegalStateException("ExampleLinkEntityDao is not annotated with @Repository")

  override fun examlpeDaoWithInterface(): ExampleDaoWithInterface = throw
      IllegalStateException("ExampleDaoWithInterface is not annotated with @Repository")

  override fun exampleEntityPkIntDao(): ExampleEntityPkIntDao = throw
      IllegalStateException("ExampleEntityPkIntDao is not annotated with @Repository")

  override fun accessGrantDao(): AccessGrantDao =  _AccessGrantDao

  public companion object {
    public const val _DB_NAME: String = "ExampleDatabase2"
  }
}
