package db2

import com.ustadmobile.door.DoorDbType
import com.ustadmobile.door.EntityInsertionAdapter
import com.ustadmobile.door.PreparedStatementConfig
import com.ustadmobile.door.SyncNode
import com.ustadmobile.door.ext.createArrayOrProxyArrayOf
import com.ustadmobile.door.ext.prepareAndUseStatement
import com.ustadmobile.door.ext.prepareAndUseStatementAsync
import com.ustadmobile.door.jdbc.PreparedStatement
import com.ustadmobile.door.jdbc.ext.executeQueryAsyncKmp
import com.ustadmobile.door.jdbc.ext.executeUpdateAsyncKmp
import com.ustadmobile.door.jdbc.ext.mapNextRow
import com.ustadmobile.door.jdbc.ext.mapRows
import com.ustadmobile.door.jdbc.ext.useResults
import com.ustadmobile.door.room.RoomDatabase
import kotlin.Boolean
import kotlin.Int
import kotlin.Long
import kotlin.String
import kotlin.collections.List

public class ExampleSyncableDao_JdbcImpl(
  public val _db: RoomDatabase,
) : ExampleSyncableDao() {
  public val _insertAdapterExampleSyncableEntity_abort:
      EntityInsertionAdapter<ExampleSyncableEntity> = object :
      EntityInsertionAdapter<ExampleSyncableEntity>(_db) {
    override fun makeSql(returnsId: Boolean): String =
        "INSERT INTO ExampleSyncableEntity (esUid, esLcsn, esMcsn, esLcb, esNumber, esName, publik) VALUES(?, ?, ?, ?, ?, ?, ?)"

    override fun bindPreparedStmtToEntity(stmt: PreparedStatement, entity: ExampleSyncableEntity) {
      if(entity.esUid == 0L) {
        stmt.setObject(1, null)
      } else {
        stmt.setLong(1, entity.esUid)
      }
      stmt.setInt(2, entity.esLcsn)
      stmt.setInt(3, entity.esMcsn)
      stmt.setInt(4, entity.esLcb)
      stmt.setInt(5, entity.esNumber)
      stmt.setString(6, entity.esName)
      stmt.setBoolean(7, entity.publik)
    }
  }

  override fun insert(syncableEntity: ExampleSyncableEntity): Long {
    val _retVal = _insertAdapterExampleSyncableEntity_abort.insertAndReturnId(syncableEntity)
    return _retVal
  }

  override fun insertList(syncableEntityLIst: List<ExampleSyncableEntity>) {
    _insertAdapterExampleSyncableEntity_abort.insertList(syncableEntityLIst)
  }

  override suspend fun insertAndReturnList(entities: List<ExampleSyncableEntity>): List<Long> {
    val _retVal = _insertAdapterExampleSyncableEntity_abort.insertListAndReturnIdsAsync(entities)
    return _retVal
  }

  override suspend fun insertAsync(syncableEntity: ExampleSyncableEntity): Long {
    val _retVal = _insertAdapterExampleSyncableEntity_abort.insertAndReturnIdAsync(syncableEntity)
    return _retVal
  }

  override suspend fun updateAsync(exampleSyncableEntity: ExampleSyncableEntity) {
    val _sql =
        "UPDATE ExampleSyncableEntity SET esLcsn = ?, esMcsn = ?, esLcb = ?, esNumber = ?, esName = ?, publik = ? WHERE esUid = ?"
    _db.prepareAndUseStatementAsync(_sql) {
       _stmt ->
      _stmt.setInt(1, exampleSyncableEntity.esLcsn)
      _stmt.setInt(2, exampleSyncableEntity.esMcsn)
      _stmt.setInt(3, exampleSyncableEntity.esLcb)
      _stmt.setInt(4, exampleSyncableEntity.esNumber)
      _stmt.setString(5, exampleSyncableEntity.esName)
      _stmt.setBoolean(6, exampleSyncableEntity.publik)
      _stmt.setLong(7, exampleSyncableEntity.esUid)
      _stmt.executeUpdateAsyncKmp()
    }
  }

  override fun findAll(): List<ExampleSyncableEntity> =
      _db.prepareAndUseStatement(PreparedStatementConfig(
    sql = "SELECT * FROM ExampleSyncableEntity",
    readOnly = true,)
  ) { _stmt -> 
    _stmt.executeQuery().useResults{ _result -> 
      _result.mapRows {
        val _tmp_esUid = _result.getLong("esUid")
        val _tmp_esLcsn = _result.getInt("esLcsn")
        val _tmp_esMcsn = _result.getInt("esMcsn")
        val _tmp_esLcb = _result.getInt("esLcb")
        val _tmp_esNumber = _result.getInt("esNumber")
        val _tmp_esName = _result.getString("esName")
        val _tmp_publik = _result.getBoolean("publik")
        ExampleSyncableEntity().apply {
          this.esUid = _tmp_esUid
          this.esLcsn = _tmp_esLcsn
          this.esMcsn = _tmp_esMcsn
          this.esLcb = _tmp_esLcb
          this.esNumber = _tmp_esNumber
          this.esName = _tmp_esName
          this.publik = _tmp_publik
        }
      }
    }
  }

  override suspend fun findByListParam(numberList: List<Long>): List<ExampleSyncableEntity> =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = "SELECT * FROM ExampleSyncableEntity WHERE esNumber IN (?)",
    hasListParams = true,
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setArray(1, _stmt.getConnection().createArrayOrProxyArrayOf("BIGINT",
        numberList.toTypedArray()))
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapRows {
        val _tmp_esUid = _result.getLong("esUid")
        val _tmp_esLcsn = _result.getInt("esLcsn")
        val _tmp_esMcsn = _result.getInt("esMcsn")
        val _tmp_esLcb = _result.getInt("esLcb")
        val _tmp_esNumber = _result.getInt("esNumber")
        val _tmp_esName = _result.getString("esName")
        val _tmp_publik = _result.getBoolean("publik")
        ExampleSyncableEntity().apply {
          this.esUid = _tmp_esUid
          this.esLcsn = _tmp_esLcsn
          this.esMcsn = _tmp_esMcsn
          this.esLcb = _tmp_esLcb
          this.esNumber = _tmp_esNumber
          this.esName = _tmp_esName
          this.publik = _tmp_publik
        }
      }
    }
  }

  override fun findByUid(uid: Long): ExampleSyncableEntity? =
      _db.prepareAndUseStatement(PreparedStatementConfig(
    sql = "SELECT * FROM ExampleSyncableEntity WHERE esUid = CAST(? AS BIGINT)",
    postgreSql = """
    |SELECT * FROM ExampleSyncableEntity WHERE esUid = ?
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,uid)
    _stmt.executeQuery().useResults{ _result -> 
      _result.mapNextRow(null) {
        val _tmp_esUid = _result.getLong("esUid")
        val _tmp_esLcsn = _result.getInt("esLcsn")
        val _tmp_esMcsn = _result.getInt("esMcsn")
        val _tmp_esLcb = _result.getInt("esLcb")
        val _tmp_esNumber = _result.getInt("esNumber")
        val _tmp_esName = _result.getString("esName")
        val _tmp_publik = _result.getBoolean("publik")
        ExampleSyncableEntity().apply {
          this.esUid = _tmp_esUid
          this.esLcsn = _tmp_esLcsn
          this.esMcsn = _tmp_esMcsn
          this.esLcb = _tmp_esLcb
          this.esNumber = _tmp_esNumber
          this.esName = _tmp_esName
          this.publik = _tmp_publik
        }
      }
    }
  }

  override fun findByName(name: String?): ExampleSyncableEntity? =
      _db.prepareAndUseStatement(PreparedStatementConfig(
    sql = "SELECT * FROM ExampleSyncableEntity WHERE esName = ?",
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setString(1,name)
    _stmt.executeQuery().useResults{ _result -> 
      _result.mapNextRow(null) {
        val _tmp_esUid = _result.getLong("esUid")
        val _tmp_esLcsn = _result.getInt("esLcsn")
        val _tmp_esMcsn = _result.getInt("esMcsn")
        val _tmp_esLcb = _result.getInt("esLcb")
        val _tmp_esNumber = _result.getInt("esNumber")
        val _tmp_esName = _result.getString("esName")
        val _tmp_publik = _result.getBoolean("publik")
        ExampleSyncableEntity().apply {
          this.esUid = _tmp_esUid
          this.esLcsn = _tmp_esLcsn
          this.esMcsn = _tmp_esMcsn
          this.esLcb = _tmp_esLcb
          this.esNumber = _tmp_esNumber
          this.esName = _tmp_esName
          this.publik = _tmp_publik
        }
      }
    }
  }

  override fun findAllWithOtherByUid(): List<ExampleSyncableEntityWithOtherSyncableEntity> =
      _db.prepareAndUseStatement(PreparedStatementConfig(
    sql =
        "SELECT ExampleSyncableEntity.*, OtherSyncableEntity.* FROM ExampleSyncableEntity LEFT JOIN OtherSyncableEntity ON ExampleSyncableEntity.esUid = OtherSyncableEntity.otherFk",
    readOnly = true,)
  ) { _stmt -> 
    _stmt.executeQuery().useResults{ _result -> 
      _result.mapRows {
        val _tmp_esUid = _result.getLong("esUid")
        val _tmp_esLcsn = _result.getInt("esLcsn")
        val _tmp_esMcsn = _result.getInt("esMcsn")
        val _tmp_esLcb = _result.getInt("esLcb")
        val _tmp_esNumber = _result.getInt("esNumber")
        val _tmp_esName = _result.getString("esName")
        val _tmp_publik = _result.getBoolean("publik")
        var _tmp_OtherSyncableEntity_nullCount = 0
        val _tmp_osUid = _result.getLong("osUid")
        if(_result.wasNull()) _tmp_OtherSyncableEntity_nullCount++
        val _tmp_osLcb = _result.getInt("osLcb")
        if(_result.wasNull()) _tmp_OtherSyncableEntity_nullCount++
        val _tmp_osMcsn = _result.getInt("osMcsn")
        if(_result.wasNull()) _tmp_OtherSyncableEntity_nullCount++
        val _tmp_osLcsn = _result.getInt("osLcsn")
        if(_result.wasNull()) _tmp_OtherSyncableEntity_nullCount++
        val _tmp_otherFk = _result.getInt("otherFk")
        if(_result.wasNull()) _tmp_OtherSyncableEntity_nullCount++
        val _tmp_otherNum = _result.getInt("otherNum")
        if(_result.wasNull()) _tmp_OtherSyncableEntity_nullCount++
        val _tmp_otherStr = _result.getString("otherStr")
        if(_result.wasNull()) _tmp_OtherSyncableEntity_nullCount++
        val _tmp_OtherSyncableEntity_isAllNull = _tmp_OtherSyncableEntity_nullCount == 7
        ExampleSyncableEntityWithOtherSyncableEntity().apply {
          this.esUid = _tmp_esUid
          this.esLcsn = _tmp_esLcsn
          this.esMcsn = _tmp_esMcsn
          this.esLcb = _tmp_esLcb
          this.esNumber = _tmp_esNumber
          this.esName = _tmp_esName
          this.publik = _tmp_publik
          if(!_tmp_OtherSyncableEntity_isAllNull) {
            this.embeddedOse = OtherSyncableEntity().apply {
              this.osUid = _tmp_osUid
              this.osLcb = _tmp_osLcb
              this.osMcsn = _tmp_osMcsn
              this.osLcsn = _tmp_osLcsn
              this.otherFk = _tmp_otherFk
              this.otherNum = _tmp_otherNum
              this.otherStr = _tmp_otherStr
            }
          }
        }
      }
    }
  }

  override fun updateNumberByUid(uid: Long, newNumber: Long) {
    _db.prepareAndUseStatement(PreparedStatementConfig(
      sql =
          "UPDATE ExampleSyncableEntity SET esNumber = CAST(? AS BIGINT),esLcb = (SELECT nodeClientId FROM SyncNode LIMIT 1) WHERE esUid = CAST(? AS BIGINT)",
      postgreSql = """
      |UPDATE ExampleSyncableEntity SET esNumber = ?,esLcb = (SELECT nodeClientId FROM SyncNode LIMIT 1) WHERE esUid = ?
      |""".trimMargin(),
      readOnly = false,)
    ) { _stmt -> 
      _stmt.setLong(1,newNumber)
      _stmt.setLong(2,uid)
      _stmt.executeUpdate()
    }
  }

  override suspend fun findOneFromWeb(): ExampleSyncableEntity? =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = "SELECT * FROM ExampleSyncableEntity LIMIT 1",
    readOnly = true,)
  ) { _stmt -> 
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow(null) {
        val _tmp_esUid = _result.getLong("esUid")
        val _tmp_esLcsn = _result.getInt("esLcsn")
        val _tmp_esMcsn = _result.getInt("esMcsn")
        val _tmp_esLcb = _result.getInt("esLcb")
        val _tmp_esNumber = _result.getInt("esNumber")
        val _tmp_esName = _result.getString("esName")
        val _tmp_publik = _result.getBoolean("publik")
        ExampleSyncableEntity().apply {
          this.esUid = _tmp_esUid
          this.esLcsn = _tmp_esLcsn
          this.esMcsn = _tmp_esMcsn
          this.esLcb = _tmp_esLcb
          this.esNumber = _tmp_esNumber
          this.esName = _tmp_esName
          this.publik = _tmp_publik
        }
      }
    }
  }

  override suspend fun findOneValue(): Int =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = "SELECT esNumber FROM ExampleSyncableEntity LIMIT 1",
    readOnly = true,)
  ) { _stmt -> 
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow(0) {
        _result.getInt(1)
      }
    }
  }

  override fun getSyncNode(): SyncNode? = _db.prepareAndUseStatement(PreparedStatementConfig(
    sql = "SELECT SyncNode.* FROM SyncNode LIMIT 1",
    readOnly = true,)
  ) { _stmt -> 
    _stmt.executeQuery().useResults{ _result -> 
      _result.mapNextRow(null) {
        val _tmp_nodeClientId = _result.getLong("nodeClientId")
        SyncNode().apply {
          this.nodeClientId = _tmp_nodeClientId
        }
      }
    }
  }

  override fun insertFromSelectQuery(name: String?, publik: Boolean) {
    _db.prepareAndUseStatement(PreparedStatementConfig(
      sql = """
      |
      |   INSERT INTO ExampleSyncableEntity (esUid, esLcsn, esMcsn, esLcb, esNumber, esName, publik)
      |        SELECT (esUid * 5) AS esUid, 0 AS esLcsn, 0 AS esMcsn, 0 AS esLcb, 42 AS esNumber, ? AS esName, 
      |               ? AS publik 
      |          FROM ExampleSyncableEntity
      |         WHERE esName = ?
      |    
      """.trimMargin(),
      readOnly = false,)
    ) { _stmt -> 
      _stmt.setString(1,name)
      _stmt.setBoolean(2,publik)
      _stmt.setString(3,name)
      _stmt.executeUpdate()
    }
  }
}
