package db3

import androidx.paging.PagingSource
import com.ustadmobile.door.RepositoryFlowLoadingStatusProvider
import kotlin.Boolean
import kotlin.Int
import kotlin.Long
import kotlin.String
import kotlin.collections.List
import kotlinx.coroutines.flow.Flow

public actual abstract class DiscussionPostDao : RepositoryFlowLoadingStatusProvider {
  public actual abstract suspend fun insertAsync(post: DiscussionPost): Long

  public actual abstract suspend fun findByUid(postUid: Long): DiscussionPost?

  public actual abstract suspend fun findByUidAsyncOverHttp(postUid: Long): DiscussionPost?

  public actual abstract suspend fun findAllRepliesWithPosterMember(postUid: Long):
      List<DiscussionPostAndPosterMember>

  public actual abstract fun findAllRepliesWithPosterMemberAsFlow(postUid: Long):
      Flow<List<DiscussionPostAndPosterMember>>

  public actual abstract suspend fun findByUidWithPosterMember(postUid: Long):
      DiscussionPostAndPosterMember?

  public actual abstract fun findByUidWithPosterMemberAsFlow(postUid: Long):
      Flow<DiscussionPostAndPosterMember?>

  public actual abstract suspend fun findByUidList(postUids: List<Long>):
      List<DiscussionPostAndPosterMember>

  public actual abstract suspend fun getNumPostsSinceTime(since: Long): Int

  public actual abstract suspend fun getNumPostsSinceTimeHttpOnly(since: Long): Int

  public actual abstract fun findAllPostAsPagingSource(since: Long):
      PagingSource<Int, DiscussionPost>

  public actual abstract fun findAllPostAsNetworkOnlyPagingSource(since: Long):
      PagingSource<Int, DiscussionPost>

  public actual abstract fun findAllPostAsNetworkWithFallbackPagingSource(since: Long):
      PagingSource<Int, DiscussionPost>

  public actual abstract suspend fun findAllRepliesForPost(postUid: Long, nodeId: Long):
      List<DiscussionPost>

  public actual abstract suspend fun findPostAndNumReplies(postUid: Long, sinceTime: Long):
      DiscussionPostAndNumReplies?

  public actual abstract fun findRootRepliesAsPagingSource(): PagingSource<Int, DiscussionPost>

  public actual abstract fun findRootPostsAndNumRepliesAsPagingSource():
      PagingSource<Int, DiscussionPostAndNumReplies>

  public actual abstract suspend fun findReplyPostsWithOffsetAndLimit(offset: Int, limit: Int):
      List<DiscussionPost>

  public actual abstract fun findRootPostAndNumRepliesAsPagingSourceWithPagedParams():
      PagingSource<Int, DiscussionPostAndNumReplies>

  public actual abstract fun findRootPostAndNumRepliesAsPagingSourceWithAsFlow():
      Flow<List<DiscussionPostAndNumReplies>>

  public actual abstract suspend fun checkNodeHasPermission(postUid: Long, nodeId: Long): Boolean

  public actual abstract suspend fun findRepliesWithAuthCheck(postUid: Long): List<DiscussionPost>

  public actual abstract suspend fun getDiscussionPostAndAuthorName(postUid: Long):
      DiscussionPostAndAuthorName?

  public actual abstract suspend fun getDiscussionPostAndAuthorNameById(postId: Long):
      DiscussionPostAndAuthorName?

  public actual abstract suspend fun update(post: DiscussionPost)

  public actual abstract suspend fun findByName(firstName: String?): List<DiscussionPost>
}
