package db3

import com.ustadmobile.door.`annotation`.ReplicateEntity
import com.ustadmobile.door.`annotation`.Trigger
import com.ustadmobile.door.ext.DoorDatabaseMetadata
import com.ustadmobile.door.replication.ReplicationEntityMetaData
import com.ustadmobile.door.replication.ReplicationFieldMetaData
import kotlin.Boolean
import kotlin.Int
import kotlin.LazyThreadSafetyMode
import kotlin.String
import kotlin.collections.List
import kotlin.collections.Map
import kotlin.reflect.KClass

public class ExampleDb3_DoorMetadata : DoorDatabaseMetadata<ExampleDb3>() {
  override val dbClass: KClass<ExampleDb3>
    get() = ExampleDb3::class

  override val hasReadOnlyWrapper: Boolean
    get() = true

  override val version: Int
    get() = 1

  override val allTables: List<String> = listOf("ExampleEntity3", "DiscussionPost", "Member",
      "OutgoingReplication", "ReplicationOperation", "PendingRepositorySession", "Badge",
      "StatementEntity")


  override val replicateEntities: Map<Int, ReplicationEntityMetaData> by
      lazy(LazyThreadSafetyMode.NONE) {
        mapOf<Int, ReplicationEntityMetaData>(
        542 to ReplicationEntityMetaData(tableId = 542, 
        entityTableName = "ExampleEntity3", 
        receiveViewName = "ExampleEntity3_ReceiveView", 
        entityPrimaryKeyFieldNames = listOf("eeUid",),
        entityVersionIdFieldName = "lastUpdatedTime", 
        entityFields = listOf(
          ReplicationFieldMetaData(
            fieldName = "cardNumber",
            dbFieldType = 4,
            nullable = false,
          ),
          ReplicationFieldMetaData(
            fieldName = "name",
            dbFieldType = -1,
            nullable = true,
          ),
          ReplicationFieldMetaData(
            fieldName = "lastUpdatedTime",
            dbFieldType = -5,
            nullable = false,
          ),
          ReplicationFieldMetaData(
            fieldName = "nullableNumber",
            dbFieldType = 4,
            nullable = true,
          ),
          ReplicationFieldMetaData(
            fieldName = "eeUid",
            dbFieldType = -5,
            nullable = false,
          ),
        )
        ,
        batchSize = 1_000,
        remoteInsertStrategy = ReplicateEntity.RemoteInsertStrategy.INSERT_INTO_RECEIVE_VIEW,
        triggers = listOf(
          Trigger(
            name = "remote_insert3",
            order = Trigger.Order.INSTEAD_OF,
            events = arrayOf(Trigger.Event.INSERT,),
            on = Trigger.On.RECEIVEVIEW,
            sqlStatements = arrayOf(
              "REPLACE INTO ExampleEntity3 (eeUid, cardNumber, name, lastUpdatedTime, nullableNumber) VALUES( CAST(NEW.eeUid AS BIGINT), NEW.cardNumber, NEW.name, CAST(NEW.lastUpdatedTime AS BIGINT), NEW.nullableNumber) ",
            ),
            postgreSqlStatements = arrayOf(
              "INSERT INTO ExampleEntity3 (eeUid, cardNumber, name, lastUpdatedTime, nullableNumber) VALUES( CAST(NEW.eeUid AS BIGINT), NEW.cardNumber, NEW.name, CAST(NEW.lastUpdatedTime AS BIGINT), NEW.nullableNumber)  ON CONFLICT(eeUid)  DO UPDATE SET cardNumber = NEW.cardNumber, name = NEW.name, lastUpdatedTime = NEW.lastUpdatedTime, nullableNumber = NEW.nullableNumber ",
            ),
            conditionSql = "",
            conditionSqlPostgres = "",
          ),),
        ),
        543 to ReplicationEntityMetaData(tableId = 543, 
        entityTableName = "DiscussionPost", 
        receiveViewName = "DiscussionPost_ReceiveView", 
        entityPrimaryKeyFieldNames = listOf("postUid",),
        entityVersionIdFieldName = "postLastModified", 
        entityFields = listOf(
          ReplicationFieldMetaData(
            fieldName = "postReplyToPostUid",
            dbFieldType = -5,
            nullable = false,
          ),
          ReplicationFieldMetaData(
            fieldName = "postTitle",
            dbFieldType = -1,
            nullable = true,
          ),
          ReplicationFieldMetaData(
            fieldName = "postText",
            dbFieldType = -1,
            nullable = true,
          ),
          ReplicationFieldMetaData(
            fieldName = "postLastModified",
            dbFieldType = -5,
            nullable = false,
          ),
          ReplicationFieldMetaData(
            fieldName = "posterMemberUid",
            dbFieldType = -5,
            nullable = false,
          ),
          ReplicationFieldMetaData(
            fieldName = "isBanned",
            dbFieldType = 16,
            nullable = true,
          ),
          ReplicationFieldMetaData(
            fieldName = "postUid",
            dbFieldType = -5,
            nullable = false,
          ),
        )
        ,
        batchSize = 1_000,
        remoteInsertStrategy = ReplicateEntity.RemoteInsertStrategy.INSERT_INTO_RECEIVE_VIEW,
        triggers = listOf(
          Trigger(
            name = "discussion_post_remote_insert",
            order = Trigger.Order.INSTEAD_OF,
            events = arrayOf(Trigger.Event.INSERT,),
            on = Trigger.On.RECEIVEVIEW,
            sqlStatements = arrayOf(
              "REPLACE INTO DiscussionPost (postUid, postReplyToPostUid, postTitle, postText, postLastModified, posterMemberUid, isBanned) VALUES( CAST(NEW.postUid AS BIGINT), CAST(NEW.postReplyToPostUid AS BIGINT), NEW.postTitle, NEW.postText, CAST(NEW.postLastModified AS BIGINT), CAST(NEW.posterMemberUid AS BIGINT), NEW.isBanned) ",
            ),
            postgreSqlStatements = arrayOf(
              "INSERT INTO DiscussionPost (postUid, postReplyToPostUid, postTitle, postText, postLastModified, posterMemberUid, isBanned) VALUES( CAST(NEW.postUid AS BIGINT), CAST(NEW.postReplyToPostUid AS BIGINT), NEW.postTitle, NEW.postText, CAST(NEW.postLastModified AS BIGINT), CAST(NEW.posterMemberUid AS BIGINT), NEW.isBanned)  ON CONFLICT(postUid)  DO UPDATE SET postReplyToPostUid = NEW.postReplyToPostUid, postTitle = NEW.postTitle, postText = NEW.postText, postLastModified = NEW.postLastModified, posterMemberUid = NEW.posterMemberUid, isBanned = NEW.isBanned ",
            ),
            conditionSql = """
            |SELECT 
            |                   CAST(NEW.postLastModified AS BIGINT) >
            |                       COALESCE((SELECT DiscussionPost_Existing.postLastModified
            |                                   FROM DiscussionPost DiscussionPost_Existing
            |                                  WHERE DiscussionPost_Existing.postUid = NEW.postUid), 0)
            |                   
            """.trimMargin(),
            conditionSqlPostgres = """
            |SELECT 
            |                   CAST(NEW.postLastModified AS BIGINT) >
            |                       COALESCE((SELECT DiscussionPost_Existing.postLastModified
            |                                   FROM DiscussionPost DiscussionPost_Existing
            |                                  WHERE DiscussionPost_Existing.postUid = NEW.postUid), 0)
            |                   
            """.trimMargin(),
          ),),
        ),
        544 to ReplicationEntityMetaData(tableId = 544, 
        entityTableName = "Member", 
        receiveViewName = "Member_ReceiveView", 
        entityPrimaryKeyFieldNames = listOf("memberUid",),
        entityVersionIdFieldName = "memberLastModified", 
        entityFields = listOf(
          ReplicationFieldMetaData(
            fieldName = "firstName",
            dbFieldType = -1,
            nullable = true,
          ),
          ReplicationFieldMetaData(
            fieldName = "lastName",
            dbFieldType = -1,
            nullable = true,
          ),
          ReplicationFieldMetaData(
            fieldName = "memberLastModified",
            dbFieldType = -5,
            nullable = false,
          ),
          ReplicationFieldMetaData(
            fieldName = "memberUid",
            dbFieldType = -5,
            nullable = false,
          ),
        )
        ,
        batchSize = 1_000,
        remoteInsertStrategy = ReplicateEntity.RemoteInsertStrategy.INSERT_INTO_RECEIVE_VIEW,
        triggers = listOf(
          Trigger(
            name = "member_remote_insert",
            order = Trigger.Order.INSTEAD_OF,
            events = arrayOf(Trigger.Event.INSERT,),
            on = Trigger.On.RECEIVEVIEW,
            sqlStatements = arrayOf(
              "REPLACE INTO Member (memberUid, firstName, lastName, memberLastModified) VALUES( CAST(NEW.memberUid AS BIGINT), NEW.firstName, NEW.lastName, CAST(NEW.memberLastModified AS BIGINT)) ",
            ),
            postgreSqlStatements = arrayOf(
              "INSERT INTO Member (memberUid, firstName, lastName, memberLastModified) VALUES( CAST(NEW.memberUid AS BIGINT), NEW.firstName, NEW.lastName, CAST(NEW.memberLastModified AS BIGINT))  ON CONFLICT(memberUid)  DO UPDATE SET firstName = NEW.firstName, lastName = NEW.lastName, memberLastModified = NEW.memberLastModified ",
            ),
            conditionSql = "",
            conditionSqlPostgres = "",
          ),),
        ),
        1_702 to ReplicationEntityMetaData(tableId = 1_702, 
        entityTableName = "Badge", 
        receiveViewName = "Badge_ReceiveView", 
        entityPrimaryKeyFieldNames = listOf("badgeUid",),
        entityVersionIdFieldName = "badgeLastChangeTime", 
        entityFields = listOf(
          ReplicationFieldMetaData(
            fieldName = "badgeName",
            dbFieldType = -1,
            nullable = true,
          ),
          ReplicationFieldMetaData(
            fieldName = "badgePoints",
            dbFieldType = 4,
            nullable = false,
          ),
          ReplicationFieldMetaData(
            fieldName = "badgeLastChangeTime",
            dbFieldType = -5,
            nullable = false,
          ),
          ReplicationFieldMetaData(
            fieldName = "badgeUid",
            dbFieldType = -5,
            nullable = false,
          ),
        )
        ,
        batchSize = 1_000,
        remoteInsertStrategy = ReplicateEntity.RemoteInsertStrategy.INSERT_INTO_RECEIVE_VIEW,
        triggers = listOf(
          Trigger(
            name = "badge_remote_insert",
            order = Trigger.Order.INSTEAD_OF,
            events = arrayOf(Trigger.Event.INSERT,),
            on = Trigger.On.RECEIVEVIEW,
            sqlStatements = arrayOf(
              "REPLACE INTO Badge (badgeUid, badgeName, badgePoints, badgeLastChangeTime) VALUES( CAST(NEW.badgeUid AS BIGINT), NEW.badgeName, NEW.badgePoints, CAST(NEW.badgeLastChangeTime AS BIGINT)) ",
            ),
            postgreSqlStatements = arrayOf(
              "INSERT INTO Badge (badgeUid, badgeName, badgePoints, badgeLastChangeTime) VALUES( CAST(NEW.badgeUid AS BIGINT), NEW.badgeName, NEW.badgePoints, CAST(NEW.badgeLastChangeTime AS BIGINT))  ON CONFLICT(badgeUid)  DO UPDATE SET badgeName = NEW.badgeName, badgePoints = NEW.badgePoints, badgeLastChangeTime = NEW.badgeLastChangeTime ",
            ),
            conditionSql = "",
            conditionSqlPostgres = "",
          ),),
        ),
        12_121 to ReplicationEntityMetaData(tableId = 12_121, 
        entityTableName = "StatementEntity", 
        receiveViewName = "StatementEntity_ReceiveView", 
        entityPrimaryKeyFieldNames = listOf("uidHi","uidLo",),
        entityVersionIdFieldName = "lct", 
        entityFields = listOf(
          ReplicationFieldMetaData(
            fieldName = "uidHi",
            dbFieldType = -5,
            nullable = false,
          ),
          ReplicationFieldMetaData(
            fieldName = "uidLo",
            dbFieldType = -5,
            nullable = false,
          ),
          ReplicationFieldMetaData(
            fieldName = "lct",
            dbFieldType = -5,
            nullable = false,
          ),
          ReplicationFieldMetaData(
            fieldName = "name",
            dbFieldType = -1,
            nullable = true,
          ),
        )
        ,
        batchSize = 1_000,
        remoteInsertStrategy = ReplicateEntity.RemoteInsertStrategy.INSERT_INTO_RECEIVE_VIEW,
        triggers = listOf(
          Trigger(
            name = "statement_remote_insert",
            order = Trigger.Order.INSTEAD_OF,
            events = arrayOf(Trigger.Event.INSERT,),
            on = Trigger.On.RECEIVEVIEW,
            sqlStatements = arrayOf(
              "REPLACE INTO StatementEntity (uidHi, uidLo, lct, name) VALUES( CAST(NEW.uidHi AS BIGINT), CAST(NEW.uidLo AS BIGINT), CAST(NEW.lct AS BIGINT), NEW.name) ",
            ),
            postgreSqlStatements = arrayOf(
              "INSERT INTO StatementEntity (uidHi, uidLo, lct, name) VALUES( CAST(NEW.uidHi AS BIGINT), CAST(NEW.uidLo AS BIGINT), CAST(NEW.lct AS BIGINT), NEW.name)  ON CONFLICT(uidHi, uidLo)  DO UPDATE SET lct = NEW.lct, name = NEW.name ",
            ),
            conditionSql = """
            |SELECT 
            |                   CAST(NEW.lct AS BIGINT) >
            |                       COALESCE((SELECT StatementEntity_Existing.lct
            |                                   FROM StatementEntity StatementEntity_Existing
            |                                  WHERE StatementEntity_Existing.uidHi = NEW.uidHi AND StatementEntity_Existing.uidLo = NEW.uidLo), 0)
            |                   
            """.trimMargin(),
            conditionSqlPostgres = """
            |SELECT 
            |                   CAST(NEW.lct AS BIGINT) >
            |                       COALESCE((SELECT StatementEntity_Existing.lct
            |                                   FROM StatementEntity StatementEntity_Existing
            |                                  WHERE StatementEntity_Existing.uidHi = NEW.uidHi AND StatementEntity_Existing.uidLo = NEW.uidLo), 0)
            |                   
            """.trimMargin(),
          ),),
        ),
        )
      }

}
