package db3

import com.ustadmobile.door.DoorDatabaseRepository
import com.ustadmobile.door.RepositoryConfig
import com.ustadmobile.door.RepositoryHelper
import com.ustadmobile.door.replication.DoorRepositoryReplicationClient
import com.ustadmobile.door.room.InvalidationTracker
import io.ktor.client.HttpClient
import kotlin.Any
import kotlin.IllegalStateException
import kotlin.Int
import kotlin.Long
import kotlin.String
import kotlin.Suppress
import kotlin.Unit
import kotlin.collections.List
import kotlinx.coroutines.flow.Flow

@Suppress("LocalVariableName", "PropertyName", "FunctionName", "ClassName")
public class ExampleDb3_Repo(
  override val db: ExampleDb3,
  dbUnwrapped: ExampleDb3,
  override val config: RepositoryConfig,
) : ExampleDb3(), DoorDatabaseRepository {
  public val context: Any
    get() = config.context

  private val _db: ExampleDb3 = dbUnwrapped

  private val _endpoint: String
    get() = config.endpoint

  public val _httpClient: HttpClient
    get() = config.httpClient

  public val _repositoryHelper: RepositoryHelper = RepositoryHelper(db, config)


  public val clientId: Long
    get() = config.nodeId

  override val dbName: String
    get() = "Repository for [${_db.toString()}] - ${config.endpoint}"

  override var connectivityStatus: Int
    get() = _repositoryHelper.connectivityStatus
    set(newValue) {
      _repositoryHelper.connectivityStatus = newValue
    }

  override val clientState: Flow<DoorRepositoryReplicationClient.ClientState>
    get() = _repositoryHelper.clientState

  override val invalidationTracker: InvalidationTracker
    get() = _db.invalidationTracker

  override val dbVersion: Int
    get() = 1

  private val _ExampleEntity3Dao: ExampleEntity3Dao_Repo by lazy {
        ExampleEntity3Dao_Repo(db, this, db.exampleEntity3Dao, _httpClient, clientId, _endpoint)
      }


  override val exampleEntity3Dao: ExampleEntity3Dao
    get() =  _ExampleEntity3Dao

  private val _DiscussionPostDao: DiscussionPostDao_Repo by lazy {
        DiscussionPostDao_Repo(db, this, db.discussionPostDao, _httpClient, clientId, _endpoint)
      }


  override val discussionPostDao: DiscussionPostDao
    get() =  _DiscussionPostDao

  private val _MemberDao: MemberDao_Repo by lazy {
        MemberDao_Repo(db, this, db.memberDao, _httpClient, clientId, _endpoint)
      }


  override val memberDao: MemberDao
    get() =  _MemberDao

  private val _BadgeDao: BadgeDao_Repo by lazy {
        BadgeDao_Repo(db, this, db.badgeDao, _httpClient, clientId, _endpoint)
      }


  override val badgeDao: BadgeDao
    get() =  _BadgeDao

  private val _StatementEntityDao: StatementEntityDao_Repo by lazy {
        StatementEntityDao_Repo(db, this, db.statementEntityDao, _httpClient, clientId, _endpoint)
      }


  override val statementEntityDao: StatementEntityDao
    get() =  _StatementEntityDao

  override fun clearAllTables(): Unit = throw
      IllegalStateException("Cannot use a repository to clearAllTables!")

  override suspend fun clearAllTablesAsync(): Unit = throw
      IllegalStateException("Cannot use this to run clearAllTablesAsync")

  override fun close() {
    _repositoryHelper.close()
  }

  override fun remoteNodeIdOrNull(): Long? = _repositoryHelper.remoteNodeIdOrNull()

  override fun remoteNodeIdOrFake(): Long = _repositoryHelper.remoteNodeIdOrFake()

  override fun createAllTables(): List<String> = throw
      IllegalStateException("Cannot use a repository to createAllTables!")

  public companion object {
    public const val _DB_NAME: String = "ExampleDb3"
  }
}
