package db2

import com.ustadmobile.door.DoorDbType
import com.ustadmobile.door.EntityInsertionAdapter
import com.ustadmobile.door.PreparedStatementConfig
import com.ustadmobile.door.ext.prepareAndUseStatement
import com.ustadmobile.door.jdbc.PreparedStatement
import com.ustadmobile.door.jdbc.ext.getIntNullable
import com.ustadmobile.door.jdbc.ext.mapRows
import com.ustadmobile.door.jdbc.ext.setIntNullable
import com.ustadmobile.door.jdbc.ext.useResults
import com.ustadmobile.door.room.RoomDatabase
import kotlin.Boolean
import kotlin.IllegalArgumentException
import kotlin.Int
import kotlin.Long
import kotlin.String
import kotlin.collections.List

public class ExampleDaoWithInterface_JdbcImpl(
  public val _db: RoomDatabase,
) : ExampleDaoWithInterface() {
  public val _insertAdapterExampleEntity2_abort: EntityInsertionAdapter<ExampleEntity2> = object :
      EntityInsertionAdapter<ExampleEntity2>(_db) {
    override fun makeSql(returnsId: Boolean): String = when(dbType) {
      DoorDbType.SQLITE -> {
        "INSERT INTO ExampleEntity2 (uid, name, someNumber, checked, rewardsCardNumber) VALUES(?, ?, ?, ?, ?)"
      }
      DoorDbType.POSTGRES ->  {
        "INSERT INTO ExampleEntity2 (uid, name, someNumber, checked, rewardsCardNumber) VALUES(COALESCE(?,nextval('ExampleEntity2_uid_seq')), ?, ?, ?, ?)" + if(returnsId) { " RETURNING uid" } else "" 
      }
      else -> {
        throw IllegalArgumentException("Unsupported db type")
      }
    }

    override fun bindPreparedStmtToEntity(stmt: PreparedStatement, entity: ExampleEntity2) {
      if(entity.uid == 0L) {
        stmt.setObject(1, null)
      } else {
        stmt.setLong(1, entity.uid)
      }
      stmt.setString(2, entity.name)
      stmt.setLong(3, entity.someNumber)
      stmt.setBoolean(4, entity.checked)
      stmt.setIntNullable(5, entity.rewardsCardNumber)
    }
  }

  public override fun insertOne(entity: ExampleEntity2): Long {
    val _retVal = _insertAdapterExampleEntity2_abort.insertAndReturnId(entity)
    return _retVal
  }

  override fun anotherQuery(`param`: Int): List<ExampleEntity2> =
      _db.prepareAndUseStatement(PreparedStatementConfig(
    sql = "SELECT * FROM ExampleEntity2 WHERE uid > ?",
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setInt(1,param)
    _stmt.executeQuery().useResults{ _result -> 
      _result.mapRows {
        val _tmp_uid = _result.getLong("uid")
        val _tmp_name = _result.getString("name")
        val _tmp_someNumber = _result.getLong("someNumber")
        val _tmp_checked = _result.getBoolean("checked")
        val _tmp_rewardsCardNumber = _result.getIntNullable("rewardsCardNumber")
        ExampleEntity2().apply {
          this.uid = _tmp_uid
          this.name = _tmp_name
          this.someNumber = _tmp_someNumber
          this.checked = _tmp_checked
          this.rewardsCardNumber = _tmp_rewardsCardNumber
        }
      }
    }
  }
}
