package db2

import androidx.paging.PagingSource
import androidx.room.Dao
import androidx.room.Delete
import androidx.room.Insert
import androidx.room.Query
import androidx.room.RawQuery
import androidx.room.Update
import com.ustadmobile.door.DoorQuery
import kotlin.Any
import kotlin.Array
import kotlin.Int
import kotlin.Long
import kotlin.String
import kotlin.collections.List
import kotlinx.coroutines.flow.Flow

@Dao
public actual abstract class ExampleDao2 {
  @Insert(
    entity = Any::class,
    onConflict = 3,
  )
  public actual abstract fun insertAndReturnId(entity: ExampleEntity2): Long

  @Insert(
    entity = Any::class,
    onConflict = 3,
  )
  public actual abstract suspend fun insertAsync(entity: ExampleEntity2)

  @Insert(
    entity = Any::class,
    onConflict = 3,
  )
  public actual abstract suspend fun insertAsyncAndGiveId(entity: ExampleEntity2): Long

  @Insert(
    onConflict = 5,
    entity = Any::class,
  )
  public actual abstract suspend fun insertOrIgnore(entity: ExampleEntity2)

  @Insert(
    entity = Any::class,
    onConflict = 3,
  )
  public actual abstract fun insertList(entityList: List<ExampleEntity2>)

  @Insert(
    entity = Any::class,
    onConflict = 3,
  )
  public actual abstract suspend fun insertListAsync(entityList: List<ExampleEntity2>)

  @Insert(
    entity = Any::class,
    onConflict = 3,
  )
  public actual abstract fun insertOtherList(entityList: List<ExampleEntity2>)

  @Insert(
    entity = Any::class,
    onConflict = 3,
  )
  public actual abstract fun insertAndReturnList(entityList: List<ExampleEntity2>): List<Long>

  @Insert(
    entity = Any::class,
    onConflict = 3,
  )
  public actual abstract fun insertListAndReturnIdsArray(entityList: List<ExampleEntity2>):
      Array<Long>

  @Insert(
    onConflict = 1,
    entity = Any::class,
  )
  public actual abstract fun replace(entityList: List<ExampleEntity2>)

  @Query(`value` = "SELECT * FROM ExampleEntity2 WHERE uid = :uid")
  public actual abstract fun findByUid(uid: Long): ExampleEntity2?

  @Query(`value` = "SELECT * FROM ExampleEntity2 WHERE uid = :uid")
  public actual abstract suspend fun findByUidAsync(uid: Long): ExampleEntity2?

  @Query(`value` = "SELECT * FROM ExampleEntity2 WHERE uid > :uid AND someNumber > :min")
  public actual abstract suspend fun findLarge(uid: Long, min: Long): ExampleEntity2?

  @Query(`value` = "SELECT * FROM ExampleEntity2 WHERE uid > :uid AND someNumber > :min")
  public actual abstract suspend fun findLargeAsync(uid: Long, min: Long): List<ExampleEntity2>

  @Query(`value` = "SELECT * FROM ExampleEntity2 WHERE name = :name")
  public actual abstract fun findWithNullableParam(name: String?): List<ExampleEntity2>

  @Query(`value` = "SELECT name FROM ExampleEntity2 WHERE uid = :uid")
  public actual abstract fun findNameByUid(uid: Long): String?

  @Query(`value` = "SELECT name FROM ExampleEntity2 WHERE uid = :uid")
  public actual abstract suspend fun findNameByUidAsync(uid: Long): String?

  @Query(`value` =
      "SELECT ExampleEntity2.*, ExampleLinkEntity.* FROM  ExampleEntity2 LEFT JOIN ExampleLinkEntity ON ExampleEntity2.uid = ExampleLinkEntity.fkValue WHERE ExampleEntity2.uid = :uid")
  public actual abstract suspend fun findByUidWithLinkEntityAsync(uid: Long):
      ExampleEntity2WithExampleLinkEntity?

  @Query(`value` = "SELECT * FROM ExampleEntity2 ORDER BY uid")
  public actual abstract fun findAll(): List<ExampleEntity2>

  @Query(`value` = "SELECT * FROM ExampleEntity2")
  public actual abstract suspend fun findAllAsync(): List<ExampleEntity2>

  @Query(`value` = "SELECT name FROM ExampleEntity2")
  public actual abstract suspend fun findAllStrings(): List<String?>

  @Query(`value` = "SELECT name FROM ExampleEntity2")
  public actual abstract fun findAllStringsSync(): List<String?>

  @Update(
    entity = Any::class,
    onConflict = 3,
  )
  public actual abstract suspend fun updateSingleItemAsync(entity: ExampleEntity2)

  @Update(
    entity = Any::class,
    onConflict = 3,
  )
  public actual abstract suspend fun updateSingleItemAndReturnCountAsync(entity: ExampleEntity2):
      Int

  @Update(
    entity = Any::class,
    onConflict = 3,
  )
  public actual abstract suspend fun updateListAsync(updateEntityList: List<ExampleEntity2>)

  @Query(`value` = "UPDATE ExampleEntity2 SET name = :newName WHERE someNumber >= :num")
  public actual abstract fun updateByParam(newName: String, num: Long): Int

  @Query(`value` = "UPDATE ExampleEntity2 SET name = :newName WHERE someNumber >= :num")
  public actual abstract suspend fun updateByParamAsync(newName: String, num: Long): Int

  @Query(`value` = "UPDATE exampleentity2 SET name = :newName WHERE someNumber >= :number")
  public actual abstract fun updateByParamNoReturn(newName: String, number: Long)

  @Delete(entity = Any::class)
  public actual abstract fun deleteSingle(entity: ExampleEntity2)

  @Delete(entity = Any::class)
  public actual abstract fun deleteList(deleteList: List<ExampleEntity2>)

  @Delete(entity = Any::class)
  public actual abstract suspend fun deleteListAsync(deleteList: List<ExampleEntity2>): Int

  @Query(`value` = "SELECT COUNT(*) FROM ExampleEntity2")
  public actual abstract fun countNumEntities(): Int

  @Query(`value` = "SELECT COUNT(*) FROM ExampleEntity2")
  public actual abstract suspend fun countNumEntitiesAsync(): Int

  @Query(`value` = "SELECT * FROM ExampleEntity2 WHERE uid IN (:uidList)")
  public actual abstract fun queryUsingArray(uidList: List<Long>): List<ExampleEntity2>

  @RawQuery(observedEntities = arrayOf())
  public actual abstract fun rawQueryForList(query: DoorQuery): List<ExampleEntity2>

  @RawQuery(observedEntities = arrayOf())
  public actual abstract suspend fun rawQueryForListAsyc(query: DoorQuery): List<ExampleEntity2>

  @RawQuery(observedEntities = arrayOf())
  public actual abstract fun rawQueryForListWithEmbeddedVals(query: DoorQuery):
      List<ExampleEntity2WithExampleLinkEntity>

  @RawQuery(observedEntities = arrayOf())
  public actual abstract fun rawQueryForSingleValue(query: DoorQuery): ExampleEntity2?

  @RawQuery(observedEntities = arrayOf())
  public actual abstract fun rawQueryWithArrParam(query: DoorQuery): List<ExampleEntity2>

  @Query(`value` =
      "\n   INSERT INTO ExampleEntity2(uid, name, someNumber)\n        SELECT (uid + 1) AS uid, name AS name, (someNumber * 2) AS someNumber\n          FROM ExampleEntity2\n         WHERE uid = :uid \n    ")
  public actual abstract fun insertFromSelectQuery(uid: Int)

  @Query(`value` = "SELECT * FROM ExampleEntity2")
  public actual abstract fun queryAllAsFlow(): Flow<List<ExampleEntity2>>

  @Query(`value` = "SELECT * FROM ExampleEntity2 WHERE someNumber > :greaterThan LIMIT 1")
  public actual abstract fun findSingleNotNullableEntity(greaterThan: Int): ExampleEntity2

  @Query(`value` = "SELECT * FROM ExampleEntity2 WHERE someNumber > :greaterThan LIMIT 1")
  public actual abstract suspend fun findSingleNotNullableEntityAsync(greaterThan: Int):
      ExampleEntity2

  @Query(`value` = "SELECT someNumber FROM ExampleEntity2 WHERE someNumber > :greaterThan LIMIT 1")
  public actual abstract fun findSingleNotNullablePrimitive(greaterThan: Int): Int

  @Query(`value` = "SELECT someNumber FROM ExampleEntity2 WHERE someNumber > :greaterThan LIMIT 1")
  public actual abstract suspend fun findSingleNotNullablePrimitiveAsync(greaterThan: Int): Int

  @Query(`value` = "SELECT someNumber FROM ExampleEntity2 WHERE someNumber > :greaterThan LIMIT 1")
  public actual abstract fun findSingleNullablePrimitive(greaterThan: Int): Int?

  @Query(`value` = "SELECT someNumber FROM ExampleEntity2 WHERE someNumber > :greaterThan LIMIT 1")
  public actual abstract suspend fun findSingleNullablePrimitiveAsync(greaterThan: Long): Long?

  @Query(`value` =
      "\n        SELECT ExampleEntity2.* FROM ExampleEntity2\n         WHERE (CASE\n                WHEN :rewardsNum IS NULL THEN (ExampleEntity2.rewardsCardNumber IS NULL)\n                ELSE (ExampleEntity2.rewardsCardNumber = :rewardsNum) \n                END)\n    ")
  public actual abstract suspend fun findWithNullableIntAsync(rewardsNum: Int?): ExampleEntity2?

  @Query(`value` =
      "\n        SELECT * \n          FROM ExampleEntity2\n         WHERE rewardsCardNumber IS NOT NULL\n           AND rewardsCardNumber >= :minNumber \n      ORDER BY rewardsCardNumber    \n    ")
  public actual abstract fun findAllWithRewardNumberAsPagingSource(minNumber: Int):
      PagingSource<Int, ExampleEntity2>

  @Query(`value` =
      "\n        SELECT * \n          FROM ExampleEntity2\n         WHERE rewardsCardNumber IS NOT NULL\n           AND rewardsCardNumber >= :minNumber \n      ORDER BY rewardsCardNumber    \n    ")
  public actual abstract suspend fun findAllWithRewardNumberAsListAsync(minNumber: Int):
      List<ExampleEntity2>

  @Query(`value` =
      "\n        SELECT * \n          FROM ExampleEntity2\n         WHERE someNumber >= :minNumber \n    ")
  public actual abstract suspend fun findByMinSomeNumber(minNumber: Long): List<ExampleEntity2>
}
