package repdb

import androidx.paging.PagingSource
import androidx.room.Dao
import androidx.room.Insert
import androidx.room.Query
import androidx.room.Update
import com.ustadmobile.door.entities.DoorNode
import kotlin.Any
import kotlin.Int
import kotlin.Long
import kotlin.String
import kotlin.collections.List
import kotlinx.coroutines.flow.Flow

@Dao
public actual interface RepDao : RepDaoInterface<RepEntity> {
  @Insert(
    entity = Any::class,
    onConflict = 3,
  )
  public actual suspend fun insertDoorNodeAsync(node: DoorNode)

  @Insert(
    entity = Any::class,
    onConflict = 3,
  )
  public actual fun insertDoorNode(node: DoorNode)

  @Insert(
    entity = Any::class,
    onConflict = 3,
  )
  public actual suspend fun insertAsync(repEntity: RepEntity): Long

  @Insert(
    entity = Any::class,
    onConflict = 3,
  )
  public actual fun insert(repEntity: RepEntity): Long

  @Insert(
    entity = Any::class,
    onConflict = 3,
  )
  public actual fun insertList(repEntityList: List<RepEntity>)

  @Update(
    entity = Any::class,
    onConflict = 3,
  )
  public actual fun update(repEntity: RepEntity)

  @Update(
    entity = Any::class,
    onConflict = 3,
  )
  public actual suspend fun updateAsync(repEntity: RepEntity)

  @Query(`value` = "\n    SELECT COUNT(*)\n      FROM RepEntity\n    ")
  public actual fun countEntities(): Int

  @Query(`value` =
      "\n    SELECT RepEntity.*\n      FROM RepEntity\n     WHERE RepEntity.rePrimaryKey = :uid \n    ")
  public actual fun findByUid(uid: Long): RepEntity?

  @Query(`value` =
      "\n    SELECT RepEntity.*\n      FROM RepEntity\n     WHERE RepEntity.rePrimaryKey = :uid \n    ")
  public actual suspend fun findByUidAsync(uid: Long): RepEntity?

  @Query(`value` = "\n    SELECT RepEntity.*\n      FROM RepEntity\n    ")
  public actual suspend fun findAllAsync(): List<RepEntity>

  @Insert(
    entity = Any::class,
    onConflict = 3,
  )
  public actual suspend fun insertHttp(entity: RepEntity): Long

  @Query(`value` =
      "\n        SELECT COALESCE(\n                    (SELECT nodeClientId \n                       FROM SyncNode \n                      LIMIT 1), 0)\n    ")
  public actual suspend fun selectSyncNodeId(): Long

  @Query(`value` =
      "\n        SELECT RepEntity.*\n          FROM RepEntity\n         WHERE reString IN (:strList) \n    ")
  public actual suspend fun findInStringList(strList: List<String>): List<RepEntity>

  @Query(`value` = "SELECT MAX(:num1, :num2)")
  public actual suspend fun sqliteOnlyFun(num1: Int, num2: Int): Long

  @Query(`value` = "\n        SELECT * FROM RepEntity\n    ")
  public actual fun findAllAsFlow(): Flow<List<RepEntity>>

  @Query(`value` = "\n        SELECT * FROM RepEntity\n    ")
  public actual fun findAllPaged(): PagingSource<Int, RepEntity>
}
