package db3

import androidx.paging.PagingSource
import androidx.room.Dao
import androidx.room.Insert
import androidx.room.Query
import androidx.room.Update
import com.ustadmobile.door.RepositoryFlowLoadingStatusProvider
import kotlin.Any
import kotlin.Boolean
import kotlin.Int
import kotlin.Long
import kotlin.String
import kotlin.collections.List
import kotlinx.coroutines.flow.Flow

@Dao
public actual abstract class DiscussionPostDao : RepositoryFlowLoadingStatusProvider {
  @Insert(
    entity = Any::class,
    onConflict = 3,
  )
  public actual abstract suspend fun insertAsync(post: DiscussionPost): Long

  @Query(`value` =
      "\n        SELECT DiscussionPost.*\n          FROM DiscussionPost\n         WHERE DiscussionPost.postUid = :postUid\n    ")
  public actual abstract suspend fun findByUid(postUid: Long): DiscussionPost?

  @Query(`value` =
      "\n        SELECT DiscussionPost.*\n          FROM DiscussionPost\n         WHERE DiscussionPost.postUid = :postUid\n    ")
  public actual abstract suspend fun findByUidAsyncOverHttp(postUid: Long): DiscussionPost?

  @Query(`value` =
      "\n        SELECT DiscussionPost.*, Member.*\n          FROM DiscussionPost\n               LEFT JOIN Member\n                     ON Member.memberUid = DiscussionPost.posterMemberUid\n         WHERE DiscussionPost.postReplyToPostUid = :postUid\n    ")
  public actual abstract suspend fun findAllRepliesWithPosterMember(postUid: Long):
      List<DiscussionPostAndPosterMember>

  @Query(`value` =
      "\n        SELECT DiscussionPost.*, Member.*\n          FROM DiscussionPost\n               LEFT JOIN Member\n                     ON Member.memberUid = DiscussionPost.posterMemberUid\n         WHERE DiscussionPost.postReplyToPostUid = :postUid\n    ")
  public actual abstract fun findAllRepliesWithPosterMemberAsFlow(postUid: Long):
      Flow<List<DiscussionPostAndPosterMember>>

  @Query(`value` =
      "\n        SELECT DiscussionPost.*, Member.*\n          FROM DiscussionPost\n               LEFT JOIN Member\n                     ON Member.memberUid = DiscussionPost.posterMemberUid\n         WHERE DiscussionPost.postUid = :postUid            \n    ")
  public actual abstract suspend fun findByUidWithPosterMember(postUid: Long):
      DiscussionPostAndPosterMember?

  @Query(`value` =
      "\n        SELECT DiscussionPost.*, Member.*\n          FROM DiscussionPost\n               LEFT JOIN Member\n                     ON Member.memberUid = DiscussionPost.posterMemberUid\n         WHERE DiscussionPost.postUid = :postUid            \n    ")
  public actual abstract fun findByUidWithPosterMemberAsFlow(postUid: Long):
      Flow<DiscussionPostAndPosterMember?>

  @Query(`value` =
      "\n        SELECT DiscussionPost.*, Member.*\n          FROM DiscussionPost\n               LEFT JOIN Member\n                     ON Member.memberUid = DiscussionPost.posterMemberUid\n         WHERE DiscussionPost.postUid IN (:postUids)    \n    ")
  public actual abstract suspend fun findByUidList(postUids: List<Long>):
      List<DiscussionPostAndPosterMember>

  @Query(`value` =
      "\n        SELECT COUNT(*) \n          FROM DiscussionPost\n         WHERE DiscussionPost.postLastModified >= :since \n    ")
  public actual abstract suspend fun getNumPostsSinceTime(since: Long): Int

  @Query(`value` =
      "\n        SELECT COUNT(*) \n          FROM DiscussionPost\n         WHERE DiscussionPost.postLastModified >= :since \n    ")
  public actual abstract suspend fun getNumPostsSinceTimeHttpOnly(since: Long): Int

  @Query(`value` =
      "\n        SELECT DiscussionPost.*\n          FROM DiscussionPost \n         WHERE DiscussionPost.postLastModified >= :since \n    ")
  public actual abstract fun findAllPostAsPagingSource(since: Long):
      PagingSource<Int, DiscussionPost>

  @Query(`value` =
      "\n        SELECT DiscussionPost.*\n          FROM DiscussionPost \n         WHERE DiscussionPost.postLastModified >= :since \n    ")
  public actual abstract fun findAllPostAsNetworkOnlyPagingSource(since: Long):
      PagingSource<Int, DiscussionPost>

  @Query(`value` =
      "\n        SELECT DiscussionPost.*\n          FROM DiscussionPost \n         WHERE DiscussionPost.postLastModified >= :since \n    ")
  public actual abstract fun findAllPostAsNetworkWithFallbackPagingSource(since: Long):
      PagingSource<Int, DiscussionPost>

  @Query(`value` =
      "\n        SELECT DiscussionPost.*\n          FROM DiscussionPost\n         WHERE DiscussionPost.postReplyToPostUid = :postUid \n           AND :nodeId != 0\n    ")
  public actual abstract suspend fun findAllRepliesForPost(postUid: Long, nodeId: Long):
      List<DiscussionPost>

  @Query(`value` =
      "\n        SELECT DiscussionPost.*,\n               (SELECT COUNT(*) \n                  FROM DiscussionPost DiscussionPostInternal\n                 WHERE DiscussionPostInternal.postReplyToPostUid = :postUid) AS numReplies\n         FROM DiscussionPost\n        WHERE DiscussionPost.postUid = :postUid\n          AND DiscussionPost.postLastModified >= :sinceTime\n    ")
  public actual abstract suspend fun findPostAndNumReplies(postUid: Long, sinceTime: Long):
      DiscussionPostAndNumReplies?

  @Query(`value` =
      "\n        SELECT DiscussionPost.*\n          FROM DiscussionPost\n         WHERE DiscussionPost.postReplyToPostUid != 0 \n    ")
  public actual abstract fun findRootRepliesAsPagingSource(): PagingSource<Int, DiscussionPost>

  @Query(`value` =
      "\n        SELECT DiscussionPost.*,\n               (SELECT COUNT(*) \n                  FROM DiscussionPost DiscussionPostInternal\n                 WHERE DiscussionPostInternal.postReplyToPostUid = DiscussionPost.postUid) AS numReplies\n         FROM DiscussionPost\n        WHERE DiscussionPost.postReplyToPostUid = 0\n    ")
  public actual abstract fun findRootPostsAndNumRepliesAsPagingSource():
      PagingSource<Int, DiscussionPostAndNumReplies>

  @Query(`value` =
      "\n        SELECT DiscussionPost.*\n          FROM DiscussionPost\n         WHERE DiscussionPost.postReplyToPostUid != 0 \n         LIMIT :limit\n        OFFSET :offset   \n    ")
  public actual abstract suspend fun findReplyPostsWithOffsetAndLimit(offset: Int, limit: Int):
      List<DiscussionPost>

  @Query(`value` =
      "\n        SELECT DiscussionPost.*,\n               (SELECT COUNT(*) \n                  FROM DiscussionPost DiscussionPostInternal\n                 WHERE DiscussionPostInternal.postReplyToPostUid = DiscussionPost.postUid) AS numReplies\n         FROM DiscussionPost\n        WHERE DiscussionPost.postReplyToPostUid = 0\n    ")
  public actual abstract fun findRootPostAndNumRepliesAsPagingSourceWithPagedParams():
      PagingSource<Int, DiscussionPostAndNumReplies>

  @Query(`value` =
      "\n        SELECT DiscussionPost.*,\n               (SELECT COUNT(*) \n                  FROM DiscussionPost DiscussionPostInternal\n                 WHERE DiscussionPostInternal.postReplyToPostUid = DiscussionPost.postUid) AS numReplies\n         FROM DiscussionPost\n        WHERE DiscussionPost.postReplyToPostUid = 0\n    ")
  public actual abstract fun findRootPostAndNumRepliesAsPagingSourceWithAsFlow():
      Flow<List<DiscussionPostAndNumReplies>>

  @Query(`value` =
      "\n        SELECT EXISTS(\n               SELECT Member.memberUid\n                 FROM Member\n                WHERE :postUid != 0\n                  AND Member.memberUid = :postUid\n                  AND :nodeId != 0\n        )\n    ")
  public actual abstract suspend fun checkNodeHasPermission(postUid: Long, nodeId: Long): Boolean

  @Query(`value` =
      "\n        SELECT DiscussionPost.*\n          FROM DiscussionPost\n         WHERE DiscussionPost.postReplyToPostUid = :postUid \n    ")
  public actual abstract suspend fun findRepliesWithAuthCheck(postUid: Long): List<DiscussionPost>

  @Query(`value` =
      "\n        SELECT DiscussionPost.*,\n               Member.firstName AS firstName,\n               Member.lastName AS lastName\n          FROM DiscussionPost\n               LEFT JOIN Member\n                         ON Member.memberUid = DiscussionPost.posterMemberUid \n         WHERE DiscussionPost.postUid = :postUid       \n    ")
  public actual abstract suspend fun getDiscussionPostAndAuthorName(postUid: Long):
      DiscussionPostAndAuthorName?

  @Query(`value` =
      "\n        SELECT DiscussionPost.*,\n               Member.firstName AS firstName,\n               Member.lastName AS lastName\n          FROM DiscussionPost\n               LEFT JOIN Member\n                         ON Member.memberUid = DiscussionPost.posterMemberUid \n         WHERE DiscussionPost.postUid = :postId       \n    ")
  public actual abstract suspend fun getDiscussionPostAndAuthorNameById(postId: Long):
      DiscussionPostAndAuthorName?

  @Update(
    entity = Any::class,
    onConflict = 3,
  )
  public actual abstract suspend fun update(post: DiscussionPost)

  @Query(`value` =
      "\n        SELECT DiscussionPost.*,\n               Member.firstName AS firstName,\n               Member.lastName AS lastName\n          FROM DiscussionPost\n               LEFT JOIN Member\n                         ON Member.memberUid = DiscussionPost.posterMemberUid \n         WHERE Member.firstName = :firstName       \n    ")
  public actual abstract suspend fun findByName(firstName: String?): List<DiscussionPost>
}
