package db3

import androidx.room.DatabaseConfiguration
import androidx.sqlite.db.SupportSQLiteOpenHelper
import com.ustadmobile.door.DoorDatabaseWrapper
import com.ustadmobile.door.log.DoorLogger
import com.ustadmobile.door.message.DoorMessageCallback
import com.ustadmobile.door.nodeevent.NodeEventManagerAndroid
import com.ustadmobile.door.nodeevent.NodeEventManagerCommon
import com.ustadmobile.door.room.RoomDatabase
import com.ustadmobile.door.util.makeDummyInvalidationHandler
import kotlin.Any
import kotlin.IllegalArgumentException
import kotlin.Long
import kotlin.String
import kotlin.Suppress
import kotlin.reflect.KClass
import kotlinx.coroutines.Runnable
import androidx.room.InvalidationTracker as AndroidxRoomInvalidationTracker
import com.ustadmobile.door.room.InvalidationTracker as DoorRoomInvalidationTracker

@Suppress("REDUNDANT_PROJECTION", "ClassName")
public class ExampleDb3_DoorWrapper(
  private val _db: ExampleDb3,
  override val nodeId: Long,
  private val messageCallback: DoorMessageCallback<ExampleDb3>,
  logger: DoorLogger,
  dbName: String,
) : ExampleDb3(), DoorDatabaseWrapper<ExampleDb3> {
  override val dbName: String
    get() = "DoorWrapper for [${_db.toString()}]"

  public val _ExampleEntity3Dao: ExampleEntity3Dao by lazy  {
        ExampleEntity3Dao_DoorWrapper(_db, _db.exampleEntity3Dao)
      }


  override val exampleEntity3Dao: ExampleEntity3Dao
    get() = _ExampleEntity3Dao

  public val _DiscussionPostDao: DiscussionPostDao by lazy  {
        DiscussionPostDao_DoorWrapper(_db, _db.discussionPostDao)
      }


  override val discussionPostDao: DiscussionPostDao
    get() = _DiscussionPostDao

  public val _MemberDao: MemberDao by lazy  {
        MemberDao_DoorWrapper(_db, _db.memberDao)
      }


  override val memberDao: MemberDao
    get() = _MemberDao

  public val _BadgeDao: BadgeDao by lazy  {
        BadgeDao_DoorWrapper(_db, _db.badgeDao)
      }


  override val badgeDao: BadgeDao
    get() = _BadgeDao

  public val _StatementEntityDao: StatementEntityDao by lazy  {
        StatementEntityDao_DoorWrapper(_db, _db.statementEntityDao)
      }


  override val statementEntityDao: StatementEntityDao
    get() = _StatementEntityDao

  override val realDatabase: RoomDatabase
    get() = _db

  override val nodeEventManager: NodeEventManagerCommon<ExampleDb3> = NodeEventManagerAndroid(_db,
      messageCallback, logger, dbName)


  override val invalidationTracker: DoorRoomInvalidationTracker
    get() = _db.invalidationTracker

  override fun clearAllTables() {
    _db.clearAllTables()
  }

  override fun runInTransaction(runnable: Runnable) {
    _db.runInTransaction(runnable)
  }

  protected override fun createOpenHelper(config: DatabaseConfiguration): SupportSQLiteOpenHelper =
      throw IllegalAccessException("Cannot use open helper on repository")

  protected override fun createInvalidationTracker(): AndroidxRoomInvalidationTracker =
      makeDummyInvalidationHandler()

  @Suppress("UNCHECKED_CAST")
  override fun <T : Any> getDaoByClass(daoClass: KClass<T>): T = when(daoClass) {
    ExampleEntity3Dao::class -> exampleEntity3Dao as T
    DiscussionPostDao::class -> discussionPostDao as T
    MemberDao::class -> memberDao as T
    BadgeDao::class -> badgeDao as T
    StatementEntityDao::class -> statementEntityDao as T
    else -> throw IllegalArgumentException("No such DAO on this DB")
  }

  override fun close() {
    _db.close()
  }
}
