package db3

import androidx.room.Dao
import androidx.room.Insert
import androidx.room.Query
import kotlin.Any
import kotlin.Boolean
import kotlin.Long
import kotlin.String

@Dao
public actual abstract class MemberDao {
  @Insert(
    entity = Any::class,
    onConflict = 3,
  )
  public actual abstract suspend fun insertAsync(member: Member): Long

  @Query(`value` =
      "\n        SELECT Member.*\n          FROM Member\n         WHERE Member.memberUid = :memberUid \n    ")
  public actual abstract suspend fun findByUid(memberUid: Long): Member?

  @Query(`value` =
      "\n        SELECT EXISTS(\n               SELECT Member.memberUid\n                 FROM Member\n                WHERE :postUid != 0\n                  AND Member.memberUid = :postUid\n                  AND :nodeId != 0\n        )\n    ")
  public actual abstract suspend fun checkMemberNodeHasPermission(postUid: Long, nodeId: Long):
      Boolean

  @Query(`value` =
      "\n        SELECT Member.*\n          FROM Member\n         WHERE Member.memberUid = \n               (SELECT DiscussionPost.posterMemberUid\n                  FROM DiscussionPost\n                 WHERE DiscussionPost.postUid = :postUid)\n    ")
  public actual abstract suspend fun findAuthorByPostUid(postUid: Long): Member?

  @Query(`value` =
      "\n        SELECT Member.firstName\n          FROM Member\n         WHERE Member.memberUid = :memberUid \n    ")
  public actual abstract suspend fun getFirstNameByMemberId(memberUid: Long): String?
}
