package db3

import androidx.paging.PagingSource
import com.ustadmobile.door.DoorDbType
import com.ustadmobile.door.EntityInsertionAdapter
import com.ustadmobile.door.PreparedStatementConfig
import com.ustadmobile.door.ext.createArrayOrProxyArrayOf
import com.ustadmobile.door.ext.prepareAndUseStatementAsync
import com.ustadmobile.door.flow.doorFlow
import com.ustadmobile.door.jdbc.PreparedStatement
import com.ustadmobile.door.jdbc.ext.executeQueryAsyncKmp
import com.ustadmobile.door.jdbc.ext.executeUpdateAsyncKmp
import com.ustadmobile.door.jdbc.ext.getBooleanNullable
import com.ustadmobile.door.jdbc.ext.mapNextRow
import com.ustadmobile.door.jdbc.ext.mapRows
import com.ustadmobile.door.jdbc.ext.setBooleanNullable
import com.ustadmobile.door.jdbc.ext.useResults
import com.ustadmobile.door.paging.DoorLimitOffsetPagingSource
import com.ustadmobile.door.room.RoomDatabase
import kotlin.Boolean
import kotlin.Int
import kotlin.Long
import kotlin.String
import kotlin.collections.List
import kotlinx.coroutines.flow.Flow

public class DiscussionPostDao_JdbcImpl(
  public val _db: RoomDatabase,
) : DiscussionPostDao() {
  public val _insertAdapterDiscussionPost_abort: EntityInsertionAdapter<DiscussionPost> = object :
      EntityInsertionAdapter<DiscussionPost>(_db) {
    override fun makeSql(returnsId: Boolean): String =
        "INSERT INTO DiscussionPost (postUid, postReplyToPostUid, postTitle, postText, postLastModified, posterMemberUid, isBanned) VALUES(?, ?, ?, ?, ?, ?, ?)"

    override fun bindPreparedStmtToEntity(stmt: PreparedStatement, entity: DiscussionPost) {
      if(entity.postUid == 0L) {
        stmt.setObject(1, null)
      } else {
        stmt.setLong(1, entity.postUid)
      }
      stmt.setLong(2, entity.postReplyToPostUid)
      stmt.setString(3, entity.postTitle)
      stmt.setString(4, entity.postText)
      stmt.setLong(5, entity.postLastModified)
      stmt.setLong(6, entity.posterMemberUid)
      stmt.setBooleanNullable(7, entity.isBanned)
    }
  }

  override suspend fun insertAsync(post: DiscussionPost): Long {
    val _retVal = _insertAdapterDiscussionPost_abort.insertAndReturnIdAsync(post)
    return _retVal
  }

  override suspend fun update(post: DiscussionPost) {
    val _sql =
        "UPDATE DiscussionPost SET postReplyToPostUid = ?, postTitle = ?, postText = ?, postLastModified = ?, posterMemberUid = ?, isBanned = ? WHERE postUid = ?"
    _db.prepareAndUseStatementAsync(_sql) {
       _stmt ->
      _stmt.setLong(1, post.postReplyToPostUid)
      _stmt.setString(2, post.postTitle)
      _stmt.setString(3, post.postText)
      _stmt.setLong(4, post.postLastModified)
      _stmt.setLong(5, post.posterMemberUid)
      _stmt.setBooleanNullable(6, post.isBanned)
      _stmt.setLong(7, post.postUid)
      _stmt.executeUpdateAsyncKmp()
    }
  }

  override suspend fun findByUid(postUid: Long): DiscussionPost? =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT DiscussionPost.*
    |          FROM DiscussionPost
    |         WHERE DiscussionPost.postUid = CAST(? AS BIGINT)
    |    
    """.trimMargin(),
    postgreSql = """
    |
    |        SELECT DiscussionPost.*
    |          FROM DiscussionPost
    |         WHERE DiscussionPost.postUid = ?
    |    
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,postUid)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow(null) {
        val _tmp_postUid = _result.getLong("postUid")
        val _tmp_postReplyToPostUid = _result.getLong("postReplyToPostUid")
        val _tmp_postTitle = _result.getString("postTitle")
        val _tmp_postText = _result.getString("postText")
        val _tmp_postLastModified = _result.getLong("postLastModified")
        val _tmp_posterMemberUid = _result.getLong("posterMemberUid")
        val _tmp_isBanned = _result.getBooleanNullable("isBanned")
        DiscussionPost().apply {
          this.postUid = _tmp_postUid
          this.postReplyToPostUid = _tmp_postReplyToPostUid
          this.postTitle = _tmp_postTitle
          this.postText = _tmp_postText
          this.postLastModified = _tmp_postLastModified
          this.posterMemberUid = _tmp_posterMemberUid
          this.isBanned = _tmp_isBanned
        }
      }
    }
  }

  override suspend fun findByUidAsyncOverHttp(postUid: Long): DiscussionPost? =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT DiscussionPost.*
    |          FROM DiscussionPost
    |         WHERE DiscussionPost.postUid = CAST(? AS BIGINT)
    |    
    """.trimMargin(),
    postgreSql = """
    |
    |        SELECT DiscussionPost.*
    |          FROM DiscussionPost
    |         WHERE DiscussionPost.postUid = ?
    |    
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,postUid)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow(null) {
        val _tmp_postUid = _result.getLong("postUid")
        val _tmp_postReplyToPostUid = _result.getLong("postReplyToPostUid")
        val _tmp_postTitle = _result.getString("postTitle")
        val _tmp_postText = _result.getString("postText")
        val _tmp_postLastModified = _result.getLong("postLastModified")
        val _tmp_posterMemberUid = _result.getLong("posterMemberUid")
        val _tmp_isBanned = _result.getBooleanNullable("isBanned")
        DiscussionPost().apply {
          this.postUid = _tmp_postUid
          this.postReplyToPostUid = _tmp_postReplyToPostUid
          this.postTitle = _tmp_postTitle
          this.postText = _tmp_postText
          this.postLastModified = _tmp_postLastModified
          this.posterMemberUid = _tmp_posterMemberUid
          this.isBanned = _tmp_isBanned
        }
      }
    }
  }

  override suspend fun findAllRepliesWithPosterMember(postUid: Long):
      List<DiscussionPostAndPosterMember> = _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT DiscussionPost.*, Member.*
    |          FROM DiscussionPost
    |               LEFT JOIN Member
    |                     ON Member.memberUid = DiscussionPost.posterMemberUid
    |         WHERE DiscussionPost.postReplyToPostUid = CAST(? AS BIGINT)
    |    
    """.trimMargin(),
    postgreSql = """
    |
    |        SELECT DiscussionPost.*, Member.*
    |          FROM DiscussionPost
    |               LEFT JOIN Member
    |                     ON Member.memberUid = DiscussionPost.posterMemberUid
    |         WHERE DiscussionPost.postReplyToPostUid = ?
    |    
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,postUid)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapRows {
        var _tmp_DiscussionPost_nullCount = 0
        val _tmp_postUid = _result.getLong("postUid")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_postReplyToPostUid = _result.getLong("postReplyToPostUid")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_postTitle = _result.getString("postTitle")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_postText = _result.getString("postText")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_postLastModified = _result.getLong("postLastModified")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_posterMemberUid = _result.getLong("posterMemberUid")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_isBanned = _result.getBooleanNullable("isBanned")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_DiscussionPost_isAllNull = _tmp_DiscussionPost_nullCount == 7
        var _tmp_Member_nullCount = 0
        val _tmp_memberUid = _result.getLong("memberUid")
        if(_result.wasNull()) _tmp_Member_nullCount++
        val _tmp_firstName = _result.getString("firstName")
        if(_result.wasNull()) _tmp_Member_nullCount++
        val _tmp_lastName = _result.getString("lastName")
        if(_result.wasNull()) _tmp_Member_nullCount++
        val _tmp_memberLastModified = _result.getLong("memberLastModified")
        if(_result.wasNull()) _tmp_Member_nullCount++
        val _tmp_Member_isAllNull = _tmp_Member_nullCount == 4
        DiscussionPostAndPosterMember().apply {
          if(!_tmp_DiscussionPost_isAllNull) {
            this.discussionPost = DiscussionPost().apply {
              this.postUid = _tmp_postUid
              this.postReplyToPostUid = _tmp_postReplyToPostUid
              this.postTitle = _tmp_postTitle
              this.postText = _tmp_postText
              this.postLastModified = _tmp_postLastModified
              this.posterMemberUid = _tmp_posterMemberUid
              this.isBanned = _tmp_isBanned
            }
          }
          if(!_tmp_Member_isAllNull) {
            this.posterMember = Member().apply {
              this.memberUid = _tmp_memberUid
              this.firstName = _tmp_firstName
              this.lastName = _tmp_lastName
              this.memberLastModified = _tmp_memberLastModified
            }
          }
        }
      }
    }
  }

  override fun findAllRepliesWithPosterMemberAsFlow(postUid: Long):
      Flow<List<DiscussionPostAndPosterMember>> = _db.doorFlow(arrayOf("Member",
      "DiscussionPost")) {
    _db.prepareAndUseStatementAsync(PreparedStatementConfig(
      sql = """
      |
      |        SELECT DiscussionPost.*, Member.*
      |          FROM DiscussionPost
      |               LEFT JOIN Member
      |                     ON Member.memberUid = DiscussionPost.posterMemberUid
      |         WHERE DiscussionPost.postReplyToPostUid = CAST(? AS BIGINT)
      |    
      """.trimMargin(),
      postgreSql = """
      |
      |        SELECT DiscussionPost.*, Member.*
      |          FROM DiscussionPost
      |               LEFT JOIN Member
      |                     ON Member.memberUid = DiscussionPost.posterMemberUid
      |         WHERE DiscussionPost.postReplyToPostUid = ?
      |    
      |""".trimMargin(),
      readOnly = true,)
    ) { _stmt -> 
      _stmt.setLong(1,postUid)
      _stmt.executeQueryAsyncKmp().useResults{ _result -> 
        _result.mapRows {
          var _tmp_DiscussionPost_nullCount = 0
          val _tmp_postUid = _result.getLong("postUid")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_postReplyToPostUid = _result.getLong("postReplyToPostUid")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_postTitle = _result.getString("postTitle")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_postText = _result.getString("postText")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_postLastModified = _result.getLong("postLastModified")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_posterMemberUid = _result.getLong("posterMemberUid")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_isBanned = _result.getBooleanNullable("isBanned")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_DiscussionPost_isAllNull = _tmp_DiscussionPost_nullCount == 7
          var _tmp_Member_nullCount = 0
          val _tmp_memberUid = _result.getLong("memberUid")
          if(_result.wasNull()) _tmp_Member_nullCount++
          val _tmp_firstName = _result.getString("firstName")
          if(_result.wasNull()) _tmp_Member_nullCount++
          val _tmp_lastName = _result.getString("lastName")
          if(_result.wasNull()) _tmp_Member_nullCount++
          val _tmp_memberLastModified = _result.getLong("memberLastModified")
          if(_result.wasNull()) _tmp_Member_nullCount++
          val _tmp_Member_isAllNull = _tmp_Member_nullCount == 4
          DiscussionPostAndPosterMember().apply {
            if(!_tmp_DiscussionPost_isAllNull) {
              this.discussionPost = DiscussionPost().apply {
                this.postUid = _tmp_postUid
                this.postReplyToPostUid = _tmp_postReplyToPostUid
                this.postTitle = _tmp_postTitle
                this.postText = _tmp_postText
                this.postLastModified = _tmp_postLastModified
                this.posterMemberUid = _tmp_posterMemberUid
                this.isBanned = _tmp_isBanned
              }
            }
            if(!_tmp_Member_isAllNull) {
              this.posterMember = Member().apply {
                this.memberUid = _tmp_memberUid
                this.firstName = _tmp_firstName
                this.lastName = _tmp_lastName
                this.memberLastModified = _tmp_memberLastModified
              }
            }
          }
        }
      }
    }
  }

  override suspend fun findByUidWithPosterMember(postUid: Long): DiscussionPostAndPosterMember? =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT DiscussionPost.*, Member.*
    |          FROM DiscussionPost
    |               LEFT JOIN Member
    |                     ON Member.memberUid = DiscussionPost.posterMemberUid
    |         WHERE DiscussionPost.postUid = CAST(? AS BIGINT)            
    |    
    """.trimMargin(),
    postgreSql = """
    |
    |        SELECT DiscussionPost.*, Member.*
    |          FROM DiscussionPost
    |               LEFT JOIN Member
    |                     ON Member.memberUid = DiscussionPost.posterMemberUid
    |         WHERE DiscussionPost.postUid = ?            
    |    
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,postUid)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow(null) {
        var _tmp_DiscussionPost_nullCount = 0
        val _tmp_postUid = _result.getLong("postUid")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_postReplyToPostUid = _result.getLong("postReplyToPostUid")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_postTitle = _result.getString("postTitle")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_postText = _result.getString("postText")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_postLastModified = _result.getLong("postLastModified")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_posterMemberUid = _result.getLong("posterMemberUid")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_isBanned = _result.getBooleanNullable("isBanned")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_DiscussionPost_isAllNull = _tmp_DiscussionPost_nullCount == 7
        var _tmp_Member_nullCount = 0
        val _tmp_memberUid = _result.getLong("memberUid")
        if(_result.wasNull()) _tmp_Member_nullCount++
        val _tmp_firstName = _result.getString("firstName")
        if(_result.wasNull()) _tmp_Member_nullCount++
        val _tmp_lastName = _result.getString("lastName")
        if(_result.wasNull()) _tmp_Member_nullCount++
        val _tmp_memberLastModified = _result.getLong("memberLastModified")
        if(_result.wasNull()) _tmp_Member_nullCount++
        val _tmp_Member_isAllNull = _tmp_Member_nullCount == 4
        DiscussionPostAndPosterMember().apply {
          if(!_tmp_DiscussionPost_isAllNull) {
            this.discussionPost = DiscussionPost().apply {
              this.postUid = _tmp_postUid
              this.postReplyToPostUid = _tmp_postReplyToPostUid
              this.postTitle = _tmp_postTitle
              this.postText = _tmp_postText
              this.postLastModified = _tmp_postLastModified
              this.posterMemberUid = _tmp_posterMemberUid
              this.isBanned = _tmp_isBanned
            }
          }
          if(!_tmp_Member_isAllNull) {
            this.posterMember = Member().apply {
              this.memberUid = _tmp_memberUid
              this.firstName = _tmp_firstName
              this.lastName = _tmp_lastName
              this.memberLastModified = _tmp_memberLastModified
            }
          }
        }
      }
    }
  }

  override fun findByUidWithPosterMemberAsFlow(postUid: Long): Flow<DiscussionPostAndPosterMember?>
      = _db.doorFlow(arrayOf("Member", "DiscussionPost")) {
    _db.prepareAndUseStatementAsync(PreparedStatementConfig(
      sql = """
      |
      |        SELECT DiscussionPost.*, Member.*
      |          FROM DiscussionPost
      |               LEFT JOIN Member
      |                     ON Member.memberUid = DiscussionPost.posterMemberUid
      |         WHERE DiscussionPost.postUid = CAST(? AS BIGINT)            
      |    
      """.trimMargin(),
      postgreSql = """
      |
      |        SELECT DiscussionPost.*, Member.*
      |          FROM DiscussionPost
      |               LEFT JOIN Member
      |                     ON Member.memberUid = DiscussionPost.posterMemberUid
      |         WHERE DiscussionPost.postUid = ?            
      |    
      |""".trimMargin(),
      readOnly = true,)
    ) { _stmt -> 
      _stmt.setLong(1,postUid)
      _stmt.executeQueryAsyncKmp().useResults{ _result -> 
        _result.mapNextRow(null) {
          var _tmp_DiscussionPost_nullCount = 0
          val _tmp_postUid = _result.getLong("postUid")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_postReplyToPostUid = _result.getLong("postReplyToPostUid")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_postTitle = _result.getString("postTitle")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_postText = _result.getString("postText")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_postLastModified = _result.getLong("postLastModified")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_posterMemberUid = _result.getLong("posterMemberUid")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_isBanned = _result.getBooleanNullable("isBanned")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_DiscussionPost_isAllNull = _tmp_DiscussionPost_nullCount == 7
          var _tmp_Member_nullCount = 0
          val _tmp_memberUid = _result.getLong("memberUid")
          if(_result.wasNull()) _tmp_Member_nullCount++
          val _tmp_firstName = _result.getString("firstName")
          if(_result.wasNull()) _tmp_Member_nullCount++
          val _tmp_lastName = _result.getString("lastName")
          if(_result.wasNull()) _tmp_Member_nullCount++
          val _tmp_memberLastModified = _result.getLong("memberLastModified")
          if(_result.wasNull()) _tmp_Member_nullCount++
          val _tmp_Member_isAllNull = _tmp_Member_nullCount == 4
          DiscussionPostAndPosterMember().apply {
            if(!_tmp_DiscussionPost_isAllNull) {
              this.discussionPost = DiscussionPost().apply {
                this.postUid = _tmp_postUid
                this.postReplyToPostUid = _tmp_postReplyToPostUid
                this.postTitle = _tmp_postTitle
                this.postText = _tmp_postText
                this.postLastModified = _tmp_postLastModified
                this.posterMemberUid = _tmp_posterMemberUid
                this.isBanned = _tmp_isBanned
              }
            }
            if(!_tmp_Member_isAllNull) {
              this.posterMember = Member().apply {
                this.memberUid = _tmp_memberUid
                this.firstName = _tmp_firstName
                this.lastName = _tmp_lastName
                this.memberLastModified = _tmp_memberLastModified
              }
            }
          }
        }
      }
    }
  }

  override suspend fun findByUidList(postUids: List<Long>): List<DiscussionPostAndPosterMember> =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT DiscussionPost.*, Member.*
    |          FROM DiscussionPost
    |               LEFT JOIN Member
    |                     ON Member.memberUid = DiscussionPost.posterMemberUid
    |         WHERE DiscussionPost.postUid IN (?)    
    |    
    """.trimMargin(),
    hasListParams = true,
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setArray(1, _stmt.getConnection().createArrayOrProxyArrayOf("BIGINT",
        postUids.toTypedArray()))
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapRows {
        var _tmp_DiscussionPost_nullCount = 0
        val _tmp_postUid = _result.getLong("postUid")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_postReplyToPostUid = _result.getLong("postReplyToPostUid")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_postTitle = _result.getString("postTitle")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_postText = _result.getString("postText")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_postLastModified = _result.getLong("postLastModified")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_posterMemberUid = _result.getLong("posterMemberUid")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_isBanned = _result.getBooleanNullable("isBanned")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_DiscussionPost_isAllNull = _tmp_DiscussionPost_nullCount == 7
        var _tmp_Member_nullCount = 0
        val _tmp_memberUid = _result.getLong("memberUid")
        if(_result.wasNull()) _tmp_Member_nullCount++
        val _tmp_firstName = _result.getString("firstName")
        if(_result.wasNull()) _tmp_Member_nullCount++
        val _tmp_lastName = _result.getString("lastName")
        if(_result.wasNull()) _tmp_Member_nullCount++
        val _tmp_memberLastModified = _result.getLong("memberLastModified")
        if(_result.wasNull()) _tmp_Member_nullCount++
        val _tmp_Member_isAllNull = _tmp_Member_nullCount == 4
        DiscussionPostAndPosterMember().apply {
          if(!_tmp_DiscussionPost_isAllNull) {
            this.discussionPost = DiscussionPost().apply {
              this.postUid = _tmp_postUid
              this.postReplyToPostUid = _tmp_postReplyToPostUid
              this.postTitle = _tmp_postTitle
              this.postText = _tmp_postText
              this.postLastModified = _tmp_postLastModified
              this.posterMemberUid = _tmp_posterMemberUid
              this.isBanned = _tmp_isBanned
            }
          }
          if(!_tmp_Member_isAllNull) {
            this.posterMember = Member().apply {
              this.memberUid = _tmp_memberUid
              this.firstName = _tmp_firstName
              this.lastName = _tmp_lastName
              this.memberLastModified = _tmp_memberLastModified
            }
          }
        }
      }
    }
  }

  override suspend fun getNumPostsSinceTime(since: Long): Int =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT COUNT(*) 
    |          FROM DiscussionPost
    |         WHERE DiscussionPost.postLastModified >= CAST(? AS BIGINT) 
    |    
    """.trimMargin(),
    postgreSql = """
    |
    |        SELECT COUNT(*) 
    |          FROM DiscussionPost
    |         WHERE DiscussionPost.postLastModified >= ? 
    |    
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,since)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow(0) {
        _result.getInt(1)
      }
    }
  }

  override suspend fun getNumPostsSinceTimeHttpOnly(since: Long): Int =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT COUNT(*) 
    |          FROM DiscussionPost
    |         WHERE DiscussionPost.postLastModified >= CAST(? AS BIGINT) 
    |    
    """.trimMargin(),
    postgreSql = """
    |
    |        SELECT COUNT(*) 
    |          FROM DiscussionPost
    |         WHERE DiscussionPost.postLastModified >= ? 
    |    
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,since)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow(0) {
        _result.getInt(1)
      }
    }
  }

  override fun findAllPostAsPagingSource(since: Long): PagingSource<Int, DiscussionPost> = object :
      DoorLimitOffsetPagingSource<DiscussionPost>(db = _db
  , tableNames = arrayOf("DiscussionPost")
  ) {
    override suspend fun loadRows(_limit: Int, _offset: Int): List<DiscussionPost> =
        _db.prepareAndUseStatementAsync(PreparedStatementConfig(
      sql = """
      |SELECT * FROM (
      |        SELECT DiscussionPost.*
      |          FROM DiscussionPost 
      |         WHERE DiscussionPost.postLastModified >= CAST(? AS BIGINT) 
      |    ) AS _PagingData LIMIT ? OFFSET ?
      """.trimMargin(),
      postgreSql = """
      |SELECT * FROM (
      |        SELECT DiscussionPost.*
      |          FROM DiscussionPost 
      |         WHERE DiscussionPost.postLastModified >= ? 
      |    ) AS _PagingData LIMIT ? OFFSET ?
      |""".trimMargin(),
      readOnly = true,)
    ) { _stmt -> 
      _stmt.setLong(1,since)
      _stmt.setInt(2,_limit)
      _stmt.setInt(3,_offset)
      _stmt.executeQueryAsyncKmp().useResults{ _result -> 
        _result.mapRows {
          val _tmp_postUid = _result.getLong("postUid")
          val _tmp_postReplyToPostUid = _result.getLong("postReplyToPostUid")
          val _tmp_postTitle = _result.getString("postTitle")
          val _tmp_postText = _result.getString("postText")
          val _tmp_postLastModified = _result.getLong("postLastModified")
          val _tmp_posterMemberUid = _result.getLong("posterMemberUid")
          val _tmp_isBanned = _result.getBooleanNullable("isBanned")
          DiscussionPost().apply {
            this.postUid = _tmp_postUid
            this.postReplyToPostUid = _tmp_postReplyToPostUid
            this.postTitle = _tmp_postTitle
            this.postText = _tmp_postText
            this.postLastModified = _tmp_postLastModified
            this.posterMemberUid = _tmp_posterMemberUid
            this.isBanned = _tmp_isBanned
          }
        }
      }
    }

    override suspend fun countRows(): Int = _db.prepareAndUseStatementAsync(PreparedStatementConfig(
      sql = """
      |SELECT COUNT(*) FROM (
      |        SELECT DiscussionPost.*
      |          FROM DiscussionPost 
      |         WHERE DiscussionPost.postLastModified >= CAST(? AS BIGINT) 
      |    ) AS _PagingCount
      """.trimMargin(),
      postgreSql = """
      |SELECT COUNT(*) FROM (
      |        SELECT DiscussionPost.*
      |          FROM DiscussionPost 
      |         WHERE DiscussionPost.postLastModified >= ? 
      |    ) AS _PagingCount
      |""".trimMargin(),
      readOnly = true,)
    ) { _stmt -> 
      _stmt.setLong(1,since)
      _stmt.executeQueryAsyncKmp().useResults{ _result -> 
        _result.mapNextRow(0) {
          _result.getInt(1)
        }
      }
    }
  }

  override fun findAllPostAsNetworkOnlyPagingSource(since: Long): PagingSource<Int, DiscussionPost>
      = object : DoorLimitOffsetPagingSource<DiscussionPost>(db = _db
  , tableNames = arrayOf("DiscussionPost")
  ) {
    override suspend fun loadRows(_limit: Int, _offset: Int): List<DiscussionPost> =
        _db.prepareAndUseStatementAsync(PreparedStatementConfig(
      sql = """
      |SELECT * FROM (
      |        SELECT DiscussionPost.*
      |          FROM DiscussionPost 
      |         WHERE DiscussionPost.postLastModified >= CAST(? AS BIGINT) 
      |    ) AS _PagingData LIMIT ? OFFSET ?
      """.trimMargin(),
      postgreSql = """
      |SELECT * FROM (
      |        SELECT DiscussionPost.*
      |          FROM DiscussionPost 
      |         WHERE DiscussionPost.postLastModified >= ? 
      |    ) AS _PagingData LIMIT ? OFFSET ?
      |""".trimMargin(),
      readOnly = true,)
    ) { _stmt -> 
      _stmt.setLong(1,since)
      _stmt.setInt(2,_limit)
      _stmt.setInt(3,_offset)
      _stmt.executeQueryAsyncKmp().useResults{ _result -> 
        _result.mapRows {
          val _tmp_postUid = _result.getLong("postUid")
          val _tmp_postReplyToPostUid = _result.getLong("postReplyToPostUid")
          val _tmp_postTitle = _result.getString("postTitle")
          val _tmp_postText = _result.getString("postText")
          val _tmp_postLastModified = _result.getLong("postLastModified")
          val _tmp_posterMemberUid = _result.getLong("posterMemberUid")
          val _tmp_isBanned = _result.getBooleanNullable("isBanned")
          DiscussionPost().apply {
            this.postUid = _tmp_postUid
            this.postReplyToPostUid = _tmp_postReplyToPostUid
            this.postTitle = _tmp_postTitle
            this.postText = _tmp_postText
            this.postLastModified = _tmp_postLastModified
            this.posterMemberUid = _tmp_posterMemberUid
            this.isBanned = _tmp_isBanned
          }
        }
      }
    }

    override suspend fun countRows(): Int = _db.prepareAndUseStatementAsync(PreparedStatementConfig(
      sql = """
      |SELECT COUNT(*) FROM (
      |        SELECT DiscussionPost.*
      |          FROM DiscussionPost 
      |         WHERE DiscussionPost.postLastModified >= CAST(? AS BIGINT) 
      |    ) AS _PagingCount
      """.trimMargin(),
      postgreSql = """
      |SELECT COUNT(*) FROM (
      |        SELECT DiscussionPost.*
      |          FROM DiscussionPost 
      |         WHERE DiscussionPost.postLastModified >= ? 
      |    ) AS _PagingCount
      |""".trimMargin(),
      readOnly = true,)
    ) { _stmt -> 
      _stmt.setLong(1,since)
      _stmt.executeQueryAsyncKmp().useResults{ _result -> 
        _result.mapNextRow(0) {
          _result.getInt(1)
        }
      }
    }
  }

  override fun findAllPostAsNetworkWithFallbackPagingSource(since: Long):
      PagingSource<Int, DiscussionPost> = object : DoorLimitOffsetPagingSource<DiscussionPost>(db =
      _db
  , tableNames = arrayOf("DiscussionPost")
  ) {
    override suspend fun loadRows(_limit: Int, _offset: Int): List<DiscussionPost> =
        _db.prepareAndUseStatementAsync(PreparedStatementConfig(
      sql = """
      |SELECT * FROM (
      |        SELECT DiscussionPost.*
      |          FROM DiscussionPost 
      |         WHERE DiscussionPost.postLastModified >= CAST(? AS BIGINT) 
      |    ) AS _PagingData LIMIT ? OFFSET ?
      """.trimMargin(),
      postgreSql = """
      |SELECT * FROM (
      |        SELECT DiscussionPost.*
      |          FROM DiscussionPost 
      |         WHERE DiscussionPost.postLastModified >= ? 
      |    ) AS _PagingData LIMIT ? OFFSET ?
      |""".trimMargin(),
      readOnly = true,)
    ) { _stmt -> 
      _stmt.setLong(1,since)
      _stmt.setInt(2,_limit)
      _stmt.setInt(3,_offset)
      _stmt.executeQueryAsyncKmp().useResults{ _result -> 
        _result.mapRows {
          val _tmp_postUid = _result.getLong("postUid")
          val _tmp_postReplyToPostUid = _result.getLong("postReplyToPostUid")
          val _tmp_postTitle = _result.getString("postTitle")
          val _tmp_postText = _result.getString("postText")
          val _tmp_postLastModified = _result.getLong("postLastModified")
          val _tmp_posterMemberUid = _result.getLong("posterMemberUid")
          val _tmp_isBanned = _result.getBooleanNullable("isBanned")
          DiscussionPost().apply {
            this.postUid = _tmp_postUid
            this.postReplyToPostUid = _tmp_postReplyToPostUid
            this.postTitle = _tmp_postTitle
            this.postText = _tmp_postText
            this.postLastModified = _tmp_postLastModified
            this.posterMemberUid = _tmp_posterMemberUid
            this.isBanned = _tmp_isBanned
          }
        }
      }
    }

    override suspend fun countRows(): Int = _db.prepareAndUseStatementAsync(PreparedStatementConfig(
      sql = """
      |SELECT COUNT(*) FROM (
      |        SELECT DiscussionPost.*
      |          FROM DiscussionPost 
      |         WHERE DiscussionPost.postLastModified >= CAST(? AS BIGINT) 
      |    ) AS _PagingCount
      """.trimMargin(),
      postgreSql = """
      |SELECT COUNT(*) FROM (
      |        SELECT DiscussionPost.*
      |          FROM DiscussionPost 
      |         WHERE DiscussionPost.postLastModified >= ? 
      |    ) AS _PagingCount
      |""".trimMargin(),
      readOnly = true,)
    ) { _stmt -> 
      _stmt.setLong(1,since)
      _stmt.executeQueryAsyncKmp().useResults{ _result -> 
        _result.mapNextRow(0) {
          _result.getInt(1)
        }
      }
    }
  }

  override suspend fun findAllRepliesForPost(postUid: Long, nodeId: Long): List<DiscussionPost> =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT DiscussionPost.*
    |          FROM DiscussionPost
    |         WHERE DiscussionPost.postReplyToPostUid = CAST(? AS BIGINT) 
    |           AND CAST(? AS BIGINT) != 0
    |    
    """.trimMargin(),
    postgreSql = """
    |
    |        SELECT DiscussionPost.*
    |          FROM DiscussionPost
    |         WHERE DiscussionPost.postReplyToPostUid = ? 
    |           AND ? != 0
    |    
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,postUid)
    _stmt.setLong(2,nodeId)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapRows {
        val _tmp_postUid = _result.getLong("postUid")
        val _tmp_postReplyToPostUid = _result.getLong("postReplyToPostUid")
        val _tmp_postTitle = _result.getString("postTitle")
        val _tmp_postText = _result.getString("postText")
        val _tmp_postLastModified = _result.getLong("postLastModified")
        val _tmp_posterMemberUid = _result.getLong("posterMemberUid")
        val _tmp_isBanned = _result.getBooleanNullable("isBanned")
        DiscussionPost().apply {
          this.postUid = _tmp_postUid
          this.postReplyToPostUid = _tmp_postReplyToPostUid
          this.postTitle = _tmp_postTitle
          this.postText = _tmp_postText
          this.postLastModified = _tmp_postLastModified
          this.posterMemberUid = _tmp_posterMemberUid
          this.isBanned = _tmp_isBanned
        }
      }
    }
  }

  override suspend fun findPostAndNumReplies(postUid: Long, sinceTime: Long):
      DiscussionPostAndNumReplies? = _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT DiscussionPost.*,
    |               (SELECT COUNT(*) 
    |                  FROM DiscussionPost DiscussionPostInternal
    |                 WHERE DiscussionPostInternal.postReplyToPostUid = CAST(? AS BIGINT)) AS numReplies
    |         FROM DiscussionPost
    |        WHERE DiscussionPost.postUid = CAST(? AS BIGINT)
    |          AND DiscussionPost.postLastModified >= CAST(? AS BIGINT)
    |    
    """.trimMargin(),
    postgreSql = """
    |
    |        SELECT DiscussionPost.*,
    |               (SELECT COUNT(*) 
    |                  FROM DiscussionPost DiscussionPostInternal
    |                 WHERE DiscussionPostInternal.postReplyToPostUid = ?) AS numReplies
    |         FROM DiscussionPost
    |        WHERE DiscussionPost.postUid = ?
    |          AND DiscussionPost.postLastModified >= ?
    |    
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,postUid)
    _stmt.setLong(2,postUid)
    _stmt.setLong(3,sinceTime)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow(null) {
        val _tmp_numReplies = _result.getInt("numReplies")
        var _tmp_DiscussionPost_nullCount = 0
        val _tmp_postUid = _result.getLong("postUid")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_postReplyToPostUid = _result.getLong("postReplyToPostUid")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_postTitle = _result.getString("postTitle")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_postText = _result.getString("postText")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_postLastModified = _result.getLong("postLastModified")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_posterMemberUid = _result.getLong("posterMemberUid")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_isBanned = _result.getBooleanNullable("isBanned")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_DiscussionPost_isAllNull = _tmp_DiscussionPost_nullCount == 7
        DiscussionPostAndNumReplies().apply {
          this.numReplies = _tmp_numReplies
          if(!_tmp_DiscussionPost_isAllNull) {
            this.discussionPost = DiscussionPost().apply {
              this.postUid = _tmp_postUid
              this.postReplyToPostUid = _tmp_postReplyToPostUid
              this.postTitle = _tmp_postTitle
              this.postText = _tmp_postText
              this.postLastModified = _tmp_postLastModified
              this.posterMemberUid = _tmp_posterMemberUid
              this.isBanned = _tmp_isBanned
            }
          }
        }
      }
    }
  }

  override fun findRootRepliesAsPagingSource(): PagingSource<Int, DiscussionPost> = object :
      DoorLimitOffsetPagingSource<DiscussionPost>(db = _db
  , tableNames = arrayOf("DiscussionPost")
  ) {
    override suspend fun loadRows(_limit: Int, _offset: Int): List<DiscussionPost> =
        _db.prepareAndUseStatementAsync(PreparedStatementConfig(
      sql = """
      |SELECT * FROM (
      |        SELECT DiscussionPost.*
      |          FROM DiscussionPost
      |         WHERE DiscussionPost.postReplyToPostUid != 0 
      |    ) AS _PagingData LIMIT ? OFFSET ?
      """.trimMargin(),
      readOnly = true,)
    ) { _stmt -> 
      _stmt.setInt(1,_limit)
      _stmt.setInt(2,_offset)
      _stmt.executeQueryAsyncKmp().useResults{ _result -> 
        _result.mapRows {
          val _tmp_postUid = _result.getLong("postUid")
          val _tmp_postReplyToPostUid = _result.getLong("postReplyToPostUid")
          val _tmp_postTitle = _result.getString("postTitle")
          val _tmp_postText = _result.getString("postText")
          val _tmp_postLastModified = _result.getLong("postLastModified")
          val _tmp_posterMemberUid = _result.getLong("posterMemberUid")
          val _tmp_isBanned = _result.getBooleanNullable("isBanned")
          DiscussionPost().apply {
            this.postUid = _tmp_postUid
            this.postReplyToPostUid = _tmp_postReplyToPostUid
            this.postTitle = _tmp_postTitle
            this.postText = _tmp_postText
            this.postLastModified = _tmp_postLastModified
            this.posterMemberUid = _tmp_posterMemberUid
            this.isBanned = _tmp_isBanned
          }
        }
      }
    }

    override suspend fun countRows(): Int = _db.prepareAndUseStatementAsync(PreparedStatementConfig(
      sql = """
      |SELECT COUNT(*) FROM (
      |        SELECT DiscussionPost.*
      |          FROM DiscussionPost
      |         WHERE DiscussionPost.postReplyToPostUid != 0 
      |    ) AS _PagingCount
      """.trimMargin(),
      readOnly = true,)
    ) { _stmt -> 
      _stmt.executeQueryAsyncKmp().useResults{ _result -> 
        _result.mapNextRow(0) {
          _result.getInt(1)
        }
      }
    }
  }

  override fun findRootPostsAndNumRepliesAsPagingSource():
      PagingSource<Int, DiscussionPostAndNumReplies> = object :
      DoorLimitOffsetPagingSource<DiscussionPostAndNumReplies>(db = _db
  , tableNames = arrayOf("DiscussionPost")
  ) {
    override suspend fun loadRows(_limit: Int, _offset: Int): List<DiscussionPostAndNumReplies> =
        _db.prepareAndUseStatementAsync(PreparedStatementConfig(
      sql = """
      |SELECT * FROM (
      |        SELECT DiscussionPost.*,
      |               (SELECT COUNT(*) 
      |                  FROM DiscussionPost DiscussionPostInternal
      |                 WHERE DiscussionPostInternal.postReplyToPostUid = DiscussionPost.postUid) AS numReplies
      |         FROM DiscussionPost
      |        WHERE DiscussionPost.postReplyToPostUid = 0
      |    ) AS _PagingData LIMIT ? OFFSET ?
      """.trimMargin(),
      readOnly = true,)
    ) { _stmt -> 
      _stmt.setInt(1,_limit)
      _stmt.setInt(2,_offset)
      _stmt.executeQueryAsyncKmp().useResults{ _result -> 
        _result.mapRows {
          val _tmp_numReplies = _result.getInt("numReplies")
          var _tmp_DiscussionPost_nullCount = 0
          val _tmp_postUid = _result.getLong("postUid")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_postReplyToPostUid = _result.getLong("postReplyToPostUid")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_postTitle = _result.getString("postTitle")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_postText = _result.getString("postText")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_postLastModified = _result.getLong("postLastModified")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_posterMemberUid = _result.getLong("posterMemberUid")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_isBanned = _result.getBooleanNullable("isBanned")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_DiscussionPost_isAllNull = _tmp_DiscussionPost_nullCount == 7
          DiscussionPostAndNumReplies().apply {
            this.numReplies = _tmp_numReplies
            if(!_tmp_DiscussionPost_isAllNull) {
              this.discussionPost = DiscussionPost().apply {
                this.postUid = _tmp_postUid
                this.postReplyToPostUid = _tmp_postReplyToPostUid
                this.postTitle = _tmp_postTitle
                this.postText = _tmp_postText
                this.postLastModified = _tmp_postLastModified
                this.posterMemberUid = _tmp_posterMemberUid
                this.isBanned = _tmp_isBanned
              }
            }
          }
        }
      }
    }

    override suspend fun countRows(): Int = _db.prepareAndUseStatementAsync(PreparedStatementConfig(
      sql = """
      |SELECT COUNT(*) FROM (
      |        SELECT DiscussionPost.*,
      |               (SELECT COUNT(*) 
      |                  FROM DiscussionPost DiscussionPostInternal
      |                 WHERE DiscussionPostInternal.postReplyToPostUid = DiscussionPost.postUid) AS numReplies
      |         FROM DiscussionPost
      |        WHERE DiscussionPost.postReplyToPostUid = 0
      |    ) AS _PagingCount
      """.trimMargin(),
      readOnly = true,)
    ) { _stmt -> 
      _stmt.executeQueryAsyncKmp().useResults{ _result -> 
        _result.mapNextRow(0) {
          _result.getInt(1)
        }
      }
    }
  }

  override suspend fun findReplyPostsWithOffsetAndLimit(offset: Int, limit: Int):
      List<DiscussionPost> = _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT DiscussionPost.*
    |          FROM DiscussionPost
    |         WHERE DiscussionPost.postReplyToPostUid != 0 
    |         LIMIT ?
    |        OFFSET ?   
    |    
    """.trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setInt(1,limit)
    _stmt.setInt(2,offset)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapRows {
        val _tmp_postUid = _result.getLong("postUid")
        val _tmp_postReplyToPostUid = _result.getLong("postReplyToPostUid")
        val _tmp_postTitle = _result.getString("postTitle")
        val _tmp_postText = _result.getString("postText")
        val _tmp_postLastModified = _result.getLong("postLastModified")
        val _tmp_posterMemberUid = _result.getLong("posterMemberUid")
        val _tmp_isBanned = _result.getBooleanNullable("isBanned")
        DiscussionPost().apply {
          this.postUid = _tmp_postUid
          this.postReplyToPostUid = _tmp_postReplyToPostUid
          this.postTitle = _tmp_postTitle
          this.postText = _tmp_postText
          this.postLastModified = _tmp_postLastModified
          this.posterMemberUid = _tmp_posterMemberUid
          this.isBanned = _tmp_isBanned
        }
      }
    }
  }

  override fun findRootPostAndNumRepliesAsPagingSourceWithPagedParams():
      PagingSource<Int, DiscussionPostAndNumReplies> = object :
      DoorLimitOffsetPagingSource<DiscussionPostAndNumReplies>(db = _db
  , tableNames = arrayOf("DiscussionPost")
  ) {
    override suspend fun loadRows(_limit: Int, _offset: Int): List<DiscussionPostAndNumReplies> =
        _db.prepareAndUseStatementAsync(PreparedStatementConfig(
      sql = """
      |SELECT * FROM (
      |        SELECT DiscussionPost.*,
      |               (SELECT COUNT(*) 
      |                  FROM DiscussionPost DiscussionPostInternal
      |                 WHERE DiscussionPostInternal.postReplyToPostUid = DiscussionPost.postUid) AS numReplies
      |         FROM DiscussionPost
      |        WHERE DiscussionPost.postReplyToPostUid = 0
      |    ) AS _PagingData LIMIT ? OFFSET ?
      """.trimMargin(),
      readOnly = true,)
    ) { _stmt -> 
      _stmt.setInt(1,_limit)
      _stmt.setInt(2,_offset)
      _stmt.executeQueryAsyncKmp().useResults{ _result -> 
        _result.mapRows {
          val _tmp_numReplies = _result.getInt("numReplies")
          var _tmp_DiscussionPost_nullCount = 0
          val _tmp_postUid = _result.getLong("postUid")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_postReplyToPostUid = _result.getLong("postReplyToPostUid")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_postTitle = _result.getString("postTitle")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_postText = _result.getString("postText")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_postLastModified = _result.getLong("postLastModified")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_posterMemberUid = _result.getLong("posterMemberUid")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_isBanned = _result.getBooleanNullable("isBanned")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_DiscussionPost_isAllNull = _tmp_DiscussionPost_nullCount == 7
          DiscussionPostAndNumReplies().apply {
            this.numReplies = _tmp_numReplies
            if(!_tmp_DiscussionPost_isAllNull) {
              this.discussionPost = DiscussionPost().apply {
                this.postUid = _tmp_postUid
                this.postReplyToPostUid = _tmp_postReplyToPostUid
                this.postTitle = _tmp_postTitle
                this.postText = _tmp_postText
                this.postLastModified = _tmp_postLastModified
                this.posterMemberUid = _tmp_posterMemberUid
                this.isBanned = _tmp_isBanned
              }
            }
          }
        }
      }
    }

    override suspend fun countRows(): Int = _db.prepareAndUseStatementAsync(PreparedStatementConfig(
      sql = """
      |SELECT COUNT(*) FROM (
      |        SELECT DiscussionPost.*,
      |               (SELECT COUNT(*) 
      |                  FROM DiscussionPost DiscussionPostInternal
      |                 WHERE DiscussionPostInternal.postReplyToPostUid = DiscussionPost.postUid) AS numReplies
      |         FROM DiscussionPost
      |        WHERE DiscussionPost.postReplyToPostUid = 0
      |    ) AS _PagingCount
      """.trimMargin(),
      readOnly = true,)
    ) { _stmt -> 
      _stmt.executeQueryAsyncKmp().useResults{ _result -> 
        _result.mapNextRow(0) {
          _result.getInt(1)
        }
      }
    }
  }

  override fun findRootPostAndNumRepliesAsPagingSourceWithAsFlow():
      Flow<List<DiscussionPostAndNumReplies>> = _db.doorFlow(arrayOf("DiscussionPost")) {
    _db.prepareAndUseStatementAsync(PreparedStatementConfig(
      sql = """
      |
      |        SELECT DiscussionPost.*,
      |               (SELECT COUNT(*) 
      |                  FROM DiscussionPost DiscussionPostInternal
      |                 WHERE DiscussionPostInternal.postReplyToPostUid = DiscussionPost.postUid) AS numReplies
      |         FROM DiscussionPost
      |        WHERE DiscussionPost.postReplyToPostUid = 0
      |    
      """.trimMargin(),
      readOnly = true,)
    ) { _stmt -> 
      _stmt.executeQueryAsyncKmp().useResults{ _result -> 
        _result.mapRows {
          val _tmp_numReplies = _result.getInt("numReplies")
          var _tmp_DiscussionPost_nullCount = 0
          val _tmp_postUid = _result.getLong("postUid")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_postReplyToPostUid = _result.getLong("postReplyToPostUid")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_postTitle = _result.getString("postTitle")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_postText = _result.getString("postText")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_postLastModified = _result.getLong("postLastModified")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_posterMemberUid = _result.getLong("posterMemberUid")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_isBanned = _result.getBooleanNullable("isBanned")
          if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
          val _tmp_DiscussionPost_isAllNull = _tmp_DiscussionPost_nullCount == 7
          DiscussionPostAndNumReplies().apply {
            this.numReplies = _tmp_numReplies
            if(!_tmp_DiscussionPost_isAllNull) {
              this.discussionPost = DiscussionPost().apply {
                this.postUid = _tmp_postUid
                this.postReplyToPostUid = _tmp_postReplyToPostUid
                this.postTitle = _tmp_postTitle
                this.postText = _tmp_postText
                this.postLastModified = _tmp_postLastModified
                this.posterMemberUid = _tmp_posterMemberUid
                this.isBanned = _tmp_isBanned
              }
            }
          }
        }
      }
    }
  }

  override suspend fun checkNodeHasPermission(postUid: Long, nodeId: Long): Boolean =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT EXISTS(
    |               SELECT Member.memberUid
    |                 FROM Member
    |                WHERE CAST(? AS BIGINT) != 0
    |                  AND Member.memberUid = CAST(? AS BIGINT)
    |                  AND CAST(? AS BIGINT) != 0
    |        )
    |    
    """.trimMargin(),
    postgreSql = """
    |
    |        SELECT EXISTS(
    |               SELECT Member.memberUid
    |                 FROM Member
    |                WHERE ? != 0
    |                  AND Member.memberUid = ?
    |                  AND ? != 0
    |        )
    |    
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,postUid)
    _stmt.setLong(2,postUid)
    _stmt.setLong(3,nodeId)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow(false) {
        _result.getBoolean(1)
      }
    }
  }

  override suspend fun findRepliesWithAuthCheck(postUid: Long): List<DiscussionPost> =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT DiscussionPost.*
    |          FROM DiscussionPost
    |         WHERE DiscussionPost.postReplyToPostUid = CAST(? AS BIGINT) 
    |    
    """.trimMargin(),
    postgreSql = """
    |
    |        SELECT DiscussionPost.*
    |          FROM DiscussionPost
    |         WHERE DiscussionPost.postReplyToPostUid = ? 
    |    
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,postUid)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapRows {
        val _tmp_postUid = _result.getLong("postUid")
        val _tmp_postReplyToPostUid = _result.getLong("postReplyToPostUid")
        val _tmp_postTitle = _result.getString("postTitle")
        val _tmp_postText = _result.getString("postText")
        val _tmp_postLastModified = _result.getLong("postLastModified")
        val _tmp_posterMemberUid = _result.getLong("posterMemberUid")
        val _tmp_isBanned = _result.getBooleanNullable("isBanned")
        DiscussionPost().apply {
          this.postUid = _tmp_postUid
          this.postReplyToPostUid = _tmp_postReplyToPostUid
          this.postTitle = _tmp_postTitle
          this.postText = _tmp_postText
          this.postLastModified = _tmp_postLastModified
          this.posterMemberUid = _tmp_posterMemberUid
          this.isBanned = _tmp_isBanned
        }
      }
    }
  }

  override suspend fun getDiscussionPostAndAuthorName(postUid: Long): DiscussionPostAndAuthorName? =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT DiscussionPost.*,
    |               Member.firstName AS firstName,
    |               Member.lastName AS lastName
    |          FROM DiscussionPost
    |               LEFT JOIN Member
    |                         ON Member.memberUid = DiscussionPost.posterMemberUid 
    |         WHERE DiscussionPost.postUid = CAST(? AS BIGINT)       
    |    
    """.trimMargin(),
    postgreSql = """
    |
    |        SELECT DiscussionPost.*,
    |               Member.firstName AS firstName,
    |               Member.lastName AS lastName
    |          FROM DiscussionPost
    |               LEFT JOIN Member
    |                         ON Member.memberUid = DiscussionPost.posterMemberUid 
    |         WHERE DiscussionPost.postUid = ?       
    |    
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,postUid)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow(null) {
        val _tmp_firstName = _result.getString("firstName")
        val _tmp_lastName = _result.getString("lastName")
        var _tmp_DiscussionPost_nullCount = 0
        val _tmp_postUid = _result.getLong("postUid")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_postReplyToPostUid = _result.getLong("postReplyToPostUid")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_postTitle = _result.getString("postTitle")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_postText = _result.getString("postText")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_postLastModified = _result.getLong("postLastModified")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_posterMemberUid = _result.getLong("posterMemberUid")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_isBanned = _result.getBooleanNullable("isBanned")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_DiscussionPost_isAllNull = _tmp_DiscussionPost_nullCount == 7
        DiscussionPostAndAuthorName().apply {
          this.firstName = _tmp_firstName
          this.lastName = _tmp_lastName
          if(!_tmp_DiscussionPost_isAllNull) {
            this.discussionPost = DiscussionPost().apply {
              this.postUid = _tmp_postUid
              this.postReplyToPostUid = _tmp_postReplyToPostUid
              this.postTitle = _tmp_postTitle
              this.postText = _tmp_postText
              this.postLastModified = _tmp_postLastModified
              this.posterMemberUid = _tmp_posterMemberUid
              this.isBanned = _tmp_isBanned
            }
          }
        }
      }
    }
  }

  override suspend fun getDiscussionPostAndAuthorNameById(postId: Long):
      DiscussionPostAndAuthorName? = _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT DiscussionPost.*,
    |               Member.firstName AS firstName,
    |               Member.lastName AS lastName
    |          FROM DiscussionPost
    |               LEFT JOIN Member
    |                         ON Member.memberUid = DiscussionPost.posterMemberUid 
    |         WHERE DiscussionPost.postUid = CAST(? AS BIGINT)       
    |    
    """.trimMargin(),
    postgreSql = """
    |
    |        SELECT DiscussionPost.*,
    |               Member.firstName AS firstName,
    |               Member.lastName AS lastName
    |          FROM DiscussionPost
    |               LEFT JOIN Member
    |                         ON Member.memberUid = DiscussionPost.posterMemberUid 
    |         WHERE DiscussionPost.postUid = ?       
    |    
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,postId)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow(null) {
        val _tmp_firstName = _result.getString("firstName")
        val _tmp_lastName = _result.getString("lastName")
        var _tmp_DiscussionPost_nullCount = 0
        val _tmp_postUid = _result.getLong("postUid")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_postReplyToPostUid = _result.getLong("postReplyToPostUid")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_postTitle = _result.getString("postTitle")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_postText = _result.getString("postText")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_postLastModified = _result.getLong("postLastModified")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_posterMemberUid = _result.getLong("posterMemberUid")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_isBanned = _result.getBooleanNullable("isBanned")
        if(_result.wasNull()) _tmp_DiscussionPost_nullCount++
        val _tmp_DiscussionPost_isAllNull = _tmp_DiscussionPost_nullCount == 7
        DiscussionPostAndAuthorName().apply {
          this.firstName = _tmp_firstName
          this.lastName = _tmp_lastName
          if(!_tmp_DiscussionPost_isAllNull) {
            this.discussionPost = DiscussionPost().apply {
              this.postUid = _tmp_postUid
              this.postReplyToPostUid = _tmp_postReplyToPostUid
              this.postTitle = _tmp_postTitle
              this.postText = _tmp_postText
              this.postLastModified = _tmp_postLastModified
              this.posterMemberUid = _tmp_posterMemberUid
              this.isBanned = _tmp_isBanned
            }
          }
        }
      }
    }
  }

  override suspend fun findByName(firstName: String?): List<DiscussionPost> =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT DiscussionPost.*,
    |               Member.firstName AS firstName,
    |               Member.lastName AS lastName
    |          FROM DiscussionPost
    |               LEFT JOIN Member
    |                         ON Member.memberUid = DiscussionPost.posterMemberUid 
    |         WHERE Member.firstName = ?       
    |    
    """.trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setString(1,firstName)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapRows {
        val _tmp_postUid = _result.getLong("postUid")
        val _tmp_postReplyToPostUid = _result.getLong("postReplyToPostUid")
        val _tmp_postTitle = _result.getString("postTitle")
        val _tmp_postText = _result.getString("postText")
        val _tmp_postLastModified = _result.getLong("postLastModified")
        val _tmp_posterMemberUid = _result.getLong("posterMemberUid")
        val _tmp_isBanned = _result.getBooleanNullable("isBanned")
        DiscussionPost().apply {
          this.postUid = _tmp_postUid
          this.postReplyToPostUid = _tmp_postReplyToPostUid
          this.postTitle = _tmp_postTitle
          this.postText = _tmp_postText
          this.postLastModified = _tmp_postLastModified
          this.posterMemberUid = _tmp_posterMemberUid
          this.isBanned = _tmp_isBanned
        }
      }
    }
  }
}
