package db3

import com.ustadmobile.door.DoorDbType
import com.ustadmobile.door.EntityInsertionAdapter
import com.ustadmobile.door.PreparedStatementConfig
import com.ustadmobile.door.ext.prepareAndUseStatementAsync
import com.ustadmobile.door.flow.doorFlow
import com.ustadmobile.door.jdbc.PreparedStatement
import com.ustadmobile.door.jdbc.ext.executeQueryAsyncKmp
import com.ustadmobile.door.jdbc.ext.executeUpdateAsyncKmp
import com.ustadmobile.door.jdbc.ext.getIntNullable
import com.ustadmobile.door.jdbc.ext.mapNextRow
import com.ustadmobile.door.jdbc.ext.mapRows
import com.ustadmobile.door.jdbc.ext.setIntNullable
import com.ustadmobile.door.jdbc.ext.useResults
import com.ustadmobile.door.room.RoomDatabase
import kotlin.Boolean
import kotlin.Int
import kotlin.Long
import kotlin.String
import kotlin.collections.List
import kotlinx.coroutines.flow.Flow

public class ExampleEntity3Dao_JdbcImpl(
  public val _db: RoomDatabase,
) : ExampleEntity3Dao() {
  public val _insertAdapterExampleEntity3_abort: EntityInsertionAdapter<ExampleEntity3> = object :
      EntityInsertionAdapter<ExampleEntity3>(_db) {
    override fun makeSql(returnsId: Boolean): String =
        "INSERT INTO ExampleEntity3 (eeUid, cardNumber, name, lastUpdatedTime, nullableNumber) VALUES(?, ?, ?, ?, ?)"

    override fun bindPreparedStmtToEntity(stmt: PreparedStatement, entity: ExampleEntity3) {
      if(entity.eeUid == 0L) {
        stmt.setObject(1, null)
      } else {
        stmt.setLong(1, entity.eeUid)
      }
      stmt.setInt(2, entity.cardNumber)
      stmt.setString(3, entity.name)
      stmt.setLong(4, entity.lastUpdatedTime)
      stmt.setIntNullable(5, entity.nullableNumber)
    }
  }

  override suspend fun insertAsync(exampleEntity3: ExampleEntity3): Long {
    val _retVal = _insertAdapterExampleEntity3_abort.insertAndReturnIdAsync(exampleEntity3)
    return _retVal
  }

  override fun insert(exampleEntity3: ExampleEntity3): Long {
    val _retVal = _insertAdapterExampleEntity3_abort.insertAndReturnId(exampleEntity3)
    return _retVal
  }

  override suspend fun insertDifferently(
    cardNumber: Int,
    name: String,
    time: Long,
  ) {
    _db.prepareAndUseStatementAsync(PreparedStatementConfig(
      sql = """
      |
      |        INSERT OR IGNORE INTO ExampleEntity3(cardNumber, name, lastUpdatedTime)
      |        SELECT ? AS cardNumber,
      |               ? AS name,
      |               CAST(? AS BIGINT) AS time
      |    
      """.trimMargin(),
      postgreSql = """
      |
      |        INSERT INTO ExampleEntity3(cardNumber, name, lastUpdatedTime)
      |        SELECT ? AS cardNumber,
      |               ? AS name,
      |               ? AS time
      |        ON CONFLICT(eeUid) DO NOTHING       
      |    
      """.trimMargin(),
      readOnly = false,)
    ) { _stmt -> 
      _stmt.setInt(1,cardNumber)
      _stmt.setString(2,name)
      _stmt.setLong(3,time)
      _stmt.executeUpdateAsyncKmp()
    }
  }

  override suspend fun insertOutgoingReplication(entityUid: Long, destination: Long) {
    _db.prepareAndUseStatementAsync(PreparedStatementConfig(
      sql = """
      |
      |        INSERT INTO OutgoingReplication(destNodeId, orTableId, orPk1, orPk2, orPk3, orPk4)
      |               SELECT CAST(? AS BIGINT) AS destNodeId, 
      |                      542 AS orTableId,
      |                      CAST(? AS BIGINT) AS orPk1,
      |                      0 AS orPk2,
      |                      0 AS orPk3,
      |                      0 AS orPk4
      |    
      """.trimMargin(),
      postgreSql = """
      |
      |        INSERT INTO OutgoingReplication(destNodeId, orTableId, orPk1, orPk2, orPk3, orPk4)
      |               SELECT ? AS destNodeId, 
      |                      542 AS orTableId,
      |                      ? AS orPk1,
      |                      0 AS orPk2,
      |                      0 AS orPk3,
      |                      0 AS orPk4
      |    
      |""".trimMargin(),
      readOnly = false,)
    ) { _stmt -> 
      _stmt.setLong(1,destination)
      _stmt.setLong(2,entityUid)
      _stmt.executeUpdateAsyncKmp()
    }
  }

  override suspend fun findByUid(uid: Long): ExampleEntity3? =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT ExampleEntity3.*
    |          FROM ExampleEntity3
    |         WHERE ExampleEntity3.eeUid = CAST(? AS BIGINT) 
    |    
    """.trimMargin(),
    postgreSql = """
    |
    |        SELECT ExampleEntity3.*
    |          FROM ExampleEntity3
    |         WHERE ExampleEntity3.eeUid = ? 
    |    
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,uid)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow(null) {
        val _tmp_eeUid = _result.getLong("eeUid")
        val _tmp_cardNumber = _result.getInt("cardNumber")
        val _tmp_name = _result.getString("name")
        val _tmp_lastUpdatedTime = _result.getLong("lastUpdatedTime")
        val _tmp_nullableNumber = _result.getIntNullable("nullableNumber")
        ExampleEntity3().apply {
          this.eeUid = _tmp_eeUid
          this.cardNumber = _tmp_cardNumber
          this.name = _tmp_name
          this.lastUpdatedTime = _tmp_lastUpdatedTime
          this.nullableNumber = _tmp_nullableNumber
        }
      }
    }
  }

  override fun findByUidAsFlow(uid: Long): Flow<ExampleEntity3?> =
      _db.doorFlow(arrayOf("ExampleEntity3")) {
    _db.prepareAndUseStatementAsync(PreparedStatementConfig(
      sql = """
      |
      |        SELECT ExampleEntity3.*
      |          FROM ExampleEntity3
      |         WHERE ExampleEntity3.eeUid = CAST(? AS BIGINT) 
      |    
      """.trimMargin(),
      postgreSql = """
      |
      |        SELECT ExampleEntity3.*
      |          FROM ExampleEntity3
      |         WHERE ExampleEntity3.eeUid = ? 
      |    
      |""".trimMargin(),
      readOnly = true,)
    ) { _stmt -> 
      _stmt.setLong(1,uid)
      _stmt.executeQueryAsyncKmp().useResults{ _result -> 
        _result.mapNextRow(null) {
          val _tmp_eeUid = _result.getLong("eeUid")
          val _tmp_cardNumber = _result.getInt("cardNumber")
          val _tmp_name = _result.getString("name")
          val _tmp_lastUpdatedTime = _result.getLong("lastUpdatedTime")
          val _tmp_nullableNumber = _result.getIntNullable("nullableNumber")
          ExampleEntity3().apply {
            this.eeUid = _tmp_eeUid
            this.cardNumber = _tmp_cardNumber
            this.name = _tmp_name
            this.lastUpdatedTime = _tmp_lastUpdatedTime
            this.nullableNumber = _tmp_nullableNumber
          }
        }
      }
    }
  }

  override suspend fun findAllWithCardNumAbove(minCardNum: Int): List<ExampleEntity3> =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT ExampleEntity3.*
    |          FROM ExampleEntity3
    |         WHERE ExampleEntity3.cardNumber >= ? 
    |    
    """.trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setInt(1,minCardNum)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapRows {
        val _tmp_eeUid = _result.getLong("eeUid")
        val _tmp_cardNumber = _result.getInt("cardNumber")
        val _tmp_name = _result.getString("name")
        val _tmp_lastUpdatedTime = _result.getLong("lastUpdatedTime")
        val _tmp_nullableNumber = _result.getIntNullable("nullableNumber")
        ExampleEntity3().apply {
          this.eeUid = _tmp_eeUid
          this.cardNumber = _tmp_cardNumber
          this.name = _tmp_name
          this.lastUpdatedTime = _tmp_lastUpdatedTime
          this.nullableNumber = _tmp_nullableNumber
        }
      }
    }
  }
}
