package db3

import com.ustadmobile.door.DoorDbType
import com.ustadmobile.door.EntityInsertionAdapter
import com.ustadmobile.door.PreparedStatementConfig
import com.ustadmobile.door.ext.prepareAndUseStatementAsync
import com.ustadmobile.door.jdbc.PreparedStatement
import com.ustadmobile.door.jdbc.ext.executeQueryAsyncKmp
import com.ustadmobile.door.jdbc.ext.mapNextRow
import com.ustadmobile.door.jdbc.ext.useResults
import com.ustadmobile.door.room.RoomDatabase
import kotlin.Boolean
import kotlin.Long
import kotlin.String

public class MemberDao_JdbcImpl(
  public val _db: RoomDatabase,
) : MemberDao() {
  public val _insertAdapterMember_abort: EntityInsertionAdapter<Member> = object :
      EntityInsertionAdapter<Member>(_db) {
    override fun makeSql(returnsId: Boolean): String =
        "INSERT INTO Member (memberUid, firstName, lastName, memberLastModified) VALUES(?, ?, ?, ?)"

    override fun bindPreparedStmtToEntity(stmt: PreparedStatement, entity: Member) {
      if(entity.memberUid == 0L) {
        stmt.setObject(1, null)
      } else {
        stmt.setLong(1, entity.memberUid)
      }
      stmt.setString(2, entity.firstName)
      stmt.setString(3, entity.lastName)
      stmt.setLong(4, entity.memberLastModified)
    }
  }

  override suspend fun insertAsync(member: Member): Long {
    val _retVal = _insertAdapterMember_abort.insertAndReturnIdAsync(member)
    return _retVal
  }

  override suspend fun findByUid(memberUid: Long): Member? =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT Member.*
    |          FROM Member
    |         WHERE Member.memberUid = CAST(? AS BIGINT) 
    |    
    """.trimMargin(),
    postgreSql = """
    |
    |        SELECT Member.*
    |          FROM Member
    |         WHERE Member.memberUid = ? 
    |    
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,memberUid)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow(null) {
        val _tmp_memberUid = _result.getLong("memberUid")
        val _tmp_firstName = _result.getString("firstName")
        val _tmp_lastName = _result.getString("lastName")
        val _tmp_memberLastModified = _result.getLong("memberLastModified")
        Member().apply {
          this.memberUid = _tmp_memberUid
          this.firstName = _tmp_firstName
          this.lastName = _tmp_lastName
          this.memberLastModified = _tmp_memberLastModified
        }
      }
    }
  }

  override suspend fun checkMemberNodeHasPermission(postUid: Long, nodeId: Long): Boolean =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT EXISTS(
    |               SELECT Member.memberUid
    |                 FROM Member
    |                WHERE CAST(? AS BIGINT) != 0
    |                  AND Member.memberUid = CAST(? AS BIGINT)
    |                  AND CAST(? AS BIGINT) != 0
    |        )
    |    
    """.trimMargin(),
    postgreSql = """
    |
    |        SELECT EXISTS(
    |               SELECT Member.memberUid
    |                 FROM Member
    |                WHERE ? != 0
    |                  AND Member.memberUid = ?
    |                  AND ? != 0
    |        )
    |    
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,postUid)
    _stmt.setLong(2,postUid)
    _stmt.setLong(3,nodeId)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow(false) {
        _result.getBoolean(1)
      }
    }
  }

  override suspend fun findAuthorByPostUid(postUid: Long): Member? =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT Member.*
    |          FROM Member
    |         WHERE Member.memberUid = 
    |               (SELECT DiscussionPost.posterMemberUid
    |                  FROM DiscussionPost
    |                 WHERE DiscussionPost.postUid = CAST(? AS BIGINT))
    |    
    """.trimMargin(),
    postgreSql = """
    |
    |        SELECT Member.*
    |          FROM Member
    |         WHERE Member.memberUid = 
    |               (SELECT DiscussionPost.posterMemberUid
    |                  FROM DiscussionPost
    |                 WHERE DiscussionPost.postUid = ?)
    |    
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,postUid)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow(null) {
        val _tmp_memberUid = _result.getLong("memberUid")
        val _tmp_firstName = _result.getString("firstName")
        val _tmp_lastName = _result.getString("lastName")
        val _tmp_memberLastModified = _result.getLong("memberLastModified")
        Member().apply {
          this.memberUid = _tmp_memberUid
          this.firstName = _tmp_firstName
          this.lastName = _tmp_lastName
          this.memberLastModified = _tmp_memberLastModified
        }
      }
    }
  }

  override suspend fun getFirstNameByMemberId(memberUid: Long): String? =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT Member.firstName
    |          FROM Member
    |         WHERE Member.memberUid = CAST(? AS BIGINT) 
    |    
    """.trimMargin(),
    postgreSql = """
    |
    |        SELECT Member.firstName
    |          FROM Member
    |         WHERE Member.memberUid = ? 
    |    
    |""".trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,memberUid)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow(null) {
        _result.getString(1)
      }
    }
  }
}
