package db2

import com.ustadmobile.door.DoorDatabaseWrapper
import com.ustadmobile.door.log.DoorLogger
import com.ustadmobile.door.message.DoorMessageCallback
import com.ustadmobile.door.nodeevent.NodeEventManagerCommon
import com.ustadmobile.door.nodeevent.NodeEventManagerJvm
import com.ustadmobile.door.room.InvalidationTracker
import com.ustadmobile.door.room.RoomDatabase
import kotlin.Any
import kotlin.IllegalArgumentException
import kotlin.Int
import kotlin.Long
import kotlin.String
import kotlin.Suppress
import kotlin.collections.List
import kotlin.reflect.KClass
import kotlinx.coroutines.Runnable

@Suppress("REDUNDANT_PROJECTION", "ClassName")
public class ExampleDatabase2_DoorWrapper(
  private val _db: ExampleDatabase2,
  override val nodeId: Long,
  private val messageCallback: DoorMessageCallback<ExampleDatabase2>,
  logger: DoorLogger,
  dbName: String,
) : ExampleDatabase2(), DoorDatabaseWrapper<ExampleDatabase2> {
  override val dbVersion: Int
    get() = 2

  override val dbName: String
    get() = "DoorWrapper for [${_db.toString()}]"

  override val realDatabase: RoomDatabase
    get() = _db

  override val nodeEventManager: NodeEventManagerCommon<ExampleDatabase2> = NodeEventManagerJvm(_db,
      messageCallback, logger, dbName)


  override val invalidationTracker: InvalidationTracker
    get() = _db.invalidationTracker

  override fun createAllTables(): List<String> = _db.createAllTables()

  override fun exampleSyncableDao(): ExampleSyncableDao = _db.exampleSyncableDao()

  override fun exampleDao2(): ExampleDao2 = _db.exampleDao2()

  override fun exampleLinkedEntityDao(): ExampleLinkEntityDao = _db.exampleLinkedEntityDao()

  override fun examlpeDaoWithInterface(): ExampleDaoWithInterface = _db.examlpeDaoWithInterface()

  override fun exampleEntityPkIntDao(): ExampleEntityPkIntDao = _db.exampleEntityPkIntDao()

  override fun accessGrantDao(): AccessGrantDao = _db.accessGrantDao()

  override fun clearAllTables() {
    _db.clearAllTables()
  }

  override fun runInTransaction(runnable: Runnable) {
    _db.runInTransaction(runnable)
  }

  @Suppress("UNCHECKED_CAST")
  override fun <T : Any> getDaoByClass(daoClass: KClass<T>): T = when(daoClass) {
    ExampleSyncableDao::class -> exampleSyncableDao() as T
    ExampleDao2::class -> exampleDao2() as T
    ExampleLinkEntityDao::class -> exampleLinkedEntityDao() as T
    ExampleDaoWithInterface::class -> examlpeDaoWithInterface() as T
    ExampleEntityPkIntDao::class -> exampleEntityPkIntDao() as T
    AccessGrantDao::class -> accessGrantDao() as T
    else -> throw IllegalArgumentException("No such DAO on this DB")
  }

  override fun close() {
    _db.close()
  }
}
