package db3

import androidx.paging.PagingSource
import com.ustadmobile.door.DoorDatabaseRepository
import com.ustadmobile.door.ext.doorNodeIdHeader
import com.ustadmobile.door.ext.pagingSourceLoadParameters
import com.ustadmobile.door.ext.setBodyJson
import com.ustadmobile.door.ext.setRepoUrl
import com.ustadmobile.door.http.RepoDaoFlowHelper
import com.ustadmobile.door.http.RepositoryDaoWithFlowHelper
import com.ustadmobile.door.http.replicateHttpRequestCatchAndLog
import com.ustadmobile.door.http.replicateHttpRequestOrThrow
import com.ustadmobile.door.http.repoHttpRequest
import com.ustadmobile.door.http.repoHttpRequestWithFallback
import com.ustadmobile.door.paging.DoorRepositoryHttpRequestPagingSource
import com.ustadmobile.door.paging.DoorRepositoryReplicatePullPagingSource
import com.ustadmobile.door.paging.endOfPaginationReached
import com.ustadmobile.door.replication.onClientRepoDoorMessageHttpResponse
import com.ustadmobile.door.replication.withRepoChangeMonitorAsync
import com.ustadmobile.door.room.RoomDatabase
import io.ktor.client.HttpClient
import io.ktor.client.request.`get`
import io.ktor.client.request.`header`
import io.ktor.client.request.parameter
import io.ktor.client.request.post
import io.ktor.client.statement.bodyAsText
import io.ktor.http.ContentType
import io.ktor.http.contentType
import kotlin.Boolean
import kotlin.Int
import kotlin.Long
import kotlin.String
import kotlin.Suppress
import kotlin.collections.List
import kotlinx.coroutines.flow.Flow
import kotlinx.serialization.builtins.ListSerializer
import kotlinx.serialization.builtins.nullable
import kotlinx.serialization.builtins.serializer

@Suppress("REDUNDANT_PROJECTION", "LocalVariableName", "ClassName")
public class DiscussionPostDao_Repo(
  public val _db: RoomDatabase,
  public val _repo: DoorDatabaseRepository,
  public val _dao: DiscussionPostDao,
  public val _httpClient: HttpClient,
  public val _clientId: Long,
  public val _endpoint: String,
) : DiscussionPostDao(), RepositoryDaoWithFlowHelper {
  override val repoDaoFlowHelper: RepoDaoFlowHelper = RepoDaoFlowHelper(_repo)


  override suspend fun insertAsync(post: DiscussionPost): Long {
    val _result = _repo.withRepoChangeMonitorAsync("DiscussionPost") {
      _dao.insertAsync(post)
    }
    return _result
  }

  override suspend fun findByUid(postUid: Long): DiscussionPost? {
    val _result = _dao.findByUid(postUid)
    return _result
  }

  override suspend fun findByUidAsyncOverHttp(postUid: Long): DiscussionPost? =
      _repo.repoHttpRequestWithFallback(
    repoPath = "DiscussionPostDao/findByUidAsyncOverHttp",
    http =  {
      _repo.config.json.decodeFromString(
        deserializer = DiscussionPost.serializer().nullable,
        string = _repo.config.httpClient.`get` {
          setRepoUrl(_repo.config, "DiscussionPostDao/findByUidAsyncOverHttp")
          doorNodeIdHeader(_repo)
          `header`("cache-control", "no-store")
          parameter("postUid", _repo.config.json.encodeToString(Long.serializer(), postUid))
        }
        .bodyAsText()
      )
    } ,
    fallback =  {
      _dao.findByUidAsyncOverHttp(postUid)
    }

  )

  override suspend fun findAllRepliesWithPosterMember(postUid: Long):
      List<DiscussionPostAndPosterMember> {
    _repo.replicateHttpRequestCatchAndLog(repoPath =
        "DiscussionPostDao/findAllRepliesWithPosterMember") {
      val _response = _httpClient.`get` {
        setRepoUrl(_repo.config, "DiscussionPostDao/findAllRepliesWithPosterMember")
        doorNodeIdHeader(_repo)
        `header`("cache-control", "no-store")
        parameter("postUid", _repo.config.json.encodeToString(Long.serializer(), postUid))
      }

      _db.onClientRepoDoorMessageHttpResponse(_response, _repo.config.json)
    }
    val _result = _dao.findAllRepliesWithPosterMember(postUid)
    return _result
  }

  override fun findAllRepliesWithPosterMemberAsFlow(postUid: Long):
      Flow<List<DiscussionPostAndPosterMember>> = repoDaoFlowHelper.asRepoFlow(
    dbFlow = _dao.findAllRepliesWithPosterMemberAsFlow(postUid),
    onMakeHttpRequest =  {
      _repo.replicateHttpRequestCatchAndLog(repoPath =
          "DiscussionPostDao/findAllRepliesWithPosterMemberAsFlow") {
        val _response = _httpClient.`get` {
          setRepoUrl(_repo.config, "DiscussionPostDao/findAllRepliesWithPosterMemberAsFlow")
          doorNodeIdHeader(_repo)
          `header`("cache-control", "no-store")
          parameter("postUid", _repo.config.json.encodeToString(Long.serializer(), postUid))
        }

        _db.onClientRepoDoorMessageHttpResponse(_response, _repo.config.json)
      }
    },
  )

  override suspend fun findByUidWithPosterMember(postUid: Long): DiscussionPostAndPosterMember? {
    _repo.replicateHttpRequestCatchAndLog(repoPath =
        "DiscussionPostDao/findByUidWithPosterMember") {
      val _response = _httpClient.`get` {
        setRepoUrl(_repo.config, "DiscussionPostDao/findByUidWithPosterMember")
        doorNodeIdHeader(_repo)
        `header`("cache-control", "no-store")
        parameter("postUid", _repo.config.json.encodeToString(Long.serializer(), postUid))
      }

      _db.onClientRepoDoorMessageHttpResponse(_response, _repo.config.json)
    }
    val _result = _dao.findByUidWithPosterMember(postUid)
    return _result
  }

  override fun findByUidWithPosterMemberAsFlow(postUid: Long): Flow<DiscussionPostAndPosterMember?>
      = repoDaoFlowHelper.asRepoFlow(
    dbFlow = _dao.findByUidWithPosterMemberAsFlow(postUid),
    onMakeHttpRequest =  {
      _repo.replicateHttpRequestCatchAndLog(repoPath =
          "DiscussionPostDao/findByUidWithPosterMemberAsFlow") {
        val _response = _httpClient.`get` {
          setRepoUrl(_repo.config, "DiscussionPostDao/findByUidWithPosterMemberAsFlow")
          doorNodeIdHeader(_repo)
          `header`("cache-control", "no-store")
          parameter("postUid", _repo.config.json.encodeToString(Long.serializer(), postUid))
        }

        _db.onClientRepoDoorMessageHttpResponse(_response, _repo.config.json)
      }
    },
  )

  override suspend fun findByUidList(postUids: List<Long>): List<DiscussionPostAndPosterMember> {
    _repo.replicateHttpRequestCatchAndLog(repoPath = "DiscussionPostDao/findByUidList") {
      val _response = _httpClient.post {
        setRepoUrl(_repo.config, "DiscussionPostDao/findByUidList")
        doorNodeIdHeader(_repo)
        `header`("cache-control", "no-store")
        contentType(ContentType.Application.Json)
        setBodyJson(
          json = _repo.config.json,
          serializer = ListSerializer(Long.serializer()),
          value = postUids,
        )
      }

      _db.onClientRepoDoorMessageHttpResponse(_response, _repo.config.json)
    }
    val _result = _dao.findByUidList(postUids)
    return _result
  }

  override suspend fun getNumPostsSinceTime(since: Long): Int = _repo.repoHttpRequestWithFallback(
    repoPath = "DiscussionPostDao/getNumPostsSinceTime",
    http =  {
      _repo.config.json.decodeFromString(
        deserializer = Int.serializer(),
        string = _repo.config.httpClient.`get` {
          setRepoUrl(_repo.config, "DiscussionPostDao/getNumPostsSinceTime")
          doorNodeIdHeader(_repo)
          `header`("cache-control", "no-store")
          parameter("since", _repo.config.json.encodeToString(Long.serializer(), since))
        }
        .bodyAsText()
      )
    } ,
    fallback =  {
      _dao.getNumPostsSinceTime(since)
    }

  )

  override suspend fun getNumPostsSinceTimeHttpOnly(since: Long): Int =
      _repo.config.json.decodeFromString(
    deserializer = Int.serializer(),
    string =_repo.repoHttpRequest(repoPath = "DiscussionPostDao/getNumPostsSinceTimeHttpOnly") {
      _repo.config.httpClient.`get` {
        setRepoUrl(_repo.config, "DiscussionPostDao/getNumPostsSinceTimeHttpOnly")
        doorNodeIdHeader(_repo)
        `header`("cache-control", "no-store")
        parameter("since", _repo.config.json.encodeToString(Long.serializer(), since))
      }
      .bodyAsText()
    }
  )

  override fun findAllPostAsPagingSource(since: Long): PagingSource<Int, DiscussionPost> =
      DoorRepositoryReplicatePullPagingSource(
    repo = _repo,
    repoPath = "DiscussionPostDao/findAllPostAsPagingSource",
    dbPagingSource = _dao.findAllPostAsPagingSource(since),
    onLoadHttp =  {
      _pagingParams -> 
      _repo.replicateHttpRequestOrThrow(repoPath = "DiscussionPostDao/findAllPostAsPagingSource") {
        val _response = _httpClient.`get` {
          setRepoUrl(_repo.config, "DiscussionPostDao/findAllPostAsPagingSource")
          doorNodeIdHeader(_repo)
          `header`("cache-control", "no-store")
          parameter("since", _repo.config.json.encodeToString(Long.serializer(), since))
          pagingSourceLoadParameters(
            json = _repo.config.json, 
            keySerializer = Int.serializer().nullable,
            loadParams = _pagingParams
          )
        }

        _db.onClientRepoDoorMessageHttpResponse(_response, _repo.config.json)
        _response.endOfPaginationReached()
      }
    },
  )

  override fun findAllPostAsNetworkOnlyPagingSource(since: Long): PagingSource<Int, DiscussionPost>
      = DoorRepositoryHttpRequestPagingSource(
    valueDeserializationStrategy = ListSerializer(DiscussionPost.serializer()),
    json = _repo.config.json,
    onLoadHttp =  {
      _pagingLoadParams ->
      _repo.config.httpClient.`get` {
        setRepoUrl(_repo.config, "DiscussionPostDao/findAllPostAsNetworkOnlyPagingSource")
        doorNodeIdHeader(_repo)
        `header`("cache-control", "no-store")
        parameter("since", _repo.config.json.encodeToString(Long.serializer(), since))
        pagingSourceLoadParameters(
          json = _repo.config.json, 
          keySerializer = Int.serializer().nullable,
          loadParams = _pagingLoadParams
        )
      }
    }
    ,)

  override fun findAllPostAsNetworkWithFallbackPagingSource(since: Long):
      PagingSource<Int, DiscussionPost> = DoorRepositoryHttpRequestPagingSource(
    valueDeserializationStrategy = ListSerializer(DiscussionPost.serializer()),
    json = _repo.config.json,
    onLoadHttp =  {
      _pagingLoadParams ->
      _repo.config.httpClient.`get` {
        setRepoUrl(_repo.config, "DiscussionPostDao/findAllPostAsNetworkWithFallbackPagingSource")
        doorNodeIdHeader(_repo)
        `header`("cache-control", "no-store")
        parameter("since", _repo.config.json.encodeToString(Long.serializer(), since))
        pagingSourceLoadParameters(
          json = _repo.config.json, 
          keySerializer = Int.serializer().nullable,
          loadParams = _pagingLoadParams
        )
      }
    }
    ,fallbackPagingSource = _dao.findAllPostAsNetworkWithFallbackPagingSource(since)
  )

  override suspend fun findAllRepliesForPost(postUid: Long, nodeId: Long): List<DiscussionPost> {
    val _result = _dao.findAllRepliesForPost(postUid, nodeId)
    return _result
  }

  override suspend fun findPostAndNumReplies(postUid: Long, sinceTime: Long):
      DiscussionPostAndNumReplies? {
    _repo.replicateHttpRequestCatchAndLog(repoPath = "DiscussionPostDao/findPostAndNumReplies") {
      val _response = _httpClient.`get` {
        setRepoUrl(_repo.config, "DiscussionPostDao/findPostAndNumReplies")
        doorNodeIdHeader(_repo)
        `header`("cache-control", "no-store")
        parameter("postUid", _repo.config.json.encodeToString(Long.serializer(), postUid))
        parameter("sinceTime", _repo.config.json.encodeToString(Long.serializer(), sinceTime))
      }

      _db.onClientRepoDoorMessageHttpResponse(_response, _repo.config.json)
    }
    val _result = _dao.findPostAndNumReplies(postUid, sinceTime)
    return _result
  }

  override fun findRootRepliesAsPagingSource(): PagingSource<Int, DiscussionPost> {
    val _result = _dao.findRootRepliesAsPagingSource()
    return _result
  }

  override fun findRootPostsAndNumRepliesAsPagingSource():
      PagingSource<Int, DiscussionPostAndNumReplies> = DoorRepositoryReplicatePullPagingSource(
    repo = _repo,
    repoPath = "DiscussionPostDao/findRootPostsAndNumRepliesAsPagingSource",
    dbPagingSource = _dao.findRootPostsAndNumRepliesAsPagingSource(),
    onLoadHttp =  {
      _pagingParams -> 
      _repo.replicateHttpRequestOrThrow(repoPath =
          "DiscussionPostDao/findRootPostsAndNumRepliesAsPagingSource") {
        val _response = _httpClient.`get` {
          setRepoUrl(_repo.config, "DiscussionPostDao/findRootPostsAndNumRepliesAsPagingSource")
          doorNodeIdHeader(_repo)
          `header`("cache-control", "no-store")
          pagingSourceLoadParameters(
            json = _repo.config.json, 
            keySerializer = Int.serializer().nullable,
            loadParams = _pagingParams
          )
        }

        _db.onClientRepoDoorMessageHttpResponse(_response, _repo.config.json)
        _response.endOfPaginationReached()
      }
    },
  )

  override suspend fun findReplyPostsWithOffsetAndLimit(offset: Int, limit: Int):
      List<DiscussionPost> {
    val _result = _dao.findReplyPostsWithOffsetAndLimit(offset, limit)
    return _result
  }

  override fun findRootPostAndNumRepliesAsPagingSourceWithPagedParams():
      PagingSource<Int, DiscussionPostAndNumReplies> = DoorRepositoryReplicatePullPagingSource(
    repo = _repo,
    repoPath = "DiscussionPostDao/findRootPostAndNumRepliesAsPagingSourceWithPagedParams",
    dbPagingSource = _dao.findRootPostAndNumRepliesAsPagingSourceWithPagedParams(),
    onLoadHttp =  {
      _pagingParams -> 
      _repo.replicateHttpRequestOrThrow(repoPath =
          "DiscussionPostDao/findRootPostAndNumRepliesAsPagingSourceWithPagedParams") {
        val _response = _httpClient.`get` {
          setRepoUrl(_repo.config,
              "DiscussionPostDao/findRootPostAndNumRepliesAsPagingSourceWithPagedParams")
          doorNodeIdHeader(_repo)
          `header`("cache-control", "no-store")
          pagingSourceLoadParameters(
            json = _repo.config.json, 
            keySerializer = Int.serializer().nullable,
            loadParams = _pagingParams
          )
        }

        _db.onClientRepoDoorMessageHttpResponse(_response, _repo.config.json)
        _response.endOfPaginationReached()
      }
    },
  )

  override fun findRootPostAndNumRepliesAsPagingSourceWithAsFlow():
      Flow<List<DiscussionPostAndNumReplies>> {
    val _result = _dao.findRootPostAndNumRepliesAsPagingSourceWithAsFlow()
    return _result
  }

  override suspend fun checkNodeHasPermission(postUid: Long, nodeId: Long): Boolean {
    val _result = _dao.checkNodeHasPermission(postUid, nodeId)
    return _result
  }

  override suspend fun findRepliesWithAuthCheck(postUid: Long): List<DiscussionPost> {
    _repo.replicateHttpRequestCatchAndLog(repoPath = "DiscussionPostDao/findRepliesWithAuthCheck") {
      val _response = _httpClient.`get` {
        setRepoUrl(_repo.config, "DiscussionPostDao/findRepliesWithAuthCheck")
        doorNodeIdHeader(_repo)
        `header`("cache-control", "no-store")
        parameter("postUid", _repo.config.json.encodeToString(Long.serializer(), postUid))
      }

      _db.onClientRepoDoorMessageHttpResponse(_response, _repo.config.json)
    }
    val _result = _dao.findRepliesWithAuthCheck(postUid)
    return _result
  }

  override suspend fun getDiscussionPostAndAuthorName(postUid: Long): DiscussionPostAndAuthorName? {
    _repo.replicateHttpRequestCatchAndLog(repoPath =
        "DiscussionPostDao/getDiscussionPostAndAuthorName") {
      val _response = _httpClient.`get` {
        setRepoUrl(_repo.config, "DiscussionPostDao/getDiscussionPostAndAuthorName")
        doorNodeIdHeader(_repo)
        `header`("cache-control", "no-store")
        parameter("postUid", _repo.config.json.encodeToString(Long.serializer(), postUid))
      }

      _db.onClientRepoDoorMessageHttpResponse(_response, _repo.config.json)
    }
    val _result = _dao.getDiscussionPostAndAuthorName(postUid)
    return _result
  }

  override suspend fun getDiscussionPostAndAuthorNameById(postId: Long):
      DiscussionPostAndAuthorName? {
    _repo.replicateHttpRequestCatchAndLog(repoPath =
        "DiscussionPostDao/getDiscussionPostAndAuthorNameById") {
      val _response = _httpClient.`get` {
        setRepoUrl(_repo.config, "DiscussionPostDao/getDiscussionPostAndAuthorNameById")
        doorNodeIdHeader(_repo)
        `header`("cache-control", "no-store")
        parameter("postId", _repo.config.json.encodeToString(Long.serializer(), postId))
      }

      _db.onClientRepoDoorMessageHttpResponse(_response, _repo.config.json)
    }
    val _result = _dao.getDiscussionPostAndAuthorNameById(postId)
    return _result
  }

  override suspend fun update(post: DiscussionPost) {
    _repo.withRepoChangeMonitorAsync("DiscussionPost") {
      _dao.update(post)
    }
  }

  override suspend fun findByName(firstName: String?): List<DiscussionPost> {
    _repo.replicateHttpRequestCatchAndLog(repoPath = "DiscussionPostDao/findByName") {
      val _response = _httpClient.`get` {
        setRepoUrl(_repo.config, "DiscussionPostDao/findByName")
        doorNodeIdHeader(_repo)
        `header`("cache-control", "no-store")
        parameter("firstName", _repo.config.json.encodeToString(String.serializer().nullable,
            firstName))
      }

      _db.onClientRepoDoorMessageHttpResponse(_response, _repo.config.json)
    }
    val _result = _dao.findByName(firstName)
    return _result
  }
}
