package db3

import com.ustadmobile.door.DoorDatabaseJdbc
import com.ustadmobile.door.DoorPrimaryKeyManager
import com.ustadmobile.door.ext.doorDatabaseMetadata
import com.ustadmobile.door.ext.execSqlBatch
import com.ustadmobile.door.ext.mutableLinkedListOf
import com.ustadmobile.door.ext.nodeIdAuthCache
import com.ustadmobile.door.ext.rootDatabase
import com.ustadmobile.door.jdbc.DataSource
import com.ustadmobile.door.log.DoorLogger
import com.ustadmobile.door.room.InvalidationTracker
import com.ustadmobile.door.room.RoomDatabase
import com.ustadmobile.door.room.RoomDatabaseJdbcImplHelper
import com.ustadmobile.door.room.RoomJdbcImpl
import com.ustadmobile.door.util.NodeIdAuthCache
import com.ustadmobile.door.util.systemTimeInMillis
import kotlin.Int
import kotlin.String
import kotlin.collections.List
import kotlin.collections.toTypedArray

public class ExampleDb3_JdbcImpl(
  override val doorJdbcSourceDatabase: RoomDatabase?,
  override val dataSource: DataSource,
  dbUrl: String,
  override val dbName: String,
  override val jdbcQueryTimeout: Int,
  jdbcDbType: Int,
  logger: DoorLogger,
) : ExampleDb3(), DoorDatabaseJdbc, RoomJdbcImpl {
  override val dbVersion: Int
    get() = 1

  override val jdbcImplHelper: RoomDatabaseJdbcImplHelper = RoomDatabaseJdbcImplHelper(
        dataSource = dataSource,
        db = this,
        dbUrl = dbUrl,
        dbName = dbName,
        logger = logger,
        tableNames = this::class.doorDatabaseMetadata().allTables,
        invalidationTracker =
            InvalidationTracker(*this::class.doorDatabaseMetadata().allTables.toTypedArray()),
        dbType = jdbcDbType,
      )


  override val realNodeIdAuthCache: NodeIdAuthCache by lazy {
        if(this == rootDatabase) {
          val nodeIdAuthCache = NodeIdAuthCache(this)
          nodeIdAuthCache
        } else {
          rootDatabase.nodeIdAuthCache
        }
      }


  override val realPrimaryKeyManager: DoorPrimaryKeyManager by lazy {
        DoorPrimaryKeyManager(ExampleDb3::class.doorDatabaseMetadata().replicateEntities.keys)
      }


  override val invalidationTracker: InvalidationTracker
    get() = jdbcImplHelper.invalidationTracker

  public val _ExampleEntity3Dao: ExampleEntity3Dao_JdbcImpl by
      lazy { ExampleEntity3Dao_JdbcImpl(this) }

  override val exampleEntity3Dao: ExampleEntity3Dao
    get() = _ExampleEntity3Dao

  public val _DiscussionPostDao: DiscussionPostDao_JdbcImpl by
      lazy { DiscussionPostDao_JdbcImpl(this) }

  override val discussionPostDao: DiscussionPostDao
    get() = _DiscussionPostDao

  public val _MemberDao: MemberDao_JdbcImpl by lazy { MemberDao_JdbcImpl(this) }

  override val memberDao: MemberDao
    get() = _MemberDao

  public val _BadgeDao: BadgeDao_JdbcImpl by lazy { BadgeDao_JdbcImpl(this) }

  override val badgeDao: BadgeDao
    get() = _BadgeDao

  public val _StatementEntityDao: StatementEntityDao_JdbcImpl by
      lazy { StatementEntityDao_JdbcImpl(this) }

  override val statementEntityDao: StatementEntityDao
    get() = _StatementEntityDao

  override fun createAllTables(): List<String> {
    val _stmtList = mutableLinkedListOf<String>()
    when(jdbcImplHelper.dbType) {
      1 ->  {
        // - create for this SQLite 
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS _doorwayinfo (dbVersion int primary key, dbHash varchar(255))"
         _stmtList += "INSERT INTO _doorwayinfo VALUES (1, '')"
        //Begin: Create table ExampleEntity3 for SQLite
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE ExampleEntity3 RENAME to ExampleEntity3_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS ExampleEntity3 (  cardNumber  INTEGER  NOT NULL , name  TEXT , lastUpdatedTime  INTEGER  NOT NULL , nullableNumber  INTEGER , eeUid  INTEGER  PRIMARY KEY  AUTOINCREMENT  NOT NULL )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO ExampleEntity3 (cardNumber, name, lastUpdatedTime, nullableNumber, eeUid) SELECT cardNumber, name, lastUpdatedTime, nullableNumber, eeUid FROM ExampleEntity3_OLD")
        _stmt.executeUpdate("DROP TABLE ExampleEntity3_OLD")
        END MIGRATION*/
        //End: Create table ExampleEntity3 for SQLite

        //Begin: Create table DiscussionPost for SQLite
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE DiscussionPost RENAME to DiscussionPost_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS DiscussionPost (  postReplyToPostUid  INTEGER  NOT NULL , postTitle  TEXT , postText  TEXT , postLastModified  INTEGER  NOT NULL , posterMemberUid  INTEGER  NOT NULL , isBanned  INTEGER , postUid  INTEGER  PRIMARY KEY  AUTOINCREMENT  NOT NULL )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO DiscussionPost (postReplyToPostUid, postTitle, postText, postLastModified, posterMemberUid, isBanned, postUid) SELECT postReplyToPostUid, postTitle, postText, postLastModified, posterMemberUid, isBanned, postUid FROM DiscussionPost_OLD")
        _stmt.executeUpdate("DROP TABLE DiscussionPost_OLD")
        END MIGRATION*/
        //End: Create table DiscussionPost for SQLite

        //Begin: Create table Member for SQLite
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE Member RENAME to Member_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS Member (  firstName  TEXT , lastName  TEXT , memberLastModified  INTEGER  NOT NULL , memberUid  INTEGER  PRIMARY KEY  AUTOINCREMENT  NOT NULL )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO Member (firstName, lastName, memberLastModified, memberUid) SELECT firstName, lastName, memberLastModified, memberUid FROM Member_OLD")
        _stmt.executeUpdate("DROP TABLE Member_OLD")
        END MIGRATION*/
        //End: Create table Member for SQLite

        //Begin: Create table OutgoingReplication for SQLite
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE OutgoingReplication RENAME to OutgoingReplication_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS OutgoingReplication (  destNodeId  INTEGER  NOT NULL , orPk1  INTEGER  NOT NULL , orPk2  INTEGER  NOT NULL  DEFAULT 0 , orPk3  INTEGER  NOT NULL  DEFAULT 0 , orPk4  INTEGER  NOT NULL  DEFAULT 0 , orTableId  INTEGER  NOT NULL , orUid  INTEGER  PRIMARY KEY  AUTOINCREMENT  NOT NULL )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO OutgoingReplication (destNodeId, orPk1, orPk2, orPk3, orPk4, orTableId, orUid) SELECT destNodeId, orPk1, orPk2, orPk3, orPk4, orTableId, orUid FROM OutgoingReplication_OLD")
        _stmt.executeUpdate("DROP TABLE OutgoingReplication_OLD")
        END MIGRATION*/
        //End: Create table OutgoingReplication for SQLite

        //Begin: Create table ReplicationOperation for SQLite
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE ReplicationOperation RENAME to ReplicationOperation_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS ReplicationOperation (  repOpRemoteNodeId  INTEGER  NOT NULL , repOpStatus  INTEGER  NOT NULL , repOpTableId  INTEGER  NOT NULL , PRIMARY KEY (repOpRemoteNodeId, repOpTableId) )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO ReplicationOperation (repOpRemoteNodeId, repOpStatus, repOpTableId) SELECT repOpRemoteNodeId, repOpStatus, repOpTableId FROM ReplicationOperation_OLD")
        _stmt.executeUpdate("DROP TABLE ReplicationOperation_OLD")
        END MIGRATION*/
        //End: Create table ReplicationOperation for SQLite

        //Begin: Create table PendingRepositorySession for SQLite
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE PendingRepositorySession RENAME to PendingRepositorySession_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS PendingRepositorySession (  endpointUrl  TEXT , remoteNodeId  INTEGER  NOT NULL , rsUid  INTEGER  PRIMARY KEY  AUTOINCREMENT  NOT NULL )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO PendingRepositorySession (endpointUrl, remoteNodeId, rsUid) SELECT endpointUrl, remoteNodeId, rsUid FROM PendingRepositorySession_OLD")
        _stmt.executeUpdate("DROP TABLE PendingRepositorySession_OLD")
        END MIGRATION*/
        //End: Create table PendingRepositorySession for SQLite

        //Begin: Create table Badge for SQLite
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE Badge RENAME to Badge_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS Badge (  badgeName  TEXT , badgePoints  INTEGER  NOT NULL , badgeLastChangeTime  INTEGER  NOT NULL , badgeUid  INTEGER  PRIMARY KEY  AUTOINCREMENT  NOT NULL )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO Badge (badgeName, badgePoints, badgeLastChangeTime, badgeUid) SELECT badgeName, badgePoints, badgeLastChangeTime, badgeUid FROM Badge_OLD")
        _stmt.executeUpdate("DROP TABLE Badge_OLD")
        END MIGRATION*/
        //End: Create table Badge for SQLite

        //Begin: Create table StatementEntity for SQLite
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE StatementEntity RENAME to StatementEntity_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS StatementEntity (  uidHi  INTEGER  NOT NULL , uidLo  INTEGER  NOT NULL , lct  INTEGER  NOT NULL , name  TEXT , PRIMARY KEY (uidHi, uidLo) )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO StatementEntity (uidHi, uidLo, lct, name) SELECT uidHi, uidLo, lct, name FROM StatementEntity_OLD")
        _stmt.executeUpdate("DROP TABLE StatementEntity_OLD")
        END MIGRATION*/
        //End: Create table StatementEntity for SQLite

      }
      2 ->  {
        // - create for this PostgreSQL 
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS _doorwayinfo (dbVersion int primary key, dbHash varchar(255))"
         _stmtList += "INSERT INTO _doorwayinfo VALUES (1, '')"
        //Begin: Create table ExampleEntity3 for PostgreSQL
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE ExampleEntity3 RENAME to ExampleEntity3_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS ExampleEntity3 (  cardNumber  INTEGER  NOT NULL , name  TEXT , lastUpdatedTime  BIGINT  NOT NULL , nullableNumber  INTEGER , eeUid  BIGSERIAL  PRIMARY KEY  NOT NULL )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO ExampleEntity3 (cardNumber, name, lastUpdatedTime, nullableNumber, eeUid) SELECT cardNumber, name, lastUpdatedTime, nullableNumber, eeUid FROM ExampleEntity3_OLD")
        _stmt.executeUpdate("DROP TABLE ExampleEntity3_OLD")
        END MIGRATION*/
        //End: Create table ExampleEntity3 for PostgreSQL

        //Begin: Create table DiscussionPost for PostgreSQL
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE DiscussionPost RENAME to DiscussionPost_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS DiscussionPost (  postReplyToPostUid  BIGINT  NOT NULL , postTitle  TEXT , postText  TEXT , postLastModified  BIGINT  NOT NULL , posterMemberUid  BIGINT  NOT NULL , isBanned  BOOL , postUid  BIGSERIAL  PRIMARY KEY  NOT NULL )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO DiscussionPost (postReplyToPostUid, postTitle, postText, postLastModified, posterMemberUid, isBanned, postUid) SELECT postReplyToPostUid, postTitle, postText, postLastModified, posterMemberUid, isBanned, postUid FROM DiscussionPost_OLD")
        _stmt.executeUpdate("DROP TABLE DiscussionPost_OLD")
        END MIGRATION*/
        //End: Create table DiscussionPost for PostgreSQL

        //Begin: Create table Member for PostgreSQL
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE Member RENAME to Member_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS Member (  firstName  TEXT , lastName  TEXT , memberLastModified  BIGINT  NOT NULL , memberUid  BIGSERIAL  PRIMARY KEY  NOT NULL )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO Member (firstName, lastName, memberLastModified, memberUid) SELECT firstName, lastName, memberLastModified, memberUid FROM Member_OLD")
        _stmt.executeUpdate("DROP TABLE Member_OLD")
        END MIGRATION*/
        //End: Create table Member for PostgreSQL

        //Begin: Create table OutgoingReplication for PostgreSQL
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE OutgoingReplication RENAME to OutgoingReplication_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS OutgoingReplication (  destNodeId  BIGINT  NOT NULL , orPk1  BIGINT  NOT NULL , orPk2  BIGINT  NOT NULL  DEFAULT 0 , orPk3  BIGINT  NOT NULL  DEFAULT 0 , orPk4  BIGINT  NOT NULL  DEFAULT 0 , orTableId  INTEGER  NOT NULL , orUid  BIGSERIAL  PRIMARY KEY  NOT NULL )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO OutgoingReplication (destNodeId, orPk1, orPk2, orPk3, orPk4, orTableId, orUid) SELECT destNodeId, orPk1, orPk2, orPk3, orPk4, orTableId, orUid FROM OutgoingReplication_OLD")
        _stmt.executeUpdate("DROP TABLE OutgoingReplication_OLD")
        END MIGRATION*/
        //End: Create table OutgoingReplication for PostgreSQL

        //Begin: Create table ReplicationOperation for PostgreSQL
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE ReplicationOperation RENAME to ReplicationOperation_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS ReplicationOperation (  repOpRemoteNodeId  BIGINT  NOT NULL , repOpStatus  INTEGER  NOT NULL , repOpTableId  INTEGER  NOT NULL , PRIMARY KEY (repOpRemoteNodeId, repOpTableId) )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO ReplicationOperation (repOpRemoteNodeId, repOpStatus, repOpTableId) SELECT repOpRemoteNodeId, repOpStatus, repOpTableId FROM ReplicationOperation_OLD")
        _stmt.executeUpdate("DROP TABLE ReplicationOperation_OLD")
        END MIGRATION*/
        //End: Create table ReplicationOperation for PostgreSQL

        //Begin: Create table PendingRepositorySession for PostgreSQL
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE PendingRepositorySession RENAME to PendingRepositorySession_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS PendingRepositorySession (  endpointUrl  TEXT , remoteNodeId  BIGINT  NOT NULL , rsUid  BIGSERIAL  PRIMARY KEY  NOT NULL )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO PendingRepositorySession (endpointUrl, remoteNodeId, rsUid) SELECT endpointUrl, remoteNodeId, rsUid FROM PendingRepositorySession_OLD")
        _stmt.executeUpdate("DROP TABLE PendingRepositorySession_OLD")
        END MIGRATION*/
        //End: Create table PendingRepositorySession for PostgreSQL

        //Begin: Create table Badge for PostgreSQL
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE Badge RENAME to Badge_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS Badge (  badgeName  TEXT , badgePoints  INTEGER  NOT NULL , badgeLastChangeTime  BIGINT  NOT NULL , badgeUid  BIGSERIAL  PRIMARY KEY  NOT NULL )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO Badge (badgeName, badgePoints, badgeLastChangeTime, badgeUid) SELECT badgeName, badgePoints, badgeLastChangeTime, badgeUid FROM Badge_OLD")
        _stmt.executeUpdate("DROP TABLE Badge_OLD")
        END MIGRATION*/
        //End: Create table Badge for PostgreSQL

        //Begin: Create table StatementEntity for PostgreSQL
        /* START MIGRATION: 
        _stmt.executeUpdate("ALTER TABLE StatementEntity RENAME to StatementEntity_OLD")
        END MIGRATION */
        _stmtList +=
            "CREATE TABLE IF NOT EXISTS StatementEntity (  uidHi  BIGINT  NOT NULL , uidLo  BIGINT  NOT NULL , lct  BIGINT  NOT NULL , name  TEXT , PRIMARY KEY (uidHi, uidLo) )"
        /* START MIGRATION: 
        _stmt.executeUpdate("INSERT INTO StatementEntity (uidHi, uidLo, lct, name) SELECT uidHi, uidLo, lct, name FROM StatementEntity_OLD")
        _stmt.executeUpdate("DROP TABLE StatementEntity_OLD")
        END MIGRATION*/
        //End: Create table StatementEntity for PostgreSQL

      }
    }
    return _stmtList
  }

  public fun makeClearAllTablesSql(): List<String> {
    val _stmtList = mutableListOf<String>()
    _stmtList += "DELETE FROM ExampleEntity3"
    _stmtList += "DELETE FROM DiscussionPost"
    _stmtList += "DELETE FROM Member"
    _stmtList += "DELETE FROM OutgoingReplication"
    _stmtList += "DELETE FROM ReplicationOperation"
    _stmtList += "DELETE FROM PendingRepositorySession"
    _stmtList += "DELETE FROM Badge"
    _stmtList += "DELETE FROM StatementEntity"
    return _stmtList
  }

  override fun clearAllTables() {
    execSqlBatch(*makeClearAllTablesSql().toTypedArray())
  }

  override fun close() {
    jdbcImplHelper.close()
  }
}
