package db3

import com.ustadmobile.door.DoorDbType
import com.ustadmobile.door.EntityInsertionAdapter
import com.ustadmobile.door.PreparedStatementConfig
import com.ustadmobile.door.ext.prepareAndUseStatementAsync
import com.ustadmobile.door.flow.doorFlow
import com.ustadmobile.door.jdbc.PreparedStatement
import com.ustadmobile.door.jdbc.ext.executeQueryAsyncKmp
import com.ustadmobile.door.jdbc.ext.mapNextRow
import com.ustadmobile.door.jdbc.ext.mapRows
import com.ustadmobile.door.jdbc.ext.useResults
import com.ustadmobile.door.room.RoomDatabase
import kotlin.Boolean
import kotlin.IllegalArgumentException
import kotlin.Long
import kotlin.String
import kotlin.collections.List
import kotlinx.coroutines.flow.Flow

public class StatementEntityDao_JdbcImpl(
  public val _db: RoomDatabase,
) : StatementEntityDao() {
  public val _insertAdapterStatementEntity_abort: EntityInsertionAdapter<StatementEntity> = object :
      EntityInsertionAdapter<StatementEntity>(_db) {
    override fun makeSql(returnsId: Boolean): String = when(dbType) {
      DoorDbType.SQLITE -> {
        "INSERT INTO StatementEntity (uidHi, uidLo, lct, name) VALUES(?, ?, ?, ?)"
      }
      DoorDbType.POSTGRES ->  {
        "INSERT INTO StatementEntity (uidHi, uidLo, lct, name) VALUES(?, ?, ?, ?)" + if(returnsId) { " RETURNING uidHi" } else "" 
      }
      else -> {
        throw IllegalArgumentException("Unsupported db type")
      }
    }

    override fun bindPreparedStmtToEntity(stmt: PreparedStatement, entity: StatementEntity) {
      stmt.setLong(1, entity.uidHi)
      stmt.setLong(2, entity.uidLo)
      stmt.setLong(3, entity.lct)
      stmt.setString(4, entity.name)
    }
  }

  override suspend fun insertAsync(entity: StatementEntity) {
    _insertAdapterStatementEntity_abort.insertAsync(entity)
  }

  override suspend fun findAllAsync(): List<StatementEntity> =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT StatementEntity.*
    |          FROM StatementEntity
    |    
    """.trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapRows {
        val _tmp_uidHi = _result.getLong("uidHi")
        val _tmp_uidLo = _result.getLong("uidLo")
        val _tmp_lct = _result.getLong("lct")
        val _tmp_name = _result.getString("name")
        StatementEntity().apply {
          this.uidHi = _tmp_uidHi
          this.uidLo = _tmp_uidLo
          this.lct = _tmp_lct
          this.name = _tmp_name
        }
      }
    }
  }

  override fun findByUidAsFlow(uidHi: Long, uidLo: Long): Flow<StatementEntity?> =
      _db.doorFlow(arrayOf("StatementEntity")) {
    _db.prepareAndUseStatementAsync(PreparedStatementConfig(
      sql = """
      |
      |        SELECT StatementEntity.*
      |          FROM StatementEntity
      |         WHERE StatementEntity.uidHi = ?
      |           AND StatementEntity.uidLo = ?
      |    
      """.trimMargin(),
      readOnly = true,)
    ) { _stmt -> 
      _stmt.setLong(1,uidHi)
      _stmt.setLong(2,uidLo)
      _stmt.executeQueryAsyncKmp().useResults{ _result -> 
        _result.mapNextRow(null) {
          val _tmp_uidHi = _result.getLong("uidHi")
          val _tmp_uidLo = _result.getLong("uidLo")
          val _tmp_lct = _result.getLong("lct")
          val _tmp_name = _result.getString("name")
          StatementEntity().apply {
            this.uidHi = _tmp_uidHi
            this.uidLo = _tmp_uidLo
            this.lct = _tmp_lct
            this.name = _tmp_name
          }
        }
      }
    }
  }

  override suspend fun findByUidAsync(uidHi: Long, uidLo: Long): StatementEntity? =
      _db.prepareAndUseStatementAsync(PreparedStatementConfig(
    sql = """
    |
    |        SELECT StatementEntity.*
    |          FROM StatementEntity
    |         WHERE StatementEntity.uidHi = ?
    |           AND StatementEntity.uidLo = ?
    |    
    """.trimMargin(),
    readOnly = true,)
  ) { _stmt -> 
    _stmt.setLong(1,uidHi)
    _stmt.setLong(2,uidLo)
    _stmt.executeQueryAsyncKmp().useResults{ _result -> 
      _result.mapNextRow(null) {
        val _tmp_uidHi = _result.getLong("uidHi")
        val _tmp_uidLo = _result.getLong("uidLo")
        val _tmp_lct = _result.getLong("lct")
        val _tmp_name = _result.getString("name")
        StatementEntity().apply {
          this.uidHi = _tmp_uidHi
          this.uidLo = _tmp_uidLo
          this.lct = _tmp_lct
          this.name = _tmp_name
        }
      }
    }
  }
}
